@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: filter1d.c,v 1.32 2006/04/13 06:20:34 pwessel Exp $
+ *    $Id: filter1d.c,v 1.67 2010/05/10 23:12:31 remko Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -39,41 +39,41 @@
 
 struct FILTER1D_CTRL {
 	struct D {	/* -D<inc> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double inc;
 	} D;
 	struct E {	/* -E */
-		BOOLEAN active;
+		GMT_LONG active;
 	} E;
 	struct F {	/* -F<type><width>[<mode>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		char filter;	/* Character codes for the filter */
 		double width;
-		int mode;
+		GMT_LONG mode;
 		char *file;	/* Character codes for the filter */
 	} F;
 	struct I {	/* -I<ignoreval> */
-		BOOLEAN active;	/* TRUE when input values that equal value should be discarded */
+		GMT_LONG active;	/* TRUE when input values that equal value should be discarded */
 		double value;
 	} I;
 	struct L {	/* -L<lackwidth> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} L;
 	struct N {	/* -N<n_cols>/<t_col> */
-		BOOLEAN active;
-		int col, ncols;
+		GMT_LONG active;
+		GMT_LONG col, ncols;
 	} N;
 	struct Q {	/* -Q<factor> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} Q;
 	struct S {	/* -S<symmetry> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} S;
 	struct T {	/* -T[<tmin/tmax/t_inc>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double min, max, inc;
 	} T;
 };
@@ -91,30 +91,30 @@ struct FILTER1D_CTRL {
 #define FILTER1D_CONVOLVE 2		/* If filter_type > FILTER1D_CONVOLVE then a FILTER1D_MEDIAN, FILTER1D_MODE, or EXTREME filter is selected  */
 
 struct FILTER1D_INFO {	/* Control structure for all aspects of the filter setup */
-	BOOLEAN	use_ends;	/* True to start/stop at ends of series instead of 1/2 width inside  */
-	BOOLEAN check_asym;	/* TRUE to test whether the data are asymmetric about the output time  */
-	BOOLEAN check_lack;	/* TRUE to test for lack of data (gap) in the filter window */
-	BOOLEAN check_q;	/* TRUE to test average weight or N in median */
-	BOOLEAN robust;		/* Look for outliers in data when TRUE */
-	BOOLEAN equidist;	/* Data is evenly sampled in t */
-	BOOLEAN out_at_time;	/* TRUE when output is required at evenly spaced intervals */
-	BOOLEAN f_operator;	/* TRUE if custom weights coefficients sum to zero */
+	GMT_LONG	use_ends;	/* True to start/stop at ends of series instead of 1/2 width inside  */
+	GMT_LONG check_asym;	/* TRUE to test whether the data are asymmetric about the output time  */
+	GMT_LONG check_lack;	/* TRUE to test for lack of data (gap) in the filter window */
+	GMT_LONG check_q;	/* TRUE to test average weight or N in median */
+	GMT_LONG robust;		/* Look for outliers in data when TRUE */
+	GMT_LONG equidist;	/* Data is evenly sampled in t */
+	GMT_LONG out_at_time;	/* TRUE when output is required at evenly spaced intervals */
+	GMT_LONG f_operator;	/* TRUE if custom weights coefficients sum to zero */
 
-	int	*n_this_col;		/* Pointer to array of counters [one per column]  */
-	int	*n_left;		/* Pointer to array of counters [one per column]  */
-	int	*n_right;		/* Pointer to array of counters [one per column]  */
-	int	n_rows;			/* Number of rows of input  */
-	int	n_cols;			/* Number of columns of input  */
-	int	t_col;			/* Column of time abscissae (independent variable)  */
-	int	n_f_wts;		/* Number of filter weights  */
-	int	half_n_f_wts;		/* Half the number of filter weights  */
-	int	n_row_alloc;		/* Number of rows of data to allocate  */
-	int	n_work_alloc;		/* Number of rows of workspace to allocate  */
-	int	filter_type;		/* Flag indicating desired filter type  */
-	int	kind;				/* -1 skip +ve, +1 skip -ve, else use all  [for the l|L|u|U filter] */
-	int	way;			/* -1 find minimum, +1 find maximum  [for the l|L|u|U filter] */
-	int	mode_selection;
-	int	n_multiples;
+	GMT_LONG	*n_this_col;		/* Pointer to array of counters [one per column]  */
+	GMT_LONG	*n_left;		/* Pointer to array of counters [one per column]  */
+	GMT_LONG	*n_right;		/* Pointer to array of counters [one per column]  */
+	GMT_LONG	n_rows;			/* Number of rows of input  */
+	GMT_LONG	n_cols;			/* Number of columns of input  */
+	GMT_LONG	t_col;			/* Column of time abscissae (independent variable)  */
+	GMT_LONG	n_f_wts;		/* Number of filter weights  */
+	GMT_LONG	half_n_f_wts;		/* Half the number of filter weights  */
+	GMT_LONG	n_row_alloc;		/* Number of rows of data to allocate  */
+	GMT_LONG	n_work_alloc;		/* Number of rows of workspace to allocate  */
+	GMT_LONG	filter_type;		/* Flag indicating desired filter type  */
+	GMT_LONG	kind;			/* -1 skip +ve, +1 skip -ve, else use all  [for the l|L|u|U filter] */
+	GMT_LONG	way;			/* -1 find minimum, +1 find maximum  [for the l|L|u|U filter] */
+	GMT_LONG	mode_selection;
+	GMT_LONG	n_multiples;
 	
 	double	*f_wt;			/* Pointer for array of filter coefficients  */
 	double	*min_loc;		/* Pointer for array of values, one per [column]  */
@@ -145,12 +145,12 @@ void allocate_more_work_space (struct FILTER1D_INFO *F);	/* Does just what it sa
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE;
+	GMT_LONG error = FALSE;
 	
-	char	c;
+	char	c, txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], *not_used = NULL;
 	char	buffer[BUFSIZ];		/* Used to scan input lines  */
 	
-	int	i, n_fields, n_expected_fields;
+	GMT_LONG	i, n_fields, n_expected_fields, n_read;
 	
 	double	last_time, new_time, *in;
 	
@@ -159,18 +159,18 @@ int main (int argc, char **argv)
 	struct FILTER1D_INFO F;
 	struct FILTER1D_CTRL *Ctrl;
 
-	int	load_data_and_check_extrema (struct FILTER1D_INFO *F);	/* Does just what it says  */
-	int	set_up_filter (struct FILTER1D_INFO *F);		/* Creates filter weights or reads them from file and sets start & stop  */
-	int	do_the_filter (struct FILTER1D_INFO *F);		/* Does the job  */
-	int	allocate_space (struct FILTER1D_INFO *F);		/* Does just what it says  */
+	GMT_LONG	load_data_and_check_extrema (struct FILTER1D_INFO *F);	/* Does just what it says  */
+	GMT_LONG	set_up_filter (struct FILTER1D_INFO *F);		/* Creates filter weights or reads them from file and sets start & stop  */
+	GMT_LONG	do_the_filter (struct FILTER1D_INFO *F);		/* Does the job  */
+	GMT_LONG	allocate_space (struct FILTER1D_INFO *F);		/* Does just what it says  */
 	void	free_space (struct FILTER1D_INFO *F);			/* Does just what it says  */
 	void	allocate_more_data_space (struct FILTER1D_INFO *F);	/* Does just what it says  */
 	void load_parameters (struct FILTER1D_INFO *C, struct FILTER1D_CTRL *Ctrl);
-	void *New_Filter1d_Ctrl (), Free_Filter1d_Ctrl (struct FILTER1D_CTRL *C);
+	void *New_filter1d_Ctrl (), Free_filter1d_Ctrl (struct FILTER1D_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct FILTER1D_CTRL *)New_Filter1d_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct FILTER1D_CTRL *)New_filter1d_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	memset ((void *)&F, 0, sizeof (struct FILTER1D_INFO));	/* Init control structure to NULL */
 	F.n_row_alloc = GMT_CHUNK;
@@ -208,6 +208,7 @@ int main (int argc, char **argv)
 								break;
 							case 'F':
 							case 'f':
+								Ctrl->F.width = DBL_MAX;	/* To avoid range test errors before reading coefficients */
 								if (argv[i][3] && !GMT_access (&argv[i][3], R_OK))
 									Ctrl->F.file = strdup (&argv[i][3]);
 								else
@@ -239,7 +240,7 @@ int main (int argc, char **argv)
 					Ctrl->L.value = atof (&argv[i][2]);
 					break;
 				case 'N':
-					if (sscanf (&argv[i][2], "%d/%d", &Ctrl->N.ncols, &Ctrl->N.col) != 2) {
+					if (sscanf (&argv[i][2], "%" GMT_LL "d/%" GMT_LL "d", &Ctrl->N.ncols, &Ctrl->N.col) != 2) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -N option:  Syntax is -N<ncol>/<tcol>\n", GMT_program);
 						error++;
 					}
@@ -255,14 +256,18 @@ int main (int argc, char **argv)
 					Ctrl->S.value = atof (&argv[i][2]);
 					break;
 				case 'T':
-					if (sscanf (&argv[i][2], "%lf/%lf/%lf", &Ctrl->T.min, &Ctrl->T.max, &Ctrl->T.inc) != 3) {
+					if (sscanf (&argv[i][2], "%[^/]/%[^/]/%lf", txt_a, txt_b, &Ctrl->T.inc) != 3) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Syntax is -T<start>/<stop>/<inc>\n", GMT_program);
+						error++;
 					}
-					else
+					else {
+						GMT_scanf_arg (txt_a, GMT_IS_UNKNOWN, &Ctrl->T.min);
+						GMT_scanf_arg (txt_b, GMT_IS_UNKNOWN, &Ctrl->T.max);
 						Ctrl->T.active = TRUE;
+					}
 					break;
 				default:
-					error = TRUE;
+					error++;
                                         GMT_default_error (argv[i][1]);
 					break;
 			}
@@ -278,28 +283,28 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "filter1d %s - Time domain filtering of 1-D time series\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: filter1d [infile] -F<type><width>[<mode>] [-D<increment>] [-E] [%s]\n", GMT_H_OPT);
-		fprintf (stderr, "\t[-I<ignore_val>] [-L<lack_width>] [-N<n_cols>/<t_col>] -Q<q_factor>] [-S<symmetry>]\n");
+		fprintf (stderr, "\t[-I<ignore_val>] [-L<lack_width>] [-N<n_cols>/<t_col>] [-Q<q_factor>] [-S<symmetry>]\n");
 		fprintf (stderr, "\t[-T<start>/<stop>/<int>] [-V] [%s] [%s]\n\n", GMT_b_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t-F sets Filtertype.  Choose from convolution and non-convolution filters.\n");
-		fprintf (stderr, "\t   and append full filter <width> in same units as time column,\n");
+		fprintf (stderr, "\t-F sets Filtertype.  Choose from convolution and non-convolution filters\n");
+		fprintf (stderr, "\t   and append full filter <width> in same units as time column.\n");
 		fprintf (stderr, "\t   Convolution filters:\n");
 		fprintf (stderr, "\t     b: Boxcar : Weights are equal.\n");
 		fprintf (stderr, "\t     c: Cosine arch : Weights given by cosine arch.\n");
-		fprintf (stderr, "\t     g: Gaussian : Weights given gy Gaussian function.\n");
+		fprintf (stderr, "\t     g: Gaussian : Weights given by Gaussian function.\n");
 		fprintf (stderr, "\t     f<name>: Custom : Weights given in one-column file <name>.\n");
 		fprintf (stderr, "\t   Non-convolution filters:\n");
 		fprintf (stderr, "\t     m: Median : Return the median value.\n");
 		fprintf (stderr, "\t     p: Maximum likelihood probability (mode) estimator : Return the mode.\n");
-		fprintf (stderr, "\t        By default, we return the average if more than one mode is found.\n");
-		fprintf (stderr, "\t        Append - or + to the width to instead return the smallest or largest mode.\n");
+		fprintf (stderr, "\t        By default, we return the average mode if more than one is found.\n");
+		fprintf (stderr, "\t        Append - or + to the width to return the smallest or largest mode instead.\n");
 		fprintf (stderr, "\t     l: Lower : Return minimum of all points.\n");
-		fprintf (stderr, "\t     L: Lower+ : Return minimum of all +ve points.\n");
+		fprintf (stderr, "\t     L: Lower+ : Return minimum of all positive points.\n");
 		fprintf (stderr, "\t     u: Upper : Return maximum of all points.\n");
-		fprintf (stderr, "\t     U: Upper- : Return maximum of all -ve points.\n");
-		fprintf (stderr, "\t   Upper case type B, C, G, M, P, F will use robust filter versions:\n");
+		fprintf (stderr, "\t     U: Upper- : Return maximum of all negative points.\n");
+		fprintf (stderr, "\t   Upper case type B, C, G, M, P, F will use robust filter versions,\n");
 		fprintf (stderr, "\t   i.e., replace outliers (2.5 L1 scale off median) with median during filtering.\n");
 		
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -367,7 +372,7 @@ int main (int argc, char **argv)
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < F.n_cols) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d fields\n", GMT_program, F.n_cols);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %ld fields\n", GMT_program, F.n_cols);
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && !Ctrl->N.active && GMT_io.ncol[GMT_IN] == 0) {
@@ -377,11 +382,9 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	load_parameters (&F, Ctrl);	/* Pass parameters from Control structure to Filter structure */
@@ -435,7 +438,7 @@ int main (int argc, char **argv)
 			break;
 	}
 
-	GMT_io.skip_if_NaN[0] = GMT_io.skip_if_NaN[1] = FALSE;	/* Turn off default GMT NaN-handling */
+	GMT_io.skip_if_NaN[GMT_X] = GMT_io.skip_if_NaN[GMT_Y] = FALSE;	/* Turn off default GMT NaN-handling */
 	GMT_io.skip_if_NaN[F.t_col] = TRUE;			/* ... But disallow NaN in "time" column */
 
 #ifdef SET_IO_MODE
@@ -458,12 +461,12 @@ int main (int argc, char **argv)
 
 	if (GMT_io.io_header[GMT_IN]) {
 		for (i = 0; i < GMT_io.n_header_recs; i++) {
-			GMT_fgets (buffer, BUFSIZ, fp_in);
-			if (GMT_io.io_header[GMT_OUT]) fputs (buffer, GMT_stdout);
+			not_used = GMT_fgets (buffer, BUFSIZ, fp_in);
+			if (GMT_io.io_header[GMT_OUT]) GMT_fputs (buffer, GMT_stdout);
 		}
 	}
 
-	F.n_rows = 0;
+	F.n_rows = n_read = 0;
 	if (F.robust || (F.filter_type == FILTER1D_MEDIAN) ) {
 		for (i = 0; i < F.n_cols; i++) {
 			F.min_loc[i] = DBL_MAX;
@@ -479,9 +482,11 @@ int main (int argc, char **argv)
 
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+		n_read++;
+
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, F.n_rows);
-			exit (EXIT_FAILURE);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields,  n_expected_fields, n_read);
+			continue;
 		}
 
 		for (i = 0; i < F.n_cols; i++) {
@@ -501,15 +506,15 @@ int main (int argc, char **argv)
 			new_time = F.data[F.t_col][F.n_rows];
 			F.n_rows++;
 			if (new_time < last_time) {
-				fprintf (stderr, "%s: Error! Time decreases at line # %d\n", GMT_program, F.n_rows);
+				fprintf (stderr, "%s: Error! Time decreases at line # %ld\n", GMT_program, F.n_rows);
 				fprintf (stderr, "\tUse UNIX utility sort and then try again.\n");
-				return(-1);
+				exit (EXIT_FAILURE);
 			}
 			last_time = new_time;
 		}
 
 		if (F.n_rows == F.n_row_alloc) {	/* Need more memory */
-			F.n_row_alloc += GMT_CHUNK;
+			F.n_row_alloc <<= 1;
 			allocate_more_data_space (&F);
 		}
 	}
@@ -517,7 +522,7 @@ int main (int argc, char **argv)
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
 
 	for (i = 0; i < F.n_cols; i++) F.data[i] = (double *) GMT_memory ((void *)F.data[i], (size_t)F.n_rows, sizeof (double), GMT_program);
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d data lines\n", GMT_program, F.n_rows);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld data lines\n", GMT_program, F.n_rows);
 
 	/* Initialize scale parameters and last_loc based on min and max of data  */
 
@@ -544,20 +549,21 @@ int main (int argc, char **argv)
 
 	free_space (&F);
 
-	if (F.n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, F.n_multiples);
+	if (F.n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %ld multiple modes found\n", GMT_program, F.n_multiples);
 
-	Free_Filter1d_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_filter1d_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-int set_up_filter (struct FILTER1D_INFO *F)
+GMT_LONG set_up_filter (struct FILTER1D_INFO *F)
 {
-	int	i, i1, i2;
+	GMT_LONG	i, i1, i2;
 	double	t_0, t_1, time, w_sum;
-	BOOLEAN	normalize = FALSE;
+	char buffer[BUFSIZ];
+	GMT_LONG	normalize = FALSE;
 	PFD get_weight[3];		/* Selects desired weight function.  */
 
 	double	boxcar_weight (double radius, double half_width);	/* Returns the weight for a given delta_time  */
@@ -572,11 +578,17 @@ int set_up_filter (struct FILTER1D_INFO *F)
 		F->f_wt = (double *) GMT_memory (VNULL, (size_t)F->n_work_alloc, sizeof(double), GMT_program);
 		F->n_f_wts = 0;
 		w_sum = 0.0;
-		while (fscanf (F->fp_wt,"%lf\n", &F->f_wt[F->n_f_wts]) != EOF) {
+		while (GMT_fgets (buffer, BUFSIZ, F->fp_wt)) {
+			if (GMT_is_a_blank_line (buffer)) continue;	/* Skip blank lines or # comments */
+			GMT_chop (buffer);
+			if (sscanf (buffer, "%lf", &F->f_wt[F->n_f_wts]) != 1) {
+				fprintf (stderr, "%s: Error decoding filter weights from file\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
 			w_sum += F->f_wt[F->n_f_wts];
 			F->n_f_wts++;
 			if (F->n_f_wts == F->n_work_alloc) {	/* Need more memory */
-				F->n_work_alloc += GMT_CHUNK;
+				F->n_work_alloc <<= 1;
 				allocate_more_work_space (F);
 			}
 		}
@@ -586,14 +598,14 @@ int set_up_filter (struct FILTER1D_INFO *F)
 		F->half_width = F->half_n_f_wts * F->dt;
 		F->filter_width = 2.0 * F->half_width;
 		F->f_wt = (double *) GMT_memory ((void *)F->f_wt, (size_t)F->n_f_wts, sizeof (double), GMT_program);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d filter weights from file.\n", GMT_program, F->n_f_wts);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld filter weights from file.\n", GMT_program, F->n_f_wts);
 	}
 	else if (F->filter_type <= FILTER1D_CONVOLVE) {
 		get_weight[FILTER1D_BOXCAR] = (PFD)boxcar_weight;
 		get_weight[FILTER1D_COS_ARCH] = (PFD)cosine_weight;
 		get_weight[FILTER1D_GAUSSIAN] = (PFD)gaussian_weight;
 		F->half_width = 0.5 * F->filter_width;
-		F->half_n_f_wts = (int)floor (F->half_width / F->dt);
+		F->half_n_f_wts = (GMT_LONG)floor (F->half_width / F->dt);
 		F->n_f_wts = 2 * F->half_n_f_wts + 1;
 
 		F->f_wt = (double *) GMT_memory (VNULL, (size_t)F->n_f_wts, sizeof(double), GMT_program);
@@ -644,20 +656,21 @@ int set_up_filter (struct FILTER1D_INFO *F)
 	return(0);
 }
 
-int do_the_filter (struct FILTER1D_INFO *F)
+GMT_LONG do_the_filter (struct FILTER1D_INFO *F)
 {
-	int	i_row, i_col, left, right, n_l, n_r, iq, i_f_wt;
-	int	i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
+	GMT_LONG	i_row, left, right, n_l, n_r, iq, i_f_wt;
+	GMT_LONG	i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
+	GMT_LONG i_col;
 	double	time, delta_time, *outval, wt, val, med, scl, small;
-	BOOLEAN	*good_one;		/* Pointer to array of logicals [one per column]  */
+	GMT_LONG	*good_one;		/* Pointer to array of logicals [one per column]  */
 	double	*wt_sum;		/* Pointer for array of weight sums [each column]  */
 	double	*data_sum;		/* Pointer for array of data * weight sums [columns]  */
 
-	BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, int left, int right);	/* Tests for lack of data condition  (optional)  */
-	void	get_robust_estimates (struct FILTER1D_INFO *F, int j, int n, int both);	/* Does just what it says  */
+	GMT_LONG	lack_check (struct FILTER1D_INFO *F, GMT_LONG i_col, GMT_LONG left, GMT_LONG right);	/* Tests for lack of data condition  (optional)  */
+	void	get_robust_estimates (struct FILTER1D_INFO *F, GMT_LONG j, GMT_LONG n, GMT_LONG both);	/* Does just what it says  */
 
 	outval = (double *)GMT_memory (VNULL, (size_t)F->n_cols, sizeof (double), GMT_program);
-	good_one = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
+	good_one = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
 	wt_sum = (double *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(double), GMT_program);
 	data_sum = (double *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(double), GMT_program);
 
@@ -730,7 +743,7 @@ int do_the_filter (struct FILTER1D_INFO *F)
 					if (!(good_one[i_col])) continue;
 					n_l = F->n_left[i_col];
 					n_r = F->n_right[i_col];
-					if ((((double)abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
+					if ((((double)GMT_abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
 				}
 			}
 			if ( (F->filter_type > FILTER1D_CONVOLVE) && F->check_q) {
@@ -772,7 +785,7 @@ int do_the_filter (struct FILTER1D_INFO *F)
 
 			for (i_row = left; i_row < right; i_row++) {
 				delta_time = time - F->data[F->t_col][i_row];
-				i_f_wt = F->half_n_f_wts + (int)floor(0.5 + delta_time/F->dt);
+				i_f_wt = F->half_n_f_wts + (GMT_LONG)floor(0.5 + delta_time/F->dt);
 				if ( (i_f_wt < 0) || (i_f_wt >= F->n_f_wts) ) continue;
 
 				for(i_col = 0; i_col < F->n_cols; i_col++) {
@@ -805,7 +818,7 @@ int do_the_filter (struct FILTER1D_INFO *F)
 				if (F->check_asym && !(F->robust) ) {
 					n_l = F->n_left[i_col];
 					n_r = F->n_right[i_col];
-					if ((((double)abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
+					if ((((double)GMT_abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
 						good_one[i_col] = FALSE;
 						continue;
 					}
@@ -847,17 +860,17 @@ int do_the_filter (struct FILTER1D_INFO *F)
 	return (0);
 }
 
-int allocate_space (struct FILTER1D_INFO *F)
+GMT_LONG allocate_space (struct FILTER1D_INFO *F)
 {
-	int	i;
+	GMT_LONG	i;
 
 	F->data = (double **) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(double *), GMT_program);
 	for (i = 0; i < F->n_cols; i++) F->data[i] = (double *) GMT_memory (VNULL, (size_t)F->n_row_alloc, sizeof(double), GMT_program);
 
-	F->n_this_col = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
+	F->n_this_col = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
 
-	if (F->check_asym) F->n_left = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
-	if (F->check_asym) F->n_right = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
+	if (F->check_asym) F->n_left = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
+	if (F->check_asym) F->n_right = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
 
 	if (F->robust || (F->filter_type > FILTER1D_CONVOLVE) ) {	/* Then we need workspace  */
 
@@ -877,14 +890,14 @@ int allocate_space (struct FILTER1D_INFO *F)
 
 void allocate_more_data_space (struct FILTER1D_INFO *F)
 {
-	int	i;
+	GMT_LONG	i;
 
 	for (i = 0; i < F->n_cols; i++) F->data[i] = (double *) GMT_memory ((void *)F->data[i], (size_t)F->n_row_alloc, sizeof(double), GMT_program);
 }
 
 void allocate_more_work_space (struct FILTER1D_INFO *F)
 {
-	int	i;
+	GMT_LONG	i;
 
 	for (i = 0; i < F->n_cols; i++) F->work[i] = (double *) GMT_memory ((void *)F->work[i], (size_t)F->n_work_alloc, sizeof(double), GMT_program);
 }
@@ -892,25 +905,31 @@ void allocate_more_work_space (struct FILTER1D_INFO *F)
 
 void free_space (struct FILTER1D_INFO *F)
 {
+	GMT_LONG i;
+	if (F->robust || (F->filter_type > FILTER1D_CONVOLVE) ) {	
+		for (i = 0; i < F->n_cols; i++)	GMT_free ((void *)F->work[i]);
+		GMT_free ((void *)F->work);
+	}
+	for (i = 0; i < F->n_cols; i++)	GMT_free ((void *)F->data[i]);
 	GMT_free ((void *)F->data);
 	GMT_free ((void *)F->n_this_col);
-	GMT_free ((void *)F->n_left);
-	GMT_free ((void *)F->n_right);
-	GMT_free ((void *)F->work);
-	GMT_free ((void *)F->min_loc);
-	GMT_free ((void *)F->max_loc);
-	GMT_free ((void *)F->last_loc);
-	GMT_free ((void *)F->this_loc);
-	GMT_free ((void *)F->min_scl);
-	GMT_free ((void *)F->max_scl);
-	GMT_free ((void *)F->last_scl);
-	GMT_free ((void *)F->this_scl);
+	if (F->check_asym) GMT_free ((void *)F->n_left);
+	if (F->check_asym) GMT_free ((void *)F->n_right);
+	if (F->min_loc) GMT_free ((void *)F->min_loc);
+	if (F->max_loc) GMT_free ((void *)F->max_loc);
+	if (F->last_loc) GMT_free ((void *)F->last_loc);
+	if (F->this_loc) GMT_free ((void *)F->this_loc);
+	if (F->min_scl) GMT_free ((void *)F->min_scl);
+	if (F->max_scl) GMT_free ((void *)F->max_scl);
+	if (F->last_scl) GMT_free ((void *)F->last_scl);
+	if (F->this_scl) GMT_free ((void *)F->this_scl);
+	if (F->n_f_wts) GMT_free ((void *)F->f_wt);
 }
 
-BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, int left, int right)
+GMT_LONG	lack_check (struct FILTER1D_INFO *F, GMT_LONG i_col, GMT_LONG left, GMT_LONG right)
 {
-	int	last_row, this_row;
-	BOOLEAN	lacking = FALSE;
+	GMT_LONG	last_row, this_row;
+	GMT_LONG	lacking = FALSE;
 	double	last_t;
 
 	last_row = left;
@@ -932,28 +951,28 @@ BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, int left, int right)
 	return (lacking);
 }
 
-void get_robust_estimates (struct FILTER1D_INFO *F, int j, int n, int both)
+void get_robust_estimates (struct FILTER1D_INFO *F, GMT_LONG j, GMT_LONG n, GMT_LONG both)
 {
-	int i, n_smooth, sort_me = TRUE;
+	GMT_LONG i, n_smooth;
+	GMT_LONG sort_me = TRUE;
 	double	low, high, last, temp;
 
-	if (F->filter_type == FILTER1D_MODE) {
+	if (F->filter_type > FILTER1D_MODE) {
+		temp = GMT_extreme (F->work[j], n, F->extreme, F->kind, F->way);
+	}
+	else if (F->filter_type == FILTER1D_MODE) {
 
 		n_smooth = n / 2;
 
 		GMT_mode (F->work[j], n, n_smooth, sort_me, F->mode_selection, &F->n_multiples, &temp);
 	}
-	else if (F->filter_type == FILTER1D_MEDIAN) {
-
+	else {
 		low = F->min_loc[j];
 		high = F->max_loc[j];
 		last = F->last_loc[j];
 
 		GMT_median (F->work[j], n, low, high, last, &temp);
 	}
-	else {	/* EXTREME filter **/
-		temp = GMT_extreme (F->work[j], n, F->extreme, F->kind, F->way);
-	}
 
 	F->last_loc[j] = F->this_loc[j] = temp;
 
@@ -1024,10 +1043,10 @@ void load_parameters (struct FILTER1D_INFO *F, struct FILTER1D_CTRL *Ctrl)
 	F->out_at_time = Ctrl->T.active;
 }
 
-void *New_Filter1d_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_filter1d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct FILTER1D_CTRL *C;
 
-	C = (struct FILTER1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct FILTER1D_CTRL), "New_Filter1d_Ctrl");
+	C = (struct FILTER1D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct FILTER1D_CTRL), "New_filter1d_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.ncols = 2;						/* Expected # of columns */
@@ -1035,7 +1054,7 @@ void *New_Filter1d_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void Free_Filter1d_Ctrl (struct FILTER1D_CTRL *C) {	/* Deallocate control structure */
-	if (C->F.file) GMT_free ((void *)C->F.file);	
+void Free_filter1d_Ctrl (struct FILTER1D_CTRL *C) {	/* Deallocate control structure */
+	if (C->F.file) free ((void *)C->F.file);	
 	GMT_free ((void *)C);	
 }
