@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2rgb.c,v 1.35 2006/04/13 06:20:34 pwessel Exp $
+ *	$Id: gmt2rgb.c,v 1.63 2010/03/22 23:37:05 jluis Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *--------------------------------------------------------------------*/
 /*
  * gmt2rgb reads either (1) an 8, 24, or 32 bit Sun rasterfile and writes out the
- * red, green, and blue components in three separate grd files, or (2) a z grd
+ * red, green, and blue components in three separate grid files, or (2) a z grd
  * file and a cpt file and compute r, g, b and write these out instead.
  *
  * Author:	Paul Wessel
@@ -31,35 +31,36 @@
 
 struct GMT2RGB_CTRL {
 	struct C {	/* -C<cptfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} C;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<nametemplate> */
-		BOOLEAN active;
-		char *template;
+		GMT_LONG active;
+		char *name;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct L {	/* -L<layer> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char layer;
 	} L;
 	struct W {	/* -W<width/height>[/<n_bytes>] */
-		BOOLEAN active;
-		int nx, ny;	/* Dimension of image */
-		int size;	/* Number of bytes per pixels */
+		GMT_LONG active;
+		GMT_LONG nx, ny;	/* Dimension of image */
+		GMT_LONG size;	/* Number of bytes per pixels */
 	} W;
 };
 
 int main (int argc, char **argv)
 {
-	int i, k, k3, nm, error = 0, index, irgb[3], entry, one_or_zero, pos;
-	BOOLEAN guess = FALSE;
+	GMT_LONG i, error = 0, index, entry, one_or_zero, pos, nm, k, k3;
+	int irgb[3];
+	GMT_LONG guess = FALSE;
 	float *z;
 	double w, e, s, n;
 	struct GRD_HEADER grd;
@@ -69,13 +70,13 @@ int main (int argc, char **argv)
 	unsigned char *picture;
 	struct GMT2RGB_CTRL *Ctrl;
 	
-	void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny);
-	unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel, int nx, int ny);
-	void *New_Gmt2rgb_Ctrl (), Free_Gmt2rgb_Ctrl (struct GMT2RGB_CTRL *C);
+	void guess_width (char *file, GMT_LONG byte_per_pixel, GMT_LONG *raw_nx, GMT_LONG *raw_ny);
+	unsigned char *loadraw (char *file, struct imageinfo *header, GMT_LONG byte_per_pixel, GMT_LONG nx, GMT_LONG ny);
+	void *New_gmt2rgb_Ctrl (), Free_gmt2rgb_Ctrl (struct GMT2RGB_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GMT2RGB_CTRL *)New_Gmt2rgb_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GMT2RGB_CTRL *)New_gmt2rgb_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	/* Check and interpret the command line arguments */
 
@@ -100,8 +101,7 @@ int main (int argc, char **argv)
 					Ctrl->F.active = TRUE;
 					break;
 				case 'G':
-					GMT_free ((void *)Ctrl->G.template);
-					Ctrl->G.template = strdup (&argv[i][2]);
+					Ctrl->G.name = strdup (&argv[i][2]);
 					break;
 				case 'I':
 					GMT_getinc (&argv[i][2], &Ctrl->I.xinc, &Ctrl->I.yinc);
@@ -149,7 +149,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"%s %s - Write r/g/b gridfiles from a gridfile, a raw RGB file, or SUN rasterfile\n\n", GMT_program, GMT_VERSION);
+		fprintf (stderr,"%s %s - Write r/g/b grid files from a grid file, a raw RGB file, or SUN rasterfile\n\n", GMT_program, GMT_VERSION);
 		fprintf (stderr,"usage: %s <rasterfile|rgbfile||grdfile> [-C<cptfile>] [-F] [-G<nametemplate>] [%s]\n", GMT_program, GMT_Id_OPT);
 		fprintf (stderr,"\t[-L<layer>] [%s] [-V] [-W<width/height>[/<n_bytes>]]\n\n", GMT_Rgeo_OPT);
 
@@ -158,17 +158,17 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t<infile> can be one of three different intput files:\n");
 		fprintf (stderr,"\t  (1) An 8, 24, or 32-bit Sun rasterfile.  Use -I, -R, and -F to change the\n");
 		fprintf (stderr,"\t      the default values of dx = dy = 1 and region 1/ncols/1/nrows.\n");
-		fprintf (stderr,"\t  (2) A regular z grdfile.  Use -C to provide a cpt file with which\n");
+		fprintf (stderr,"\t  (2) A regular z grid file.  Use -C to provide a cpt file with which\n");
 		fprintf (stderr,"\t      to convert z to r/g/b triplets. -R, -I, and -F are ignored.\n");
-		fprintf (stderr,"\t  (3) A RGB or RGBA raw rasterfile. Since raw rastefiles have no header, you have to\n");
+		fprintf (stderr,"\t  (3) A RGB or RGBA raw rasterfile. Since raw rasterfiles have no header, you have to\n");
 		fprintf (stderr,"\t      give the image dimensions via -W\n");
 		fprintf (stderr,"\t      However, you may take the chance of letting the program try to\n");
 		fprintf (stderr,"\t      guess the image dimensions (slow, via FFT spectrum).\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
-		fprintf (stderr,"\t-C color palette file to convert z to rgb.  If given, we assume a z grdfile is provided,\n");
+		fprintf (stderr,"\t-C color palette file to convert z to rgb.  If given, we assume a z grid file is provided,\n");
 		fprintf (stderr,"\t   else we will try to read a Sun rasterfile.\n");
 		fprintf (stderr, "\t-F will force pixel registration [Default is grid registration]\n");
-		fprintf (stderr,"\t-G Give outputfile name template for the three red, green, blue gridfiles.\n");
+		fprintf (stderr,"\t-G Give outputfile name template for the three red, green, blue grid files.\n");
 		fprintf (stderr,"\t   The template MUST contain the format code %%c which will be replaced with r, g, and b.\n");
 		fprintf (stderr,"\t   [Default is gmt2rgb_%%c.grd]\n");
 		fprintf (stderr, "\t-I specifies grid size(s).  Append m (or c) to <dx> and/or <dy> for minutes (or seconds)\n");
@@ -185,6 +185,8 @@ int main (int argc, char **argv)
 
 	/* Check that the options selected are mutually consistent */
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!Ctrl->C.active) {
 		if (!file) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input raster file\n", GMT_program);
@@ -206,17 +208,17 @@ int main (int argc, char **argv)
 			guess_width (file, Ctrl->W.size, &Ctrl->W.nx, &Ctrl->W.ny);
 
 		if (Ctrl->W.active && Ctrl->W.nx <= 0) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Witdth of raw raster file must be a positive integer. Not %d\n", GMT_program, Ctrl->W.nx);
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Witdth of raw raster file must be a positive integer. Not %ld\n", GMT_program, Ctrl->W.nx);
 			error++;
 		}
 		if (Ctrl->W.active && Ctrl->W.ny <= 0) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Height of raw raster file must be a positive integer. Not %d\n", GMT_program, Ctrl->W.ny);
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Height of raw raster file must be a positive integer. Not %ld\n", GMT_program, Ctrl->W.ny);
 			error++;
 		}
 	}
 	else {
 		if (!file) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input z grd file\n", GMT_program);
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input z grid file\n", GMT_program);
 			error++;
 		}
 		if (!Ctrl->C.file) {
@@ -224,7 +226,7 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
-	if (!strstr (Ctrl->G.template, "%c")) {
+	if (!strstr (Ctrl->G.name, "%c")) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  output template must contain %%c\n", GMT_program);
 		error++;
 	}
@@ -234,35 +236,25 @@ int main (int argc, char **argv)
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	GMT_grd_init (&grd, argc, argv, FALSE);
 
 	if (Ctrl->C.active) {
 		GMT_read_cpt (Ctrl->C.file);
-		if (GMT_read_grd_info (file, &grd)) {
-			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, file);
-			exit (EXIT_FAILURE);
-		}
-		nm = grd.nx * grd.ny;
+		GMT_err_fail (GMT_read_grd_info (file, &grd), file);
+		nm = GMT_get_nm (grd.nx, grd.ny);
+		
 		z = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		for (i = 0; i < 3; i++) {	/* Do the r, g, and b channels */
 			if (Ctrl->L.active && Ctrl->L.layer != rgb[i]) continue;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Processing the %s components\n", GMT_program, comp[i]);
-			if (GMT_read_grd (file, &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-				fprintf (stderr, "%s: Error reading file %s\n", GMT_program, file);
-				exit (EXIT_FAILURE);
-			}
-			sprintf (grdfile, Ctrl->G.template, rgb[i]);
+			GMT_err_fail (GMT_read_grd (file, &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), file);
+			sprintf (grdfile, Ctrl->G.name, rgb[i]);
 			sprintf (grd.remark, "Grid of %s components in the 0-255 range", comp[i]);
 			for (k = 0; k < nm; k++) {
-				index = GMT_get_rgb24 (z[k], irgb);
+				index = GMT_get_rgb_from_z (z[k], irgb);
 				z[k] = (float)irgb[i];
 			}
-			if (GMT_write_grd (grdfile, &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-				fprintf (stderr, "%s: Error writing file %s\n", GMT_program, grdfile);
-				exit (EXIT_FAILURE);
-			}
+			GMT_err_fail (GMT_write_grd (grdfile, &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grdfile);
 		}
 	}
 	else {
@@ -299,16 +291,16 @@ int main (int argc, char **argv)
 		}
 		if (w == e && s == n) {	/* R not given, provide default */
 			w = s = 0.0;
-			e = header.width - one_or_zero;
-			n = header.height - one_or_zero;
+			e = (double)(header.width - one_or_zero);
+			n = (double)(header.height - one_or_zero);
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Assign default -R0/%g/0/%g\n", GMT_program, e, n);
 		}
 
-		grd.nx = irint ((e-w)/Ctrl->I.xinc) + one_or_zero;
-		grd.ny = irint ((n-s)/Ctrl->I.yinc) + one_or_zero;
+		grd.nx = (int)(irint ((e-w)/Ctrl->I.xinc) + one_or_zero);
+		grd.ny = (int)(irint ((n-s)/Ctrl->I.yinc) + one_or_zero);
 		if (Ctrl->W.active && !Ctrl->I.active) {		/* This isn't correct because it doesn't deal with -F */
-			grd.nx = Ctrl->W.nx;
-			grd.ny = Ctrl->W.ny;
+			grd.nx = (int)Ctrl->W.nx;
+			grd.ny = (int)Ctrl->W.ny;
 			Ctrl->I.xinc = (e-w)/(Ctrl->W.nx - one_or_zero);
 			Ctrl->I.yinc = (n-s)/(Ctrl->W.ny - one_or_zero);
 		}
@@ -323,9 +315,9 @@ int main (int argc, char **argv)
 		grd.x_min = w;	grd.x_max = e;
 		grd.y_min = s;	grd.y_max = n;
 		grd.x_inc = Ctrl->I.xinc;	grd.y_inc = Ctrl->I.yinc;
-		nm = grd.nx * grd.ny;
+		nm = GMT_get_nm (grd.nx, grd.ny);
 
-		GMT_grd_RI_verify (&grd, 1);
+		GMT_err_fail (GMT_grd_RI_verify (&grd, 1), Ctrl->G.name);
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
 
@@ -334,7 +326,7 @@ int main (int argc, char **argv)
 		for (i = 0; i < 3; i++) {	/* Do the r, g, and b channels */
 			if (Ctrl->L.active && Ctrl->L.layer != rgb[i]) continue;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Processing the %s components\n", GMT_program, comp[i]);
-			sprintf (grdfile, Ctrl->G.template, rgb[i]);
+			sprintf (grdfile, Ctrl->G.name, rgb[i]);
 			sprintf (grd.remark, "Grid of %s components in the 0-255 range", comp[i]);
 			k3 = i;
 			for (k = 0; k < nm; k++) {
@@ -345,28 +337,25 @@ int main (int argc, char **argv)
 					k3 += 3;
 				}
 			}
-			if (GMT_write_grd (grdfile, &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-				fprintf (stderr, "%s: Error writing file %s\n", GMT_program, grdfile);
-				exit (EXIT_FAILURE);
-			}
+			GMT_err_fail (GMT_write_grd (grdfile, &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grdfile);
 		}
 		GMT_free ((void *)picture);
 	}
 	GMT_free ((void *)z);
 
-	Free_Gmt2rgb_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_gmt2rgb_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-
-unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel, int nx, int ny) {
+unsigned char *loadraw (char *file, struct imageinfo *header, GMT_LONG byte_per_pixel, GMT_LONG nx, GMT_LONG ny) {
 	/* loadraw reads a raw binary grb or rgba rasterfile of depth 24, or 32 into memory */
 
-	int j, i, nm;
+	GMT_LONG j, i, nm;
 	unsigned char *buffer;
+
 	FILE *fp;
 
 	if ((fp = GMT_fopen (file, "rb")) == NULL) {
@@ -377,13 +366,13 @@ unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel
 	/* Lets pretend that the raw file is a sunraster file. This way the gmt2rgb code
 	   can be used with very little changes */
 	header->depth = 24;
-	header->width = nx;
-	header->height = ny;
-	nm = nx * ny * byte_per_pixel;
-	header->length = nm;
+	header->width = (int)nx;
+	header->height = (int)ny;
+	nm = ((size_t)nx) * ((size_t)ny) * ((size_t)byte_per_pixel);
+	header->length = (int)nm;
 
 	buffer = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (unsigned char), GMT_program);
-	if (fread ((void *)buffer, (size_t)1, (size_t)nm, fp) != (size_t)nm) {
+	if (GMT_fread ((void *)buffer, (size_t)1, (size_t)nm, fp) != (size_t)nm) {
 		if (byte_per_pixel == 3)
 			fprintf (stderr, "%s: Trouble reading raw 24-bit rasterfile!\n", GMT_program);
 		if (byte_per_pixel == 4)
@@ -403,10 +392,11 @@ unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel
 	return (buffer);
 }
 
-void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
+void guess_width (char *file, GMT_LONG byte_per_pixel, GMT_LONG *raw_nx, GMT_LONG *raw_ny) {
 	unsigned char *buffer;
 	float	*work, *datac, *img_pow, pow_max = -FLT_MAX, pm;
-	int	narray, img_size, k = 0, j, rgb[3], inc, n_pix, i, l, even;
+	GMT_LONG k = 0, j, inc, i, l, even, narray, img_size, n_pix;
+	int rgb[3];
 	FILE *fp;
 
 	if ((fp = GMT_fopen (file, "rb")) == NULL) {
@@ -414,9 +404,9 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 		exit (EXIT_FAILURE);
 	}
 
-	fseek (fp, (long)0, SEEK_END);
-	img_size = ftell (fp);
-	rewind(fp);
+	GMT_fseek (fp, 0L, SEEK_END);
+	img_size = GMT_ftell (fp);
+	GMT_fseek (fp, 0L, SEEK_SET);
 
 	n_pix = img_size / byte_per_pixel;
 
@@ -426,7 +416,7 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 	img_pow = (float *) GMT_memory (VNULL, (size_t)n_pix/2, sizeof (float), GMT_program);
 	memset ((char *)work, 0, (size_t)(2*n_pix * sizeof(float)));
 
-	if (fread ((void *)buffer, (size_t)1, (size_t)img_size, fp) != (size_t)img_size) {
+	if (GMT_fread ((void *)buffer, (size_t)1, (size_t)img_size, fp) != (size_t)img_size) {
 		if (byte_per_pixel == 3)
 			fprintf (stderr, "%s: Trouble_ reading raw 24-bit rasterfile!\n", GMT_program);
 		if (byte_per_pixel == 4)
@@ -475,12 +465,12 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 		fprintf (stderr, "%s WARNING: first test based on FFT failed to guess image dimensions.\n\tI'll do now a second try\t", GMT_program);
 		k = 1;		pm = 1;		l = 1;
 		while (k < 41) {
-			i = *raw_ny + (int)copysign((float)l,pm);
+			i = *raw_ny + (int)irint (copysign((double)l, (double)pm));
 			pm *= -1.;
-			j = (*raw_nx) + (int)copysign((float)l,pm);
+			j = (*raw_nx) + (int)irint (copysign((double)l, (double)pm));
 			if (i*j == n_pix) {	/* Got a good candidate */
 				*raw_ny = i;	*raw_nx = j;
-				fprintf (stderr, "... SUCESS (W = %d, H = %d)\n", *raw_nx, *raw_ny);
+				fprintf (stderr, "... SUCESS (W = %ld, H = %ld)\n", *raw_nx, *raw_ny);
 				break;
 			}
 			even = (k%2 == 0) ? 1: 0;
@@ -489,11 +479,11 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 		}
 	}
 	else
-		if (gmtdefs.verbose) fprintf (stderr, "File %s has %d Lines and %d Cols\n", file, *raw_ny, *raw_nx);
+		if (gmtdefs.verbose) fprintf (stderr, "File %s has %ld Lines and %ld Cols\n", file, *raw_ny, *raw_nx);
 
 	/* If both attempts failed */
 	if ((*raw_nx) * (*raw_ny) != n_pix) {
-		fprintf (stderr, "FAILURE while guessing image dimensions (W = %d, H = %d)\n", *raw_nx, *raw_ny);
+		fprintf (stderr, "FAILURE while guessing image dimensions (W = %ld, H = %ld)\n", *raw_nx, *raw_ny);
 		exit (EXIT_FAILURE);
 	}
 
@@ -504,21 +494,21 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 	GMT_free ((void *)img_pow);
 }
 
-void *New_Gmt2rgb_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_gmt2rgb_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GMT2RGB_CTRL *C;
 	
-	C = (struct GMT2RGB_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMT2RGB_CTRL), "New_Gmt2rgb_Ctrl");
+	C = (struct GMT2RGB_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT2RGB_CTRL), "New_gmt2rgb_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
-	C->G.template = strdup ("gmt2rgb_%c.grd");
+	C->G.name = strdup ("gmt2rgb_%c.grd");
 	C->W.size = 3;	/* 3 bytes per pixel */
 		
 	return ((void *)C);
 }
 
-void Free_Gmt2rgb_Ctrl (struct GMT2RGB_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	if (C->G.template) GMT_free ((void *)C->G.template);	
+void Free_gmt2rgb_Ctrl (struct GMT2RGB_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) free ((void *)C->C.file);	
+	if (C->G.name) free ((void *)C->G.name);	
 	GMT_free ((void *)C);	
 }
