@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtdefaults.c,v 1.29 2009/01/09 04:02:33 guru Exp $
+ *	$Id: gmtdefaults.c,v 1.40 2010/06/22 01:09:40 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,11 +29,11 @@
 
 int main (int argc, char **argv)
 {
-	int i, get = 0;
+	GMT_LONG i, get = 0;
 
-	BOOLEAN error = FALSE, get_sys_defaults = FALSE, get_user_defaults = FALSE;
+	GMT_LONG error = FALSE, get_sys_defaults = FALSE, get_user_defaults = FALSE;
 
-	char *path;
+	char *path = NULL;
 
 	/* SPECIAL INITIALIZATION SINCE BMT_begin IS NOT USED HERE !! */
 #ifdef DEBUG
@@ -44,7 +44,7 @@ int main (int argc, char **argv)
 	GMT_set_home ();
 	GMT_init_fonts (&GMT_N_FONTS);
 
-	GMT_stdout = stdout;	GMT_stdin = stdin;
+	GMT_begin_io ();
 
 	for (i = 1; !error && i < argc; i++) {
 		if (argv[i][0] != '-') continue;
@@ -80,7 +80,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "gmtdefaults %s - List GMT-SYSTEM default parameters to stdout\n\n", GMT_VERSION);
+		fprintf (stderr, "gmtdefaults %s - List GMT default parameters to stdout\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtdefaults [-D[s|u] | -L]\n\n");
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		fprintf (stderr, "\t-D prints the default settings for the GMT system\n");
@@ -103,7 +103,7 @@ int main (int argc, char **argv)
 	else {
 		GMT_getdefpath (get, &path);
 		GMT_getdefaults (path);
-		free (path);
+		free ((void *)path);
 	}
 
 	GMT_putdefaults ("-");
