@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2cpt.c,v 1.36 2006/10/11 02:25:13 remko Exp $
+ *	$Id: grd2cpt.c,v 1.62 2010/03/24 03:19:51 jluis Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -35,68 +35,70 @@
  *		JL: 27-APR-2003, added a -R option
  *		SE: 17-SEP-2003, added a -T option
  * Version:	4
- * 
+ *
  */
 
 #include "gmt.h"
 
 struct GRD2CPT_CTRL {
 	struct C {	/* -C<cpt> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} C;
 	struct D {	/* -D */
-		BOOLEAN active;
+		GMT_LONG active;
 	} D;
 	struct E {	/* -E<nlevels> */
-		BOOLEAN active;
+		GMT_LONG active;
 		int levels;
 	} E;
 	struct I {	/* -I */
-		BOOLEAN active;
+		GMT_LONG active;
 	} I;
 	struct L {	/* -L<min_limit>/<max_limit> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double min, max;
 	} L;
 	struct M {	/* -M */
-		BOOLEAN active;
+		GMT_LONG active;
 	} M;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct Q {	/* -Q[i|o */
-		BOOLEAN active;
-		int mode;
+		GMT_LONG active;
+		GMT_LONG mode;
 	} Q;
 	struct S {	/* -S<z_start>/<z_stop>/<z_inc> */
-		BOOLEAN active;
-		BOOLEAN cpt;
+		GMT_LONG active;
+		GMT_LONG cpt;
 		double low, high, inc;
 		char *file;
 	} S;
 	struct T {	/* -T<kind> */
-		BOOLEAN active;
-		int kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
+		GMT_LONG active;
+		GMT_LONG kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
 	} T;
 	struct Z {	/* -Z */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Z;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, global = FALSE;
+	GMT_LONG error = FALSE, global = FALSE;
 
-	char *grdfile = CNULL, CPT_lis[BUFSIZ], CPT_file[BUFSIZ], format[BUFSIZ], kind;
+#define mgrd 1000
 
-	int i, j, nxy, nx, ny, nfound, ngood;
+	char *grdfile[mgrd], CPT_lis[BUFSIZ], CPT_file[BUFSIZ], format[BUFSIZ], kind;
+
+	GMT_LONG j, ngrd = 0, i, nxy, nxyg, nfound, ngood;
 
 	float	*zdata;
 
 	double *z, mean, sd, w, e, s, n;
 
-	struct GRD_HEADER grd;
+	struct GRD_HEADER grd, new;
 	struct CDF_CPT {
 		double	z;	/* Data value  */
 		double	f;	/* Cumulative distribution function f(z)  */
@@ -105,17 +107,17 @@ int main (int argc, char **argv)
 
 	FILE	*fpc = NULL;
 
-	void *New_Grd2cpt_Ctrl (), Free_Grd2cpt_Ctrl (struct GRD2CPT_CTRL *C);
+	void *New_grd2cpt_Ctrl (), Free_grd2cpt_Ctrl (struct GRD2CPT_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRD2CPT_CTRL *)New_Grd2cpt_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRD2CPT_CTRL *)New_grd2cpt_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	w = e = s = n = 0.0;
 
-	/* Get list of available color tables in $GMTHOME/share */
+	/* Get list of available color tables in $GMT_SHAREDIR */
 
-	GMT_getsharepath (CNULL, "GMT_CPT", ".lis", CPT_lis);
+	GMT_getsharepath ("conf", "gmt_cpt", ".conf", CPT_lis);
 	if ((fpc = fopen (CPT_lis, "r")) == NULL) {
 		fprintf (stderr, "%s: ERROR: Cannot open file %s\n", GMT_program, CPT_lis);
 		exit (EXIT_FAILURE);
@@ -224,22 +226,35 @@ int main (int argc, char **argv)
 			}
 		}
 		else {
-			grdfile = argv[i];
-			if (GMT_read_grd_info (grdfile, &grd)) {
-				fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
-				error++;
+			if (ngrd > mgrd) {
+				fprintf (stderr, "%s: GMT USAGE ERROR:  Cannot handle more than %i grids\n", GMT_program, mgrd);
+				exit(EXIT_FAILURE);
 			}
+
+			grdfile[ngrd] = argv[i];
+
+			if (ngrd == 0) {
+				if (GMT_err_pass (GMT_read_grd_info (grdfile[0], &grd), grdfile[0])) error++;
+			}
+			else {
+				if (GMT_err_pass (GMT_read_grd_info (grdfile[ngrd], &new), grdfile[ngrd])) error++;
+				if (new.x_min != grd.x_min || new.x_max != grd.x_max || new.y_min != grd.y_min || new.y_max != grd.y_max || new.nx != grd.nx || new.ny != grd.ny) {
+					fprintf (stderr, "%s: GMT USAGE ERROR:  Grid %s does not have the same dimensions and size as grid %s\n", GMT_program, grdfile[ngrd], grdfile[0]);
+					exit (EXIT_FAILURE);
+				}
+			}
+			ngrd++;
 		}
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grd2cpt %s - Make a a linear or histogram-equalized color palette table from a grdfile\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grd2cpt <grdfile> [-C<table>] [-D] [-E<nlevels> [-I] [-L<min_limit>/<max_limit>]\n");
+		fprintf (stderr, "grd2cpt %s - Make a linear or histogram-equalized color palette table from a grdfile\n\n", GMT_VERSION);
+		fprintf (stderr, "usage: grd2cpt <grdfiles> [-C<table>] [-D] [-E<nlevels> [-I] [-L<min_limit>/<max_limit>]\n");
 		fprintf (stderr, "\t[-M] [-N] [-Q[i|o]] [%s] [-S<z_start>/<z_stop>/<z_inc>] [-T<-|+|=|_>] [-V] [-Z]\n", GMT_Rgeo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
+		fprintf (stderr, "\t<grdfiles> names of one or more 2-D binary data sets\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C  Specify a colortable [Default is rainbow]:\n");
 		fprintf (stderr, "\t   [Original z-range is given in brackets]\n");
@@ -274,11 +289,16 @@ int main (int argc, char **argv)
 		Ctrl->C.active = TRUE;
 		Ctrl->C.file = strdup ("rainbow");
 	}
-		
+	if (!Ctrl->E.active) Ctrl->E.levels =  11;	/* Default number of levels */
+
 	/* Open the specified master color table */
 
 	error += GMT_set_cpt_path (CPT_file, Ctrl->C.file);
 
+	if (ngrd < 1) {
+		fprintf(stderr,"%s: GMT USAGE ERROR:  No grid name(s) specified.\n", GMT_program);
+		error++;
+	}
 	if (Ctrl->L.active && Ctrl->L.min >= Ctrl->L.max) {
 		fprintf(stderr,"%s: GMT SYNTAX ERROR -L option:  min_limit must be less than max_limit.\n", GMT_program);
 		error++;
@@ -287,10 +307,12 @@ int main (int argc, char **argv)
 		fprintf (stderr,"%s: GMT SYNTAX ERROR -S option:  Bad arguments\n", GMT_program);
 		error++;
 	}
-	
-	if (error) exit (EXIT_FAILURE);
+	if (Ctrl->S.active && (Ctrl->T.active || Ctrl->E.active)) {
+		fprintf (stderr,"%s: GMT USAGE ERROR -S option:  Cannot be combined with -E nor -T option.\n", GMT_program);
+		error++;
+	}
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
+	if (error) exit (EXIT_FAILURE);
 
 	if (Ctrl->N.active) GMT_cpt_flags += 1;	/* bit 0 controls if BFN will be written out */
 	if (Ctrl->D.active) GMT_cpt_flags += 2;	/* bit 1 controls if BF will be set to equal bottom/top rgb value */
@@ -306,27 +328,18 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT ERROR: Subset exceeds data domain!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
-		ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
-		nxy = nx * ny;
-
-		zdata = (float *) GMT_memory (VNULL, (size_t) nxy, sizeof (float), GMT_program);
-
-		if (GMT_read_grd (grdfile, &grd, zdata, w, e, s, n, GMT_pad, FALSE)) {
-			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grdfile);
-			GMT_free ((void *)zdata);
-			exit (EXIT_FAILURE);
-		}
+		nxy = GMT_get_nm (GMT_get_n (w, e, grd.x_inc, grd.node_offset), GMT_get_n (s, n, grd.y_inc, grd.node_offset));
 	}
-	else {
-		nxy = grd.nx * grd.ny;
-		zdata = (float *) GMT_memory (VNULL, (size_t) nxy, sizeof (float), GMT_program);
+	else
+		nxy = GMT_get_nm (grd.nx, grd.ny);
 
-		if (GMT_read_grd (grdfile, &grd, zdata, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grdfile);
-			GMT_free ((void *)zdata);
-			exit (EXIT_FAILURE);
-		}
+	nxyg = GMT_get_nm (nxy, ngrd);
+
+	zdata = (float *) GMT_memory (VNULL, (size_t) nxyg, sizeof (float), GMT_program);
+
+	for (i = 0; i < ngrd; i++) {
+		GMT_err_fail (GMT_read_grd_info (grdfile[i], &grd), grdfile[i]);
+		GMT_err_fail (GMT_read_grd (grdfile[i], &grd, &(zdata[i*nxy]), w, e, s, n, GMT_pad, FALSE), grdfile[i]);
 	}
 
 	/* Loop over the file and find NaNs.  If set limits, may create more NaNs  */
@@ -337,7 +350,7 @@ int main (int argc, char **argv)
 
 		grd.z_min = Ctrl->L.min;
 		grd.z_max = Ctrl->L.max;
-		for (i = 0; i < nxy; i++) {
+		for (i = 0; i < nxyg; i++) {
 			if (GMT_is_fnan (zdata[i]))
 				nfound++;
 			else {
@@ -355,7 +368,7 @@ int main (int argc, char **argv)
 	else {
 		Ctrl->L.min = grd.z_max;	/* This is just to double check grd.z_min, grd.z_max  */
 		Ctrl->L.max = grd.z_min;
-		for (i = 0; i < nxy; i++) {
+		for (i = 0; i < nxyg; i++) {
 			if (GMT_is_fnan (zdata[i]))
 				nfound++;
 			else {
@@ -368,7 +381,7 @@ int main (int argc, char **argv)
 		grd.z_min = Ctrl->L.min;
 		grd.z_max = Ctrl->L.max;
 	}
-	ngood = nxy - nfound;	/* This is the number of non-NaN points for the cdf function  */
+	ngood = nxyg - nfound;	/* This is the number of non-NaN points for the cdf function  */
 	mean /= ngood;
 	sd /= ngood;
 	sd = sqrt(sd - mean*mean);
@@ -379,7 +392,8 @@ int main (int argc, char **argv)
 
 	/* Now the zdata are ready.  Decide how to make steps in z.  */
 	if (Ctrl->S.active) {
-		Ctrl->E.levels =  (grd.z_min < Ctrl->S.low) ? 1 : 0;
+		/* Use predefined levels and interval */
+		Ctrl->E.levels = (grd.z_min < Ctrl->S.low) ? 1 : 0;
 		Ctrl->E.levels += (int)floor((Ctrl->S.high - Ctrl->S.low)/Ctrl->S.inc) + 1;
 		if (grd.z_max > Ctrl->S.high) Ctrl->E.levels++;
 		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
@@ -399,67 +413,59 @@ int main (int argc, char **argv)
 		}
 		if (j == Ctrl->E.levels-1) cdf_cpt[j].z = grd.z_max;
 	}
-	else {
-		/* Make a equaldistant color map from grd.z_min to grd.z_max */
-		if (Ctrl->T.active) {
-			double start, range;
-			
-			if ( !Ctrl->E.active ) {
-				Ctrl->E.levels =  11;
-			}
-			switch (Ctrl->T.kind) {
-				case -1:
-					start = -fabs((double)grd.z_min);
-					break;
-				case 1:
-					start = -fabs((double)grd.z_max);
-					break;
-				case -2:
-					start = -MIN(fabs((double)grd.z_min), fabs((double)grd.z_max));
-					break;
-				case 2:
-					start = -MAX(fabs((double)grd.z_min), fabs((double)grd.z_max));
-					break;
-				default:
-					start = grd.z_min;
-					break;
-			}
-			range = (Ctrl->T.kind) ? 2.0 * fabs (start) : grd.z_max - grd.z_min;
-			Ctrl->S.inc = range / (double)(Ctrl->E.levels - 1);
-			cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
-			for (i = 0; i < Ctrl->E.levels; i++) {
-				cdf_cpt[i].z = start + i * Ctrl->S.inc;
-			}
-		}
-		else {
-			/* This is completely ad-hoc.  It chooses z based on steps of 0.1 for a Gaussian CDF:  */
-			Ctrl->E.levels = 11;
-			cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
-                	/* Stupid bug fix here:  
-                	        If (mean-1.28155*sd <= grd.z_min || mean+1.28155*sd >= grd.z_max) then
-                	          reset mean and sd so they fit inside available range:  */
-                          
-			if ((mean - 1.28155*sd) <= grd.z_min || (mean + 1.28155*sd) >= grd.z_max) {
-				mean = 0.5 * (grd.z_min + grd.z_max);
-				sd = (grd.z_max - mean) / 1.5;
-				if (sd <= 0.0) {
-					fprintf (stderr, "%s:  ERROR.  Min and Max data values are equal.\n", GMT_program);
-					exit (EXIT_FAILURE);
-				}
-			}	/* End of stupid bug fix  */
 
-			cdf_cpt[0].z = grd.z_min;
-			cdf_cpt[1].z = mean - 1.28155 * sd;
-			cdf_cpt[2].z = mean - 0.84162 * sd;
-			cdf_cpt[3].z = mean - 0.52440 * sd;
-			cdf_cpt[4].z = mean - 0.25335 * sd;
-			cdf_cpt[5].z = mean;
-			cdf_cpt[6].z = mean + 0.25335 * sd;
-			cdf_cpt[7].z = mean + 0.52440 * sd;
-			cdf_cpt[8].z = mean + 0.84162 * sd;
-			cdf_cpt[9].z = mean + 1.28155 * sd;
-			cdf_cpt[10].z = grd.z_max;
+	else if (Ctrl->T.active || Ctrl->E.active) {
+		/* Make a equaldistant color map from grd.z_min to grd.z_max */
+		double start, range;
+
+		switch (Ctrl->T.kind) {
+			case -1:
+				start = -fabs((double)grd.z_min);
+				break;
+			case 1:
+				start = -fabs((double)grd.z_max);
+				break;
+			case -2:
+				start = -MIN(fabs((double)grd.z_min), fabs((double)grd.z_max));
+				break;
+			case 2:
+				start = -MAX(fabs((double)grd.z_min), fabs((double)grd.z_max));
+				break;
+			default:
+				start = grd.z_min;
+				break;
 		}
+		range = (Ctrl->T.kind) ? 2.0 * fabs (start) : grd.z_max - grd.z_min;
+		Ctrl->S.inc = range / (double)(Ctrl->E.levels - 1);
+		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
+		for (i = 0; i < Ctrl->E.levels; i++) {
+			cdf_cpt[i].z = start + i * Ctrl->S.inc;
+		}
+	}
+
+	else {
+		/* This is completely ad-hoc.  It chooses z based on steps of 0.1 for a Gaussian CDF:  */
+		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
+		if ((mean - 1.28155*sd) <= grd.z_min || (mean + 1.28155*sd) >= grd.z_max) {
+			mean = 0.5 * (grd.z_min + grd.z_max);
+			sd = (grd.z_max - mean) / 1.5;
+			if (sd <= 0.0) {
+				fprintf (stderr, "%s:  ERROR.  Min and Max data values are equal.\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+		}	/* End of stupid bug fix  */
+
+		cdf_cpt[0].z = grd.z_min;
+		cdf_cpt[1].z = mean - 1.28155 * sd;
+		cdf_cpt[2].z = mean - 0.84162 * sd;
+		cdf_cpt[3].z = mean - 0.52440 * sd;
+		cdf_cpt[4].z = mean - 0.25335 * sd;
+		cdf_cpt[5].z = mean;
+		cdf_cpt[6].z = mean + 0.25335 * sd;
+		cdf_cpt[7].z = mean + 0.52440 * sd;
+		cdf_cpt[8].z = mean + 0.84162 * sd;
+		cdf_cpt[9].z = mean + 1.28155 * sd;
+		cdf_cpt[10].z = grd.z_max;
 	}
 
 	/* Get here when we are ready to go.  cdf_cpt[].z contains the sample points.  */
@@ -472,7 +478,7 @@ int main (int argc, char **argv)
 			cdf_cpt[j].f = 1.0;
 		else {
 			nfound = 0;
-			for (i = 0; i < nxy; i++) {
+			for (i = 0; i < nxyg; i++) {
 				if (!GMT_is_fnan (zdata[i]) && zdata[i] <= cdf_cpt[j].z) nfound++;
 			}
 			cdf_cpt[j].f = (double)(nfound-1)/(double)(ngood-1);
@@ -484,9 +490,9 @@ int main (int argc, char **argv)
 
 	/* Write to GMT_stdout.  */
 
-	fprintf (GMT_stdout, "#\tcpt file created by: %s", GMT_program);
-	for (i = 1; i < argc; i++) fprintf (GMT_stdout, " %s", argv[i]);
-	fprintf (GMT_stdout, "\n");
+	sprintf (format, "#\tcpt file created by: %s", GMT_program);	GMT_fputs (format, GMT_stdout);
+	for (i = 1; i < argc; i++) {sprintf (format, " %s", argv[i]);	GMT_fputs (format, GMT_stdout);}
+	GMT_fputs ("\n", GMT_stdout);
 
 
 	z = (double *) GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof (double), GMT_program);
@@ -499,23 +505,23 @@ int main (int argc, char **argv)
 	GMT_free ((void *)zdata);
 	GMT_free ((void *)z);
 
-	Free_Grd2cpt_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grd2cpt_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Grd2cpt_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grd2cpt_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRD2CPT_CTRL *C;
-	
-	C = (struct GRD2CPT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRD2CPT_CTRL), "New_Grd2cpt_Ctrl");
-	
+
+	C = (struct GRD2CPT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRD2CPT_CTRL), "New_grd2cpt_Ctrl");
+
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	return ((void *)C);
 }
 
-void Free_Grd2cpt_Ctrl (struct GRD2CPT_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	GMT_free ((void *)C);	
+void Free_grd2cpt_Ctrl (struct GRD2CPT_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) free ((void *)C->C.file);
+	GMT_free ((void *)C);
 }
