@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfft.c,v 1.26 2006/05/27 01:26:32 remko Exp $
+ *	$Id: grdfft.c,v 1.55 2010/03/24 03:24:06 jluis Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -18,8 +18,7 @@
 /*
  *  grdfft.c
  *
- * A program to do various operations
- * on grdfiles in the frequency domain.
+ * A program to do various operations on grid files in the frequency domain.
  *
  * W.H.F. Smith, 7 March 1990.
  *
@@ -30,6 +29,7 @@
  *   Note to myself:  users want azimuthal power spectra, and
  *   cross-spectra between two grids.  --whfs
  * 5/17/2006 PW: Added enhanced -F option for Gaussian band-pass.
+ * 6/17/2009 PW: Added enhanced -F option for Butterworth band-pass.
  *
  * Version:	4
  */
@@ -41,55 +41,55 @@
 #endif
 
 struct GRDFFT_CTRL {
-	int n_op_count, n_par;
-	int *operation;
+	GMT_LONG n_op_count, n_par;
+	GMT_LONG *operation;
 	double	*par;
 
 	struct A {	/* -A<azimuth> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} A;
 	struct C {	/* -C<zlevel> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} C;
 	struct D {	/* -D[<scale>|g] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} D;
 	struct E {	/* -E[x_or_y][w] */
-		BOOLEAN active;
-		int mode;
+		GMT_LONG active;
+		GMT_LONG mode;
 	} E;
 	struct F {	/* -F[x_or_y]<lc>/<lp>/<hp>/<hc> or -F[x_or_y]<lo>/<hi> */
-		BOOLEAN active;
-		int mode;
+		GMT_LONG active;
+		GMT_LONG mode;
 		double lc, lp, hp, hc;
 	} F;
 	struct G {	/* -G<outfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -I[<scale>|g] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} I;
 	struct L {	/* -L */
-		BOOLEAN active;
+		GMT_LONG active;
 	} L;
 	struct M {	/* -M */
-		BOOLEAN active;
+		GMT_LONG active;
 	} M;
 	struct N {	/* -N<stuff> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} N;
 	struct S {	/* -S<scale> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} S;
 	struct T {	/* -T<te/rl/rm/rw/ri> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double te, rhol, rhom, rhow, rhoi;
 	} T;
 };
@@ -102,9 +102,10 @@ struct GRDFFT_CTRL {
 #define DIFFERENTIATE	2
 #define INTEGRATE	3
 #define ISOSTASY	4
-#define FILTER_COS	5
-#define FILTER_EXP	6
-#define SPECTRUM	7
+#define FILTER_EXP	5
+#define FILTER_BW	6
+#define FILTER_COS	7
+#define SPECTRUM	8
 
 /* Macro definition ij_data(i,j) finds the array index to an element
 	containing the real data(i,j) in the padded complex array:  */
@@ -112,10 +113,10 @@ struct GRDFFT_CTRL {
 #define	ij_data(i,j) (2*(nx2*((j)+j_data_start)+(i)+i_data_start))
 
 
-int	narray[2];
-int	i, j, k, n, nx2 = 0, ny2 = 0, ndatac, i_data_start, j_data_start;
+GMT_LONG	narray[2];
+GMT_LONG	i, j, k, n, nx2 = 0, ny2 = 0, ndatac, i_data_start, j_data_start;
 
-BOOLEAN	map_units = FALSE, force_narray = FALSE, suggest_narray = FALSE, leave_trend_alone = FALSE, n_user_set = FALSE;
+GMT_LONG	map_units = FALSE, force_narray = FALSE, suggest_narray = FALSE, leave_trend_alone = FALSE, n_user_set = FALSE;
 
 double	data_var, data2_var, delta_kx, delta_ky;
 double	a[3];	/* Plane fitting coefficients  */
@@ -133,16 +134,19 @@ struct F_INFO {
 	double	htaper[3];	/* High taper width for r, x, and y	*/
 	double	llambda[3];	/* Low full-wavelength where Gauss amp = 0.5 for r, x, and y	*/
 	double	hlambda[3];	/* High full-wavelength where Gauss amp = 0.5  for r, x, and y	*/
-	int	do_this[3];	/* T/F this filter wanted for r, x, and y	*/
-	int	set_already;
-	int	kind;		/* 0 = cosine taper, 1 = Gaussian */
+	double	bw_order;	/* Order, N, of Butterworth filter	*/
+	PFD filter;		/* Points to the correct filter function */
+	GMT_LONG	do_this[3];	/* T/F this filter wanted for r, x, and y	*/
+	GMT_LONG	set_already;
+	GMT_LONG	kind;		/* FILTER_EXP, FILTER_BW, FILTER_COS  */
+	GMT_LONG	arg;		/* 0 = Gaussian, 1 = Butterworth, 2 = cosine taper,  */
 } f_info;
 
 struct FFT_SUGGESTION {
-	int	nx;
-	int	ny;
-	int	worksize;	/* # single-complex elements needed in work array  */
-	int	totalbytes;	/* (8*(nx*ny + worksize))  */
+	GMT_LONG	nx;
+	GMT_LONG	ny;
+	GMT_LONG	worksize;	/* # single-complex elements needed in work array  */
+	GMT_LONG	totalbytes;	/* (8*(nx*ny + worksize))  */
 	double	run_time;
 	double	rms_rel_err;
 }	fft_sug[3];	/* [0] holds fastest, [1] most accurate, [2] least storage  */
@@ -151,21 +155,25 @@ struct FFT_SUGGESTION {
 float	*datac, *workc;
 double	scale_out = 1.0;
 
+double cosine_weight (double freq, int j), gauss_weight (double freq, int j), bw_weight (double freq, int j);
+
 int main (int argc, char **argv)
 {
-	int op_count = 0, par_count = 0, filter_type = 0;
+	GMT_LONG op_count = 0, par_count = 0, filter_type = 0;
 
-	BOOLEAN error = FALSE, stop, give_wavelength = FALSE;
+	GMT_LONG error = FALSE, stop, give_wavelength = FALSE;
 	struct GRDFFT_CTRL *Ctrl;
-	int parse_f_string (char *c), read_data(struct GRDFFT_CTRL *Ctrl, int argc, char **argv),  count_slashes (char *txt);
-	void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par), do_isostasy(double *par);
-	void do_filter_cos(void), do_filter_exp(void), do_spectrum(double *par, int give_wavelength);
+	GMT_LONG parse_f_string (char *c), count_slashes (char *txt);
+	void read_data(struct GRDFFT_CTRL *Ctrl, int argc, char **argv);
+	void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par);
+	void do_filter(void), do_spectrum(double *par, GMT_LONG give_wavelength), do_isostasy(double *par);
+	double get_filter_weight (GMT_LONG k);
 
-	void *New_Grdfft_Ctrl (), Free_Grdfft_Ctrl (struct GRDFFT_CTRL *C);
+	void *New_grdfft_Ctrl (), Free_grdfft_Ctrl (struct GRDFFT_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDFFT_CTRL *) New_Grdfft_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDFFT_CTRL *) New_grdfft_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	for (i = 0; i < 3; i++) {
 		f_info.lc[i] = f_info.lp[i] = -1.0;	/* Set negative, below valid frequency range  */
@@ -190,7 +198,7 @@ int main (int argc, char **argv)
 			
                                 case 'A':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = AZIMUTHAL_DERIVATIVE;
 					op_count++;
 					Ctrl->n_par++;
@@ -203,7 +211,7 @@ int main (int argc, char **argv)
                                         break;
                                 case 'C':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = UP_DOWN_CONTINUE;
 					op_count++;
 					Ctrl->n_par++;
@@ -216,11 +224,12 @@ int main (int argc, char **argv)
                                         break;
                                 case 'D':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = DIFFERENTIATE;
 					op_count++;
 					Ctrl->n_par++;
 					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
+					Ctrl->par[par_count] = 0.0;
                                         if (argv[i][2]) {
 						Ctrl->par[par_count] = (argv[i][2] == 'g' || argv[i][2] == 'G') ? mGal_at_45 : atof (&argv[i][2]);
 						if (Ctrl->par[par_count] == 0.0) {
@@ -232,7 +241,7 @@ int main (int argc, char **argv)
                                         break;
                                 case 'E':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = SPECTRUM;
 					op_count++;
 					Ctrl->n_par++;
@@ -254,12 +263,12 @@ int main (int argc, char **argv)
                                 case 'F':
                                 	if (!(f_info.set_already)) {
 						filter_type = count_slashes (&argv[i][2]);
+						f_info.kind = FILTER_EXP + (filter_type - 1);
 						Ctrl->n_op_count++;
-						Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
-                                        	Ctrl->operation[op_count] = (filter_type > 2) ? FILTER_COS : FILTER_EXP;
+						Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
+                                        	Ctrl->operation[op_count] = f_info.kind;
                                         	op_count++;
                                         	f_info.set_already = TRUE;
-						f_info.kind = (filter_type == 1);
                                         }
                                         if (parse_f_string(&argv[i][2])) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: ", GMT_program);
@@ -271,12 +280,13 @@ int main (int argc, char **argv)
                                         break;
                                 case 'I':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = INTEGRATE;
 					op_count++;
 					Ctrl->n_par++;
 					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
-                                        if (argv[i][2]) {
+   					Ctrl->par[par_count] = 0.0;
+					if (argv[i][2]) {
 						Ctrl->par[par_count] = (argv[i][2] == 'g' || argv[i][2] == 'G') ? mGal_at_45 : atof (&argv[i][2]);
 						if (Ctrl->par[par_count] == 0.0) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  scale must be nonzero\n", GMT_program);
@@ -297,7 +307,7 @@ int main (int argc, char **argv)
 					else if (argv[i][2] == 'q' || argv[i][2] == 'Q')
 						suggest_narray = TRUE;
 					else {
-	                                        sscanf(&argv[i][2], "%d/%d", &nx2, &ny2);
+	                                        sscanf(&argv[i][2], "%" GMT_LL "d/%" GMT_LL "d", &nx2, &ny2);
 						n_user_set = TRUE;
 					}
                                         break;
@@ -306,8 +316,9 @@ int main (int argc, char **argv)
                                         scale_out = (argv[i][2] == 'd' || argv[i][2] == 'D') ? 1.0e6: atof (&argv[i][2]);
                                         break;
                                 case 'T':
+                                        Ctrl->T.active = TRUE;
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = ISOSTASY;
 					op_count++;
  					Ctrl->n_par += 5;
@@ -334,36 +345,38 @@ int main (int argc, char **argv)
         }
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdfft %s - Perform mathematical operations on grdfiles in the wavenumber (or frequency) domain\n\n", GMT_VERSION);
+		fprintf (stderr, "grdfft %s - Perform mathematical operations on grid files in the wavenumber (or frequency) domain\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: grdfft <in_grdfile> [-G<out_grdfile>] [-A<azimuth>] [-C<zlevel>]\n");
 		fprintf (stderr,"\t[-D[<scale>|g]] [-E[x_or_y][w]] [-F[x_or_y]<parameters>] [-I[<scale>|g]] [-L] [-M]\n");
 		fprintf (stderr,"\t[-N<stuff>] [-S<scale>] [-T<te/rl/rm/rw/ri>] [-V]\n\n");
 		
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		
-		fprintf (stderr,"\tin_grdfile is the input netCDF grdfile\n");
+		fprintf (stderr,"\tin_grdfile is the input netCDF grid file\n");
 		fprintf (stderr, "\tOPTIONS:\n");
-		fprintf (stderr,"\t-G filename for output netCDF grdfile\n");
+		fprintf (stderr,"\t-G filename for output netCDF grid file\n");
 		fprintf (stderr,"\t-A<azimuth> Take azimuthal derivative along line <azimuth> degrees CW from North.\n");
 		fprintf (stderr,"\t-C<zlevel>  Continue field upward (+) or downward (-) to zlevel (meters).\n");
 		fprintf (stderr,"\t-D Differentiate, i.e., multiply by kr [ * scale].  Use -Dg to get mGal from m]\n");
 		fprintf (stderr,"\t-E Estimate spEctrum of r [x] [y].  Write f, power[f], 1 std dev(power[f]) to stdout.\n");
 		fprintf (stderr,"\t\tAppend w to write wavelength instead of frequency.\n");
-		fprintf (stderr,"\t-F Filter r [x] [y] freq according to one of two kinds of filter specifications:.\n");
+		fprintf (stderr,"\t-F Filter r [x] [y] freq according to one of three kinds of filter specifications:.\n");
 		fprintf (stderr,"\t   a) Cosine band-pass: Append four wavelengths <lc>/<lp>/<hp>/<hc>.\n");
 		fprintf (stderr,"\t      freq outside <lc>/<hc> are cut; inside <lp>/<hp> are passed, rest are tapered.\n");
 		fprintf (stderr,"\t      Replace wavelength by - to skip, e.g.  -F-/-/500/100 is a low-pass filter.\n");
 		fprintf (stderr,"\t   b) Gaussian band-pass: Append two wavelengths <lo>/<hi> where filter amplitudes = 0.5.\n");
 		fprintf (stderr,"\t      Replace wavelength by - to skip, e.g.  -F300/- is a high-pass Gaussian filter.\n");
+		fprintf (stderr,"\t   c) Butterworth band-pass: Append two wavelengths and order <lo>/<hi>/<order> where filter amplitudes = 0.5.\n");
+		fprintf (stderr,"\t      Replace wavelength by - to skip, e.g.  -F300/-/2 is a high-pass 2nd-order Butterworth filter.\n");
 		fprintf (stderr,"\t-I Integrate, i.e., divide by kr [ * scale].  Use -Ig to get m from mGal]\n");
 		fprintf (stderr,"\t-L Leave trend alone.  Do not remove least squares plane from data.  [Default removes plane].\n");
 		fprintf (stderr,"\t-M Map units used.  Convert grid dimensions from degrees to meters.\n");
 		fprintf (stderr,"\t-N<stuff>  Choose or inquire about suitable grid dimensions for FFT.\n");
 		fprintf (stderr,"\t\t-Nf will force the FFT to use the dimensions of the data.\n");
 		fprintf (stderr,"\t\t-Nq will inQuire about more suitable dimensions.\n");
-		fprintf (stderr,"\t\t-N<nx>/<ny> will do FFT on array size <nx>/<ny> (Must be >= grdfile size).\n");
+		fprintf (stderr,"\t\t-N<nx>/<ny> will do FFT on array size <nx>/<ny> (Must be >= grid size).\n");
 		fprintf (stderr,"\t\tDefault chooses dimensions >= data which optimize speed, accuracy of FFT.\n");
-		fprintf (stderr,"\t\tIf FFT dimensions > grdfile dimensions, data are extended and tapered to zero.\n");
+		fprintf (stderr,"\t\tIf FFT dimensions > grid dimensions, data are extended and tapered to zero.\n");
 		fprintf (stderr,"\t-S multiply field by scale after inverse FFT [1.0]\n");
 		fprintf (stderr,"\t   Give -Sd to convert deflection of vertical to micro-radians\n");
 		fprintf (stderr,"\t-T Compute isostatic response.  Input file is topo load. Append elastic thickness,\n");
@@ -400,13 +413,8 @@ int main (int argc, char **argv)
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
+	read_data (Ctrl, argc, argv);
 
-	if (read_data(Ctrl, argc, argv) ) {
-		fprintf (stderr,"%s: Fatal GMT_memory error.  Exiting.\n", GMT_program);
-		exit (EXIT_FAILURE);
-	}
-	
 	/* Check that no NaNs are present */
 	
 	stop = FALSE;
@@ -416,6 +424,16 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+#ifdef FTEST
+	{
+		double f = 0.0;
+		while (f < 3.0) {
+			printf ("%g\t%g\n", f, f_info.filter (f, 0));	/* Radial filter */
+			f += 0.01;
+		}
+		exit (-1);
+	}
+#endif
 	if (gmtdefs.verbose) fprintf (stderr, "%s: forward FFT...", GMT_program);
 	
 	GMT_fourt (datac, narray, 2, -1, 1, workc);
@@ -448,12 +466,16 @@ int main (int argc, char **argv)
 				par_count += 5;
 				break;
 			case FILTER_COS:
-				if (gmtdefs.verbose) fprintf (stderr, "filter...");
-				do_filter_cos ();
+				if (gmtdefs.verbose) fprintf (stderr, "cosine filter...");
+				do_filter ();
 				break;
 			case FILTER_EXP:
-				if (gmtdefs.verbose) fprintf (stderr, "filter...");
-				do_filter_exp ();
+				if (gmtdefs.verbose) fprintf (stderr, "Gaussian filter...");
+				do_filter ();
+				break;
+			case FILTER_BW:
+				if (gmtdefs.verbose) fprintf (stderr, "Butterworth filter...");
+				do_filter ();
 				break;
 			case SPECTRUM:
 				if (gmtdefs.verbose) fprintf (stderr, "spectrum...");
@@ -474,10 +496,7 @@ int main (int argc, char **argv)
 
 		/* The data are in the middle of the padded array:  */
 
-		if (GMT_write_grd (Ctrl->G.file, &h, datac, h.x_min, h.x_max, h.y_min, h.y_max, GMT_pad, TRUE)) {
-			fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
-			exit (EXIT_FAILURE);
-		}
+		GMT_err_fail (GMT_write_grd (Ctrl->G.file, &h, datac, h.x_min, h.x_max, h.y_min, h.y_max, GMT_pad, TRUE), Ctrl->G.file);
 	}
 	
 	GMT_free ((void *)workc);
@@ -485,32 +504,29 @@ int main (int argc, char **argv)
 
 	if (gmtdefs.verbose) fprintf (stderr, "Done\n");
 
-	Free_Grdfft_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdfft_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-int read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
+void read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 {
-	int	worksize, factors[32];
+	GMT_LONG	worksize, factors[32];
 	double	tdummy, edummy;
-	void	suggest_fft(int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print);
-	void	fourt_stats(int nx, int ny, int *f, double *r, int *s, double *t);
+	void	suggest_fft(GMT_LONG nx, GMT_LONG ny, struct FFT_SUGGESTION *fft_sug, GMT_LONG do_print);
+	void	fourt_stats(GMT_LONG nx, GMT_LONG ny, GMT_LONG *f, double *r, GMT_LONG *s, double *t);
 	void remove_plane(void), taper_edges(void);
 	
-	if (GMT_read_grd_info (infile, &h)) {
-		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, infile);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_read_grd_info (infile, &h), infile);
 	
 	GMT_grd_init (&h, argc, argv, TRUE);
 
 	/* Get dimensions as may be appropriate:  */
 	if (n_user_set) {
 		if (nx2 < h.nx || ny2 < h.ny) {
-			fprintf(stderr,"%s: Error: You specified -Nnx/ny smaller than input grdfile.  Ignored.\n", GMT_program);
+			fprintf(stderr,"%s: Error: You specified -Nnx/ny smaller than input grid.  Ignored.\n", GMT_program);
 			n_user_set = FALSE;
 		}
 	}
@@ -539,7 +555,7 @@ int read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 	narray[0] = nx2;
 	narray[1] = ny2;
 	fourt_stats(nx2, ny2, factors, &edummy, &worksize, &tdummy);
-	if (gmtdefs.verbose) fprintf(stderr,"%s:  Data dimension %d %d\tFFT dimension %d %d\n", GMT_program,
+	if (gmtdefs.verbose) fprintf(stderr,"%s:  Data dimension %d %d\tFFT dimension %ld %ld\n", GMT_program,
 		h.nx, h.ny, nx2, ny2);
 
 	/* Make an array of floats 2 * nx2 * ny2 for complex data:  */
@@ -565,10 +581,7 @@ int read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 	GMT_pad[1] = nx2 - h.nx - GMT_pad[0];
 	GMT_pad[2] = ny2 - h.ny - GMT_pad[3];
 
-	if (GMT_read_grd (infile, &h, datac, h.x_min, h.x_max, h.y_min, h.y_max, GMT_pad, TRUE)) {
-		fprintf (stderr, "%s: Error reading file %s\n", GMT_program, infile);
-		return (-1);
-	}
+	GMT_err_fail (GMT_read_grd (infile, &h, datac, h.x_min, h.x_max, h.y_min, h.y_max, GMT_pad, TRUE), infile);
 
 	if (!(Ctrl->L.active) )remove_plane();
 	if (!(force_narray) )taper_edges();
@@ -580,8 +593,6 @@ int read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 		delta_kx /= (project_info.M_PR_DEG * cosd (0.5 * (h.y_min + h.y_max)) );
 		delta_ky /= project_info.M_PR_DEG;
 	}
-
-	return(0);
 }
 
 void remove_plane (void)
@@ -595,7 +606,7 @@ void remove_plane (void)
 	spend some multiplications on normalizing the 
 	range of x,y into [-1,1], to avoid roundoff error.  */
 
-	int	one_or_zero;
+	GMT_LONG	one_or_zero;
 	double	x_half_length, one_on_xhl, y_half_length, one_on_yhl;
 	double	sumx2, sumy2, x, y, z;
 
@@ -640,7 +651,7 @@ void remove_plane (void)
 
 void taper_edges (void)
 {
-	int	im, jm, il1, ir1, il2, ir2, jb1, jb2, jt1, jt2;
+	GMT_LONG	im, jm, il1, ir1, il2, ir2, jb1, jb2, jt1, jt2;
 	double	scale, cos_wt;
 
 	/* Note that if nx2 = h.nx+1 and ny2 = h.ny + 1, then this routine
@@ -697,35 +708,35 @@ void taper_edges (void)
 	}
 }
 
-double kx (int k)
+double kx (GMT_LONG k)
 {
 	/* Return the value of kx given k,
 		where kx = 2 pi / lambda x,
 		and k refers to the position
 		in the datac array, datac[k].  */
 
-	int	ii;
+	GMT_LONG	ii;
 
-	ii = ((int)(k/2))%nx2;
+	ii = (k/2)%nx2;
 	if (ii > nx2/2) ii -= nx2;
 	return(ii * delta_kx);
 }
 
-double ky (int k)
+double ky (GMT_LONG k)
 {
 	/* Return the value of ky given k,
 		where ky = 2 pi / lambda y,
 		and k refers to the position
 		in the datac array, datac[k].  */
 
-	int	jj;
+	GMT_LONG	jj;
 
-	jj = ((int)(k/2))/nx2;
+	jj = (k/2)/nx2;
 	if (jj > ny2/2) jj -= ny2;
 	return(jj * delta_ky);
 }
 
-double modk (int k)
+double modk (GMT_LONG k)
 {
 	/* Return the value of sqrt(kx*kx + ky*ky),
 		given k, where k is array position.  */
@@ -735,7 +746,7 @@ double modk (int k)
 
 void do_differentiate (double *par)
 {
-	int	k;
+	GMT_LONG	k;
 	double scale, fact;
 
 	/* Differentiate in frequency domain by multiplying by kr [scale optional] */
@@ -752,7 +763,7 @@ void do_differentiate (double *par)
 void do_integrate (double *par)
 {
 	/* Integrate in frequency domain by dividing by kr [scale optional] */
-	int	k;
+	GMT_LONG	k;
 	double fact, scale;
 
 	scale = (*par != 0.0) ? *par : 1.0;
@@ -766,7 +777,7 @@ void do_integrate (double *par)
 
 void do_continuation (double *zlevel)
 {
-	int	k;
+	GMT_LONG	k;
 	double tmp;
 
 	/* If z is positive, the field will be upward continued using exp[- k z].  */
@@ -780,12 +791,12 @@ void do_continuation (double *zlevel)
 
 void do_azimuthal_derivative (double *azim)
 {
-	int	k;
+	GMT_LONG	k;
 	float	tempr, tempi, fact;
 	double cos_azim, sin_azim;
 
-	cos_azim = cos ((*azim) * D2R);
-	sin_azim = sin ((*azim) * D2R);
+	cos_azim = cosd (*azim);
+	sin_azim = sind (*azim);
 
 	datac[0] = datac[1] = 0.0;
 	for (k = 2; k < ndatac; k+= 2) {
@@ -805,7 +816,7 @@ void do_isostasy (double *par)
 	densities in kg/m**3, Te in m, etc.
 	rw, the water density, is used to set the Airy ratio and the restoring
 	force on the plate (rm - ri)*gravity if ri = rw; so use zero for topo in air.  */
-	int	k;
+	GMT_LONG	k;
 	double	airy_ratio, rigidity_d, d_over_restoring_force, mk, k2, k4, transfer_fn;
 
 	double	te;	/* Elastic thickness, SI units (m)  */
@@ -838,66 +849,22 @@ void do_isostasy (double *par)
 	}
 }
 
-void do_filter_cos (void)
+void do_filter (void)
 {
-	int	k;
-	double	weight, f_wt_cos(int k);
+	GMT_LONG	k;
+	double	weight;
+	double get_filter_weight (GMT_LONG k);
 	for (k = 0; k < ndatac; k += 2) {
-		weight = f_wt_cos(k);
+		weight = get_filter_weight(k);
 		datac[k] *= (float)weight;
 		datac[k+1] *= (float)weight;
 	}
 }
 
-void do_filter_exp (void)
-{
-	int	k;
-	double	weight, f_wt_exp(int k);
-	for (k = 0; k < ndatac; k += 2) {
-		weight = f_wt_exp(k);
-		datac[k] *= (float)weight;
-		datac[k+1] *= (float)weight;
-	}
-}
-
-double f_wt_cos (int k)
+double get_filter_weight (GMT_LONG k)
 {
 	double	freq, return_value = 1.0;
-	int	j;
-	
-	/* Always return immediately when zero weight encountered.  */
-	
-	for (j = 0; j < 3; j++) {
-		if (!(f_info.do_this[j])) continue;	/* Only do one of x, y, or r filtering */
-		switch (j) {
-			case 0:
-				freq = modk(k);
-				break;
-			case 1:
-				freq = kx(k);
-				break;
-			default:
-				freq = ky(k);
-				break;
-		}
-		if (freq <= f_info.lc[j] || freq >= f_info.hc[j]) return(0.0);	/* In fully cut range.  Weight is zero.  */
-			
-		if (freq > f_info.lc[j] && freq < f_info.lp[j])
-			return_value *= 0.5 * (1.0 + cos (M_PI * (freq - f_info.lp[j]) * f_info.ltaper[j]));
-		else if (freq > f_info.hp[j] && freq < f_info.hc[j])
-			return_value *= 0.5 * (1.0 + cos (M_PI * (freq - f_info.hp[j]) * f_info.htaper[j]));
-		/* Else freq is in the fully passed range, so weight is multiplied by 1.0  */
-	}
-
-	/* Get here when all weights have been multiplied on return_value:  */
-	
-	return (return_value);
-}
-
-double f_wt_exp (int k)
-{
-	double	freq, return_value = 1.0, lo, hi;
-	int	j;
+	GMT_LONG	j;
 		
 	for (j = 0; j < 3; j++) {
 		if (!(f_info.do_this[j])) continue;	/* Only do one of x, y, or r filtering */
@@ -912,28 +879,47 @@ double f_wt_exp (int k)
 				freq = ky(k);
 				break;
 		}
-		lo = (f_info.llambda[j] == -1.0) ? 0.0 : exp (-M_LN2 * pow (freq * f_info.llambda[j], 2.0));
-		hi = (f_info.hlambda[j] == -1.0) ? 1.0 : exp (-M_LN2 * pow (freq * f_info.hlambda[j], 2.0));
-		return_value = hi - lo;
+		return_value = f_info.filter (freq, j);
 	}
 
 	return (return_value);
 }
 
-int count_slashes (char *txt)
+double gauss_weight (double freq, int j) {
+	double hi, lo;
+	lo = (f_info.llambda[j] == -1.0) ? 0.0 : exp (-M_LN2 * pow (freq * f_info.llambda[j], 2.0));	/* Low-pass part */
+	hi = (f_info.hlambda[j] == -1.0) ? 1.0 : exp (-M_LN2 * pow (freq * f_info.hlambda[j], 2.0));	/* Hi-pass given by its complementary low-pass */
+	return (hi - lo);
+}
+
+double bw_weight (double freq, int j) {
+	double hi, lo;
+	lo = (f_info.llambda[j] == -1.0) ? 0.0 : sqrt (1.0 / (1.0 + pow (freq * f_info.llambda[j], f_info.bw_order)));	/* Low-pass part */
+	hi = (f_info.hlambda[j] == -1.0) ? 1.0 : sqrt (1.0 / (1.0 + pow (freq * f_info.hlambda[j], f_info.bw_order)));	/* Hi-pass given by its complementary low-pass */
+	return (hi - lo);
+}
+
+double cosine_weight (double freq, int j) {
+	if (freq <= f_info.lc[j] || freq >= f_info.hc[j]) return(0.0);	/* In fully cut range.  Weight is zero.  */
+	if (freq > f_info.lc[j] && freq < f_info.lp[j]) return (0.5 * (1.0 + cos (M_PI * (freq - f_info.lp[j]) * f_info.ltaper[j])));
+	if (freq > f_info.hp[j] && freq < f_info.hc[j]) return (0.5 * (1.0 + cos (M_PI * (freq - f_info.hp[j]) * f_info.htaper[j])));
+	return (1.0);	/* Freq is in the fully passed range, so weight is multiplied by 1.0  */
+}
+
+GMT_LONG count_slashes (char *txt)
 {
-	int i, n;
+	GMT_LONG i, n;
 	for (i = n = 0; txt[i]; i++) if (txt[i] == '/') n++;
 	return (n);
 }
 
-int parse_f_string (char *c)
+GMT_LONG parse_f_string (char *c)
 {
-	int	i, j, n_tokens, pos, descending;
+	GMT_LONG	i, j, n_tokens, pos, descending;
 	double	fourvals[4];
 	char	line[GMT_LONG_TEXT], p[GMT_LONG_TEXT];
 	
-	/* Syntax is either -F[x|y]lc/hc/lp/hp (Cosine taper) or -F[x|y]lo/hi (Gaussian) */
+	/* Syntax is either -F[x|y]lc/hc/lp/hp (Cosine taper), -F[x|y]lo/hi (Gaussian), or 0F[x|y]lo/hi/order (Butterworth) */
 	
 	strcpy(line, c);
 	i = j = 0;	/* j is Filter type:  r=0, x=1, y=2  */
@@ -961,18 +947,20 @@ int parse_f_string (char *c)
 		}
 		else {
 			if ((sscanf(p, "%lf", &fourvals[n_tokens])) != 1) {
-				fprintf(stderr,"%s:  Cannot read token %d.\n", GMT_program, n_tokens);
+				fprintf(stderr,"%s:  Cannot read token %ld.\n", GMT_program, n_tokens);
 				return(TRUE);
 			}
 		}
 		n_tokens++;
 	}
 	
-	if (!(n_tokens == 2 || n_tokens == 4)) {
-		fprintf(stderr,"%s: -F Cannot find 2 or 4 tokens separated by slashes.\n", GMT_program);
+	if (!(n_tokens == 2 || n_tokens == 3 || n_tokens == 4)) {
+		fprintf(stderr,"%s: -F Cannot find 2-4 tokens separated by slashes.\n", GMT_program);
 		return(TRUE);
 	}
 	descending = TRUE;
+	if (f_info.kind == FILTER_BW && n_tokens == 3) n_tokens = 2;	/* So we dont check the order as a wavelength */
+
 	for (i = 1; i < n_tokens; i++) {
 		if (fourvals[i] == -1.0 || fourvals[i-1] == -1.0) continue;
 		if (fourvals[i] > fourvals[i-1]) descending = FALSE;
@@ -981,7 +969,7 @@ int parse_f_string (char *c)
 		fprintf(stderr,"%s: -F Wavelengths are not in descending order.\n", GMT_program);
 		return(TRUE);
 	}
-	if (f_info.kind == 0) {	/* Cosine band-pass specification */
+	if (f_info.kind == FILTER_COS) {	/* Cosine band-pass specification */
 		if ( (fourvals[0] * fourvals[1]) < 0.0 || (fourvals[2] * fourvals[3]) < 0.0) {
 			fprintf(stderr,"%s: -F Pass/Cut specification error.\n", GMT_program);
 			return(TRUE);
@@ -1000,16 +988,24 @@ int parse_f_string (char *c)
 			f_info.hc[j] = (2.0 * M_PI)/fourvals[3];
 			if (fourvals[2] != fourvals[3]) f_info.htaper[j] = 1.0/(f_info.hc[j] - f_info.hp[j]);
 		}
+		f_info.filter = (PFD) cosine_weight;
+	}
+	else if (f_info.kind == FILTER_BW) {	/* Butterworth specification */
+		f_info.llambda[j] = (fourvals[0] == -1.0) ? -1.0 : fourvals[0] / TWO_PI;	/* TWO_PI is used to counteract the 2*pi in the wavenumber */
+		f_info.hlambda[j] = (fourvals[1] == -1.0) ? -1.0 : fourvals[1] / TWO_PI;
+		f_info.bw_order = 2.0 * fourvals[2];
+		f_info.filter = (PFD) bw_weight;
 	}
 	else {	/* Gaussian half-amp specifications */
-		f_info.llambda[j] = fourvals[0] / TWO_PI;	/* TWO_PI is used to counteract the 2*pi in the wavenumber */
-		f_info.hlambda[j] = fourvals[1] / TWO_PI;
+		f_info.llambda[j] = (fourvals[0] == -1.0) ? -1.0 : fourvals[0] / TWO_PI;	/* TWO_PI is used to counteract the 2*pi in the wavenumber */
+		f_info.hlambda[j] = (fourvals[1] == -1.0) ? -1.0 : fourvals[1] / TWO_PI;
+		f_info.filter = (PFD) gauss_weight;
 	}
-	
+	f_info.arg = f_info.kind - FILTER_EXP;
 	return(FALSE);
 }
 
-void do_spectrum (double *par, int give_wavelength)
+void do_spectrum (double *par, GMT_LONG give_wavelength)
 {
 	/* This is modeled on the 1-D case, using the following ideas:
 	 *	In 1-D, we ensemble average over samples of length L = 
@@ -1025,10 +1021,10 @@ void do_spectrum (double *par, int give_wavelength)
 	 *	approximation of the integral.
 	 */
 
-	int	k, nk, nused, ifreq;
+	GMT_LONG	k, nk, nused, ifreq;
 	double	delta_k, r_delta_k, freq, *power, eps_pow;
 	PFD	get_k;
-	char	format[GMT_TEXT_LEN];
+	char	format[GMT_TEXT_LEN], buffer[GMT_LONG_TEXT];
 	/*  Added by WHFS in version 3.001 to normalize power  :  */
 	double	powfactor;
 
@@ -1083,11 +1079,12 @@ void do_spectrum (double *par, int give_wavelength)
 		freq = (k + 1) * delta_k;
 		if (give_wavelength) freq = 1.0/freq;
 		power[k] *= powfactor;
-		fprintf (GMT_stdout, format, freq, power[k], eps_pow * power[k]);
+		sprintf (buffer, format, freq, power[k], eps_pow * power[k]);
+		GMT_fputs (buffer, GMT_stdout);
 	}
 }
 
-void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
+void fourt_stats (GMT_LONG nx, GMT_LONG ny, GMT_LONG *f, double *r, GMT_LONG *s, double *t)
 {
 	/* Find the proportional run time, t, and rms relative error, r,
 	 * of a Fourier transform of size nx,ny.  Also gives s, the size
@@ -1126,21 +1123,22 @@ void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
 	 * W. H. F. Smith, 26 February 1992.
 	 *  */
 
-	int	n_factors, i, ntotal, sum2, sumnot2, nnot2;
-	int	nonsymx, nonsymy, nonsym, storage;
+	GMT_LONG	n_factors, i, sum2, sumnot2, nnot2;
+	GMT_LONG	nonsymx, nonsymy, nonsym, storage;
+	GMT_LONG ntotal;
 	double	err_scale, sig_bits = FSIGNIF;
-	int get_prime_factors(int n, int *f), get_non_symmetric_f(int *f, int n);
+	GMT_LONG get_prime_factors(GMT_LONG n, GMT_LONG *f), get_non_symmetric_f(GMT_LONG *f, GMT_LONG n);
 
 	/* Find workspace needed.  First find non_symmetric factors in nx, ny  */
-	n_factors = get_prime_factors(nx, f);
+	n_factors = get_prime_factors((GMT_LONG)nx, f);
 	nonsymx = get_non_symmetric_f(f, n_factors);
-	n_factors = get_prime_factors(ny, f);
+	n_factors = get_prime_factors((GMT_LONG)ny, f);
 	nonsymy = get_non_symmetric_f(f, n_factors);
 	nonsym = MAX(nonsymx, nonsymy);
 
 	/* Now get factors of ntotal  */
-	ntotal = nx * ny;
-        n_factors = get_prime_factors(ntotal, f);
+	ntotal = ((size_t)nx) * ((size_t)ny);
+        n_factors = get_prime_factors (ntotal, f);
 	storage = MAX(nonsym, f[n_factors - 1]);
 	if (storage == 2)
 		*s = 0;
@@ -1167,10 +1165,10 @@ void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
 	return;
 } 
 
-int get_non_symmetric_f (int *f, int n)
+GMT_LONG get_non_symmetric_f (GMT_LONG *f, GMT_LONG n)
 {
 	/* Return the product of the non-symmetric factors in f[]  */
-	int	i = 0, j = 1, retval = 1;
+	GMT_LONG	i = 0, j = 1, retval = 1;
 
 	if (n == 1) return(f[0]);
 
@@ -1184,24 +1182,24 @@ int get_non_symmetric_f (int *f, int n)
 	return(retval);
 }
 
-void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
+void suggest_fft (GMT_LONG nx, GMT_LONG ny, struct FFT_SUGGESTION *fft_sug, GMT_LONG do_print)
 {
-	int	f[64], xstop, ystop;
-	int	nx_best_t, ny_best_t;
-	int	nx_best_e, ny_best_e;
-	int	nx_best_s, ny_best_s;
-        int     nxg, nyg;       /* Guessed by this routine  */
-        int     nx2, ny2, nx3, ny3, nx5, ny5;   /* For powers  */
+	GMT_LONG	f[64], xstop, ystop;
+	GMT_LONG	nx_best_t, ny_best_t;
+	GMT_LONG	nx_best_e, ny_best_e;
+	GMT_LONG	nx_best_s, ny_best_s;
+        GMT_LONG     nxg, nyg;       /* Guessed by this routine  */
+        GMT_LONG     nx2, ny2, nx3, ny3, nx5, ny5;   /* For powers  */
 	double	current_time, best_time, given_time, s_time, e_time;
-	int	current_space, best_space, given_space, e_space, t_space;
+	GMT_LONG	current_space, best_space, given_space, e_space, t_space;
 	double	current_err, best_err, given_err, s_err, t_err;
-	void	fourt_stats(int nx, int ny, int *f, double *r, int *s, double *t);
+	void	fourt_stats(GMT_LONG nx, GMT_LONG ny, GMT_LONG *f, double *r, GMT_LONG *s, double *t);
 
 
 	fourt_stats(nx, ny, f, &given_err, &given_space, &given_time);
 	given_space += nx*ny;
 	given_space *= 8;
-	if (do_print) fprintf(stderr,"%s:  Data dimension\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+	if (do_print) fprintf(stderr,"%s:  Data dimension\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %ld\n", GMT_program,
 		nx, ny, given_time, given_err, given_space);
 
 	best_err = s_err = t_err = given_err;
@@ -1259,11 +1257,11 @@ void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	}
 
 	if (do_print) {
-		fprintf(stderr,"%s:  Highest speed\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Highest speed\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %ld\n", GMT_program,
 			nx_best_t, ny_best_t, best_time, t_err, t_space);
-		fprintf(stderr,"%s:  Most accurate\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Most accurate\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %ld\n", GMT_program,
 			nx_best_e, ny_best_e, e_time, best_err, e_space);
-		fprintf(stderr,"%s:  Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Least storage\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %lds\n", GMT_program,
 			nx_best_s, ny_best_s, s_time, s_err, best_space);
 	}
 	/* Fastest solution:  */
@@ -1291,7 +1289,7 @@ void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	return;
 }
 
-int get_prime_factors (int n, int *f)
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1318,25 +1316,26 @@ int get_prime_factors (int n, int *f)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992, after D.E. Knuth, vol. II  */
 
-	int	current_factor;	/* The factor currently being tried  */
-	int	max_factor;	/* Don't try any factors bigger than this  */
-	int	n_factors = 0;	/* Returned; one if n is prime  */
-	int	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
-	int	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
-	int	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
-	int	m;	/* Used to keep a working copy of n  */
+	GMT_LONG	current_factor;	/* The factor currently being tried  */
+	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
+	GMT_LONG	n_factors = 0;	/* Returned; one if n is prime  */
+	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
+	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
+	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
+	GMT_LONG	m;	/* Used to keep a working copy of n  */
 
 
 	/* Initialize m and max_factor  */
-	m = abs(n);
+	
+	m = GMT_abs(n);
 	if (m < 2) return(0);
-	max_factor = (int)floor(sqrt((double)m));
+	max_factor = (GMT_LONG)floor(sqrt((double)m));
 
 	/* First find the 2s  */
 	current_factor = 2;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1345,7 +1344,7 @@ int get_prime_factors (int n, int *f)
 	current_factor = 3;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1354,7 +1353,7 @@ int get_prime_factors (int n, int *f)
 	current_factor = 5;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1393,7 +1392,7 @@ int get_prime_factors (int n, int *f)
 
 		while(!(m%current_factor)) {
 			m /= current_factor;
-			f[n_factors] = current_factor;
+			f[n_factors] = (GMT_LONG)current_factor;
 			n_factors++;
 		}
 	}
@@ -1402,25 +1401,25 @@ int get_prime_factors (int n, int *f)
 
 	if (m > 1) {
 		/* m is an additional prime factor of n  */
-		f[n_factors] = m;
+		f[n_factors] = (GMT_LONG)m;
 		n_factors++;
 	}
 	return (n_factors);
 }
 
-void *New_Grdfft_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdfft_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDFFT_CTRL *C;
 	
-	C = (struct GRDFFT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDFFT_CTRL), "New_Grdfft_Ctrl");
+	C = (struct GRDFFT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDFFT_CTRL), "New_grdfft_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 
 	return ((void *)C);
 }
 
-void Free_Grdfft_Ctrl (struct GRDFFT_CTRL *C) {	/* Deallocate control structure */
+void Free_grdfft_Ctrl (struct GRDFFT_CTRL *C) {	/* Deallocate control structure */
 	if (C->operation) GMT_free ((void *)C->operation);	
 	if (C->par) GMT_free ((void *)C->par);	
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
