@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdmask.c,v 1.45 2006/05/29 03:06:46 pwessel Exp $
+ *	$Id: grdmask.c,v 1.85 2010/06/24 19:12:07 guru Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -41,27 +41,28 @@
 #define GRDMASK_INSIDE		2
 
 struct GRDMASK_CTRL {
-	struct A {	/* -A[step] */
-		BOOLEAN active;
+	struct A {	/* -A[m|p|step] */
+		GMT_LONG active;
+		GMT_LONG mode;
 		double step;
 	} A;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<maskfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N<maskvalues> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double mask[GRDMASK_N_CLASSES];	/* values for each level */
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
@@ -69,18 +70,19 @@ struct GRDMASK_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN	error = FALSE, done, nofile = TRUE, greenwich, periodic = FALSE, resample = FALSE;
+	GMT_LONG	error = FALSE, done, nofile = TRUE, periodic = FALSE, resample = FALSE;
 
-	char line[BUFSIZ], ptr[BUFSIZ];
+	char line[BUFSIZ], ptr[BUFSIZ], *not_used = NULL;
 
-	int	i, j, k, ij, nm, n_path, side, fno, n_files = 0, n_alloc = GMT_CHUNK, n_args;
-	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, pos;
-	int 	distance_flag = 0, n_pol = 0;
+	GMT_LONG	i, j, side, fno, n_files = 0, n_args;
+	GMT_LONG	di, dj, i0, j0, n_fields, n_expected_fields, pos;
+	GMT_LONG 	distance_flag = 0, n_pol = 0;
+	GMT_LONG	k, ij, nm, n_path, n_read, n_alloc = GMT_CHUNK;
 
 	float *data;
 
-	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, lon_sum, lat_sum, dlon;
-	double xinc2, yinc2, distance, shrink = 1.0, idx, idy, x0, y0, *in;
+	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, lon_sum, lat_sum, dlon, step;
+	double distance, shrink = 1.0, idx, idy, x0, y0, *in;
 	double xmin1, xmin2, xmax1, xmax2, lon_w;
 
 	FILE *fp;
@@ -89,11 +91,11 @@ int main (int argc, char **argv)
 	struct GMT_LINE_SEGMENT P;
 	struct GRDMASK_CTRL *Ctrl;
 	
-	void *New_Grdmask_Ctrl (), Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C);
+	void *New_grdmask_Ctrl (), Free_grdmask_Ctrl (struct GRDMASK_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDMASK_CTRL *)New_Grdmask_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDMASK_CTRL *)New_grdmask_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	GMT_grd_init (&header, argc, argv, FALSE);
 
@@ -106,11 +108,13 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'R':
 				case 'V':
 				case ':':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
@@ -118,7 +122,11 @@ int main (int argc, char **argv)
 				/* Supplemental parameters */
 
 				case 'A':	/* Turn off draw_arc mode */
-					if (argv[i][2])
+					if (argv[i][2] == 'm')
+						Ctrl->A.mode = 1;
+					else if (argv[i][2] == 'p')
+						Ctrl->A.mode = 2;
+					else if (argv[i][2])
 						Ctrl->A.step = atof (&argv[i][2]);	/* Undocumented test feature */
 					else
 						Ctrl->A.active = TRUE;
@@ -150,9 +158,6 @@ int main (int argc, char **argv)
 					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
 					Ctrl->S.radius = GMT_getradius (&argv[i][2]);
@@ -169,22 +174,22 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdmask %s - Create mask grdfile from polygons or point coverage\n\n", GMT_VERSION);
+		fprintf (stderr, "grdmask %s - Create mask grid file from polygons or point coverage\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> %s\n", GMT_I_OPT);
-		fprintf (stderr, "\t%s [-A] [-F] [%s] [-N<out>/<edge>/<in>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
-		fprintf (stderr, "\t[%s] [-S<radius>[m|c|k|K] [-V] [%s] [%s] [%s]\n\n", GMT_Mo_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "\t%s [-A[m|p]] [-F] [%s] [-N<out>/<edge>/<in>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t[-S<radius>[m|c|k|K] [-V] [%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\txyfiles is one or more polygon [or point] files\n");
-		fprintf (stderr, "\t-G Specify file name for output mask grd file.\n");
+		fprintf (stderr, "\t-G Specify file name for output mask grid file.\n");
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A Suppress connecting points using great circle arcs for geographic data\n");
+		fprintf (stderr, "\t-A Suppress connecting points using great circle arcs, i.e. connect by straight lines\n");
+		fprintf (stderr, "\t   unless m or p is appended to first follow meridian then parallel, or vice versa.\n");
 		fprintf (stderr, "\t-F Force pixel registration for output grid [Default is gridline orientation]\n");
 		GMT_explain_option ('H');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N sets values to use if point is outside, on the path, or inside.\n");
 		fprintf (stderr, "\t   NaN is a valid entry.  Default values are 0/0/1.\n");
 		fprintf (stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
@@ -192,7 +197,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   whether they are inside a circle of specified radius [0] from the nearest data point.\n");
 		fprintf (stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf (stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is spherical then great circle distances are used.\n");
 		fprintf (stderr, "\t    [Default is to treat xyfiles as polygons and use inside/outside searching].\n");
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
@@ -200,10 +205,13 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t    [Default is 2 input columns].\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -225,32 +233,30 @@ int main (int argc, char **argv)
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
-
+	if (Ctrl->S.active && (Ctrl->S.radius <= 0.0 || GMT_is_dnan (Ctrl->S.radius))) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Radius is NaN or negative\n", GMT_program);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
 
-	header.node_offset = Ctrl->F.active;
+	header.node_offset = (int)Ctrl->F.active;
 	header.x_inc = Ctrl->I.xinc;
 	header.y_inc = Ctrl->I.yinc;
 	GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
-	GMT_grd_RI_verify (&header, 1);
+	GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 
 	header.xy_off = 0.5 * header.node_offset;
-	xinc2 = header.xy_off * header.x_inc;
-	yinc2 = header.xy_off * header.y_inc;
-	one_or_zero = 1 - header.node_offset;
 	idx = 1.0 / header.x_inc;
 	idy = 1.0 / header.y_inc;
-	nm = header.nx * header.ny;
+	nm = GMT_get_nm (header.nx, header.ny);
 	data = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
 	x = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
@@ -266,17 +272,20 @@ int main (int argc, char **argv)
 	}
 
 	if (distance_flag > 0) {
-		shrink = cos (0.5 * (header.y_min + header.y_max) * D2R);
-		di = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
-		dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.y_inc));
+		double width;
+		shrink = cosd (MAX(fabs(header.y_min), fabs(header.y_max)));
+		width = (shrink < GMT_CONV_LIMIT) ? DBL_MAX : ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
+		di = (width < (double)header.nx) ? (GMT_LONG) width : header.nx;
+		dj = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.y_inc));
 	}
 	else {
-		di = (int)ceil (Ctrl->S.radius * idx);
-		dj = (int)ceil (Ctrl->S.radius * idy);
+		di = (GMT_LONG)ceil (Ctrl->S.radius * idx);
+		dj = (GMT_LONG)ceil (Ctrl->S.radius * idy);
 	}
 	periodic = (GMT_io.in_col_type[GMT_X] == GMT_IS_LON);	/* Dealing with geographic coordinates */
 	resample = (!Ctrl->A.active && periodic);
-	greenwich = (header.x_min <= 0.0 || header.x_max <= 0.0);
+	/* Maximum step size (in degrees) used for interpolation of line segments along great circles (when requested) */
+	step = Ctrl->A.step / project_info.x_scale / project_info.M_PR_DEG;
 	if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
 
 	switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
@@ -296,9 +305,9 @@ int main (int argc, char **argv)
 	}
 
 	memset ((void *)&P, 0, sizeof (struct GMT_LINE_SEGMENT));
-	P.coord = (double **) GMT_memory (VNULL, 2, sizeof (double *), GMT_program);	/* Needed as pointers below */
-	P.min = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
-	P.max = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
+	P.coord = (double **) GMT_memory (VNULL, (size_t)2, sizeof (double *), GMT_program);	/* Needed as pointers below */
+	P.min = (double *) GMT_memory (VNULL, (size_t)2, (size_t)sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
+	P.max = (double *) GMT_memory (VNULL, (size_t)2, (size_t)sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
 	
 	/* Initialize all nodes to the 'outside' value */
 
@@ -312,6 +321,7 @@ int main (int argc, char **argv)
 
 	done = FALSE;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
+	GMT_io.skip_duplicates = TRUE;	/* The inonout algorithm assumes no duplicates */
 
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -329,7 +339,7 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		n_read = 0;
 		n_fields = GMT_input (fp,  &n_expected_fields, &in);
@@ -346,9 +356,10 @@ int main (int argc, char **argv)
 			lon_sum = lat_sum = 0.0;
 
 			while (!(GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE  */
+				n_read++;
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
-					exit (EXIT_FAILURE);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields,  n_expected_fields, n_read);
+					continue;
 				}
 				x[n_path] = in[GMT_X];
 				y[n_path] = in[GMT_Y];
@@ -370,8 +381,8 @@ int main (int argc, char **argv)
 						dlon = x[n_path] - x[n_path-1];
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 						lon_sum += dlon;
-						lat_sum += y[n_path];
 					}
+					lat_sum += y[n_path];
 				}
 				else {	/* Cartesian x */
 					if (x[n_path] > xmax) xmax = x[n_path];
@@ -380,7 +391,7 @@ int main (int argc, char **argv)
 				n_path++;
 
 				if (n_path == (n_alloc-1)) {	/* n_alloc-1 since we may need 1 more to close polygon */
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					x = (double *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof(double), GMT_program);
 					y = (double *) GMT_memory ((void *)y, (size_t)n_alloc, sizeof(double), GMT_program);
 				}
@@ -409,13 +420,8 @@ int main (int argc, char **argv)
 			if (Ctrl->S.active) {	/* Assign 'inside' to nodes within given distance of data constrains */
 				for (k = 0; k < n_path; k++) {
 
-					if (y[k] < header.y_min || y[k] > header.y_max) continue;
-					if (periodic) {	/* Make sure we are not off by 360 somewhere */
-						x[k] -= 360.0;
-						while (x[k] < header.x_min) x[k] += 360.0;
-						if (x[k] > header.x_max) continue;
-					}
-					else if (x[k] < header.x_min || x[k] > header.x_max) continue;
+					if (GMT_y_is_outside (y[k], header.y_min, header.y_max)) continue;	/* Outside y-range */
+					if (GMT_x_is_outside (&x[k], header.x_min, header.x_max)) continue;	/* Outside x-range (or longitude) */
 
 					/* OK, this point is within bounds, but may be exactly on the border */
 
@@ -430,7 +436,7 @@ int main (int argc, char **argv)
 						if (j < 0 || j >= header.ny) continue;
 						for (i = i0 - di; i <= (i0 + di); i++) {
 							if (i < 0 || i >= header.nx) continue;
-							ij = j * header.nx + i;
+							ij = GMT_IJ (j, i, header.nx);
 							x0 = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
 							y0 = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, header.xy_off, header.ny);
 							distance = (GMT_distance_func) (x[k], y[k], x0, y0);
@@ -447,7 +453,6 @@ int main (int argc, char **argv)
 					x[n_path] = x[0];
 					y[n_path] = y[0];
 					dlon = x[n_path] - x[n_path-1];
-					lat_sum += y[n_path];
 					n_path++;
 					if (periodic) {
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
@@ -455,7 +460,10 @@ int main (int argc, char **argv)
 					}
 				}
 
-				if (resample) n_path = GMT_fix_up_path (&x, &y, n_path, greenwich, Ctrl->A.step);
+				if (resample) {
+					n_path = GMT_fix_up_path (&x, &y, n_path, step, Ctrl->A.mode);
+					n_alloc = n_path;	/* Since it got reset */
+				}
 
 				if (periodic) {	/* Make polygon structure so we can use spherical polygon machinery */
 					P.coord[GMT_X] = x;
@@ -463,7 +471,7 @@ int main (int argc, char **argv)
 					P.n_rows = n_path;
 					P.min[GMT_Y] = ymin;	P.max[GMT_Y] = ymax;
 					P.min[GMT_X] = xmin;	P.max[GMT_X] = xmax;
-					if (fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
+					if (GMT_360_RANGE (lon_sum, 0.0)) {	/* Contains a pole, convert to polar x,y */
 						P.pole = (lat_sum < 0.0) ? -1 : +1;	/* S or N pole */
 					}
 					else
@@ -504,9 +512,10 @@ int main (int argc, char **argv)
 
 						/* Here, point is inside or on edge, we must assign value */
 
-						data[j*header.nx+i] = (float)Ctrl->N.mask[side];
+						ij = GMT_IJ (j, i, header.nx);
+						data[ij] = (float)Ctrl->N.mask[side];
 					}
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Polygon %d scanning row %5.5d\r", GMT_program, n_pol, j);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Polygon %ld scanning row %5.5ld\r", GMT_program, n_pol, j);
 				}
 			}
 		}
@@ -514,28 +523,28 @@ int main (int argc, char **argv)
 
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
+	GMT_io.skip_duplicates = FALSE;	/* Reset to FALSE */
 
-	if (GMT_write_grd (Ctrl->G.file, &header, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	GMT_free ((void *)data);
 	GMT_free ((void *)x);
 	GMT_free ((void *)y);
 	GMT_free ((void *)P.coord);
+	GMT_free ((void *)P.min);
+	GMT_free ((void *)P.max);
 
-	Free_Grdmask_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdmask_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDMASK_CTRL *C;
 	
-	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
+	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDMASK_CTRL), "New_grdmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->A.step = gmtdefs.line_step;
@@ -543,7 +552,7 @@ void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure *
 	return ((void *)C);
 }
 
-void Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+void Free_grdmask_Ctrl (struct GRDMASK_CTRL *C) {	/* Deallocate control structure */
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
