@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdproject.c,v 1.47 2009/01/09 04:02:34 guru Exp $
+ *	$Id: grdproject.c,v 1.54 2010/03/24 02:36:44 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -31,54 +31,54 @@
 
 struct GRDPROJECT_CTRL {
 	struct A {	/* -A[k|m|n|i|c|p] */
-		BOOLEAN active;
+		GMT_LONG active;
 		char unit;
 	} A;
 	struct C {	/* -C[<dx/dy>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double easting, northing;
 	} C;
 	struct D {	/* -Ddx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} D;
 	struct E {	/* -E<dpi> */
-		BOOLEAN active;
-		int dpi;
+		GMT_LONG active;
+		GMT_LONG dpi;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -I */
-		BOOLEAN active;
+		GMT_LONG active;
 	} I;
 	struct M {	/* -Mc|i|m */
-		BOOLEAN active;
+		GMT_LONG active;
 		char unit;
 	} M;
 	struct N {	/* -N<nx/ny> */
-		BOOLEAN active;
-		int nx, ny;
+		GMT_LONG active;
+		GMT_LONG nx, ny;
 	} N;
 	struct S {	/* -S[-]b|c|l|n[/<threshold>] */
-		BOOLEAN active;
-		BOOLEAN antialias;
-		int interpolant;
+		GMT_LONG active;
+		GMT_LONG antialias;
+		GMT_LONG interpolant;
 		double threshold;
 	} S;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, set_n = FALSE, shift_xy = FALSE, offset;
+	GMT_LONG error = FALSE, set_n = FALSE, shift_xy = FALSE, offset;
 
 	char *infile, format[BUFSIZ], unit_name[GRD_UNIT_LEN], scale_unit_name[GRD_UNIT_LEN];
 
-	int i, j, unit = 0;
+	GMT_LONG i, j, unit = 0;
 	
 	GMT_LONG nm;
 	
@@ -91,11 +91,11 @@ int main (int argc, char **argv)
 	struct GMT_EDGEINFO edgeinfo;
 	struct GRDPROJECT_CTRL *Ctrl;
 
-	void *New_Grdproject_Ctrl (), Free_Grdproject_Ctrl (struct GRDPROJECT_CTRL *C);
+	void *New_grdproject_Ctrl (), Free_grdproject_Ctrl (struct GRDPROJECT_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDPROJECT_CTRL *)New_Grdproject_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDPROJECT_CTRL *)New_grdproject_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	infile = CNULL;
 	w = e = s = n = 0.0;
@@ -153,7 +153,7 @@ int main (int argc, char **argv)
 					Ctrl->M.unit = argv[i][2];
 					break;
 				case 'N':
-					sscanf (&argv[i][2], "%d/%d", &Ctrl->N.nx, &Ctrl->N.ny);
+					sscanf (&argv[i][2], "%" GMT_LL "d/%" GMT_LL "d", &Ctrl->N.nx, &Ctrl->N.ny);
 					if (Ctrl->N.ny == 0) Ctrl->N.ny = Ctrl->N.nx;
 					Ctrl->N.active = TRUE;
 					break;
@@ -548,17 +548,17 @@ int main (int argc, char **argv)
 	GMT_free ((void *)geo);
 	GMT_free ((void *)rect);
 
-	Free_Grdproject_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdproject_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Grdproject_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdproject_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDPROJECT_CTRL *C;
 	
-	C = (struct GRDPROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDPROJECT_CTRL), "New_Grdproject_Ctrl");
+	C = (struct GRDPROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDPROJECT_CTRL), "New_grdproject_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->S.antialias = TRUE; C->S.interpolant = BCR_BICUBIC; C->S.threshold = 0.5;
@@ -566,7 +566,7 @@ void *New_Grdproject_Ctrl () {	/* Allocate and initialize a new control structur
 	return ((void *)C);
 }
 
-void Free_Grdproject_Ctrl (struct GRDPROJECT_CTRL *C) {	/* Deallocate control structure */
+void Free_grdproject_Ctrl (struct GRDPROJECT_CTRL *C) {	/* Deallocate control structure */
 	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
