@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvector.c,v 1.35 2006/10/17 22:05:16 remko Exp $
+ *	$Id: grdvector.c,v 1.62 2010/03/22 21:51:28 jluis Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
-   grdvector reads 2 grdfiles that contains the 2 components of a vector
+   grdvector reads 2 grid files that contains the 2 components of a vector
    field (cartesian or polar) and plots vectors at the grid positions.
    This is basically a short-hand for using grd2xyz | psxy -SV and is
    more convenient for such plots on a grid.
@@ -34,53 +34,55 @@
 
 struct GRDVECTOR_CTRL {
 	struct A {	/* -A */
-		BOOLEAN active;
+		GMT_LONG active;
 	} A;
 	struct C {	/* -C<cpt> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} C;
 	struct E {	/* -E */
-		BOOLEAN active;
+		GMT_LONG active;
 	} E;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct Q {	/* -Q<params> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double width, length, thickness, norm;
 	} Q;
 	struct S {	/* -S[l]<scale>][<unit>] */
-		BOOLEAN active;
-		BOOLEAN constant;
+		GMT_LONG active;
+		GMT_LONG constant;
 		char unit;
 		double factor;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 	struct W {	/* -W<pen> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_PEN pen;
 	} W;
 	struct Z {	/* -Z */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Z;
 };
 
 int main (int argc, char **argv)
 {
-	int	i, j, n = 0, nm, nx, ny, ij, i0, j0, di, dj;
+	GMT_LONG	i, j, n = 0, nx, ny, i0, j0, di, dj;
 
-	BOOLEAN shrink_properties = FALSE, error = FALSE;
+	GMT_LONG ij, nm;
+	
+	GMT_LONG shrink_properties = FALSE, error = FALSE;
 
 	char *file[2], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT];
 
@@ -93,11 +95,11 @@ int main (int argc, char **argv)
 	struct GRD_HEADER h[2];
 	struct GRDVECTOR_CTRL *Ctrl;
 
-	void *New_Grdvector_Ctrl (), Free_Grdvector_Ctrl (struct GRDVECTOR_CTRL *C);
+	void *New_grdvector_Ctrl (), Free_grdvector_Ctrl (struct GRDVECTOR_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDVECTOR_CTRL *)New_Grdvector_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDVECTOR_CTRL *)New_grdvector_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	west = east = south = north = 0.0;
 	di = dj = 1;
@@ -157,7 +159,16 @@ int main (int argc, char **argv)
 					break;
 				case 'Q':
 					Ctrl->Q.active = TRUE;
-					if (argv[i][2] && argv[i][3] != 'n') {
+					for (j = 2; argv[i][j] && argv[i][j] != 'n'; j++);
+					if (argv[i][j]) {	/* Normalize option used */
+						Ctrl->Q.norm = GMT_convert_units (&argv[i][j+1], GMT_INCH);
+						if (Ctrl->Q.norm <= 0.0) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -Qn option:  No reference length given\n", GMT_program);
+							error++;
+						}
+						argv[i][j] = '\0';	/* Temporarily chop of the n<norm> string */
+					}
+					if (argv[i][2] && argv[i][3] != 'n') {	/* We specified the three parameters */
 						if (sscanf (&argv[i][2], "%[^/]/%[^/]/%s", txt_a, txt_b, txt_c) != 3) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -Q option:  Could not decode arrowwidth/headlength/headwidth\n", GMT_program);
 							error++;
@@ -168,18 +179,11 @@ int main (int argc, char **argv)
 							Ctrl->Q.width  = GMT_convert_units (txt_c, GMT_INCH);
 						}
 					}
-					for (j = 2; argv[i][j] && argv[i][j] != 'n'; j++);
-					if (argv[i][j]) {	/* Normalize option used */
-						Ctrl->Q.norm = atof (&argv[i][j+1]);
-						if (Ctrl->Q.norm <= 0.0) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -Qn option:  No reference length given\n", GMT_program);
-							error++;
-						}
-					}
+					if (Ctrl->Q.norm > 0.0) argv[i][j] = 'n';	/* Restore the n<norm> string */
 					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
-					j = strlen (argv[i]) - 1;
+					j = (int)strlen (argv[i]) - 1;
 					if (strchr ("cimpCIMP", (int)argv[i][j]))	/* Recognized unit character */
 						Ctrl->S.unit = argv[i][j];
 					else if (! (argv[i][j] == '.' || isdigit ((int)argv[i][j]))) {	/* Not decimal point or digit means trouble */
@@ -219,17 +223,17 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdvector %s - Plot vector fields from grdfiles\n\n", GMT_VERSION);
+		fprintf (stderr, "grdvector %s - Plot vector fields from grid files\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdvector compx.grd compy.grd %s %s [-A]\n", GMT_J_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[%s] [-C<cpt>] [-E] [-G<fill>] [-I<dx/dy>] [-K] [-N] [-O] [-P] [-Q<params>] [-S[l]<scale>[<unit>]]\n", GMT_B_OPT);
 		fprintf (stderr, "\t[-T] [%s] [-V] [-W<pen>] [%s] [%s] [-Z] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\tcompx & compy are grdfiles with the 2 vector components.\n");
+		fprintf (stderr, "\tcompx & compy are grid files with the 2 vector components.\n");
 		GMT_explain_option ('j');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A means grdfiles have polar (r, theta) components [Default is Cartesian]\n");
+		fprintf (stderr, "\t-A means grids have polar (r, theta) components [Default is Cartesian]\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Use cpt-file to assign colors based on vector length\n");
 		fprintf (stderr, "\t-E cEnter vectors on grid nodes [Default draws from grid node]\n");
@@ -242,7 +246,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Q Select vector plot [Default is stick-plot].\n");
 		fprintf (stderr, "\t   Optionally, specify vector parameters\n");
 		fprintf (stderr, "\t   <params> are arrowwidth/headlength/headwidth [Default is 0.03i/0.12i/0.09i]\n");
-		fprintf (stderr, "\t   Append n<size> which will cause vectors shorter than <size> to be\n");
+		fprintf (stderr, "\t   Append n<size>[unit] which will cause vectors shorter than <size> to be\n");
 		fprintf (stderr, "\t     scaled down\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S sets scale for vector length in data units per %s [1]\n", GMT_unit_names[gmtdefs.measure_unit]);
@@ -262,6 +266,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	if (Ctrl->I.active && (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  Must specify positive increments\n", GMT_program);
 		error++;
@@ -295,29 +301,22 @@ int main (int argc, char **argv)
 		error++;
 	}
 	if (n != 2) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify two input grdfiles\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify two input grid files\n", GMT_program);
 		error++;
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (Ctrl->C.active) GMT_read_cpt (Ctrl->C.file);
 
 	if (!(strcmp (file[0], "=") || strcmp (file[1], "="))) {
-		fprintf (stderr, "%s: Piping of grdfiles not supported!\n", GMT_program);
+		fprintf (stderr, "%s: Piping of grid files not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
-	if (GMT_read_grd_info (file[0], &h[0])) {
-		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, file[0]);
-		exit (EXIT_FAILURE);
-	}
-
-	if (GMT_read_grd_info (file[1], &h[1])) {
-		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, file[1]);
-		exit (EXIT_FAILURE);
+	for (i = 0; i < 2; i++) {
+		GMT_grd_init (&h[i], argc, argv, FALSE);
+		GMT_err_fail (GMT_read_grd_info (file[i], &h[i]), file[i]);
 	}
 
 	if (!(h[0].nx == h[1].nx && h[0].ny == h[1].ny && h[0].x_min == h[1].x_min && h[0].y_min == h[1].y_min 
@@ -335,32 +334,29 @@ int main (int argc, char **argv)
 		north = h[0].y_max;
 	}
 
-	GMT_map_setup (west, east, south, north);
+	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	/* Determine the wesn to be used to read the grdfile */
+	/* Determine the wesn to be used to read the grid file */
 
-	if (GMT_grd_setregion (&h[0], &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+	if (GMT_grd_setregion (&h[0], &data_west, &data_east, &data_south, &data_north, BCR_BILINEAR)) {
+		/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	/* Read data */
 
 	nx = GMT_get_n (data_west, data_east, h[0].x_inc, h[0].node_offset);
 	ny = GMT_get_n (data_south, data_north, h[0].y_inc, h[0].node_offset);
-	nm = nx * ny;
+	nm = GMT_get_nm (nx, ny);
 	r = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	theta = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
-	if (GMT_read_grd (file[0], &h[0], r, data_west, data_east, data_south, data_north, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error reading file %s\n", GMT_program, file[0]);
-		exit (EXIT_FAILURE);
-	}
-	if (GMT_read_grd (file[1], &h[1], theta, data_west, data_east, data_south, data_north, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error reading file %s\n", GMT_program, file[1]);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_read_grd (file[0], &h[0], r, data_west, data_east, data_south, data_north, GMT_pad, FALSE), file[0]);
+	GMT_err_fail (GMT_read_grd (file[1], &h[1], theta, data_west, data_east, data_south, data_north, GMT_pad, FALSE), file[1]);
+
 	if (!Ctrl->S.constant) Ctrl->S.factor = 1.0 / Ctrl->S.factor;
 
 	switch (Ctrl->S.unit) {	/* Adjust for possible unit selection */
@@ -413,7 +409,7 @@ int main (int argc, char **argv)
 		y = GMT_j_to_y (j, h[0].y_min, h[0].y_max, h[0].y_inc, h[0].xy_off, h[0].ny);
 		for (i = i0; i < h[1].nx; i += di) {
 
-			ij = j * h[0].nx + i;
+			ij = GMT_IJ (j, i, h[0].nx);
 			if (GMT_is_fnan (r[ij]) || GMT_is_fnan (theta[ij])) continue;	/* Cannot plot NaN-vectors */
 
 			value = r[ij];
@@ -421,7 +417,7 @@ int main (int argc, char **argv)
 			if (!Ctrl->A.active) {
 				value = hypot (theta[ij], r[ij]);
 				if (value == 0.0) continue;
-				theta[ij] = (float)(R2D * atan2 (theta[ij], r[ij]));
+				theta[ij] = (float)(atan2d (theta[ij], r[ij]));
 				r[ij] = (float)value;
 			}
 			else if (r[ij] < 0.0) {
@@ -430,7 +426,7 @@ int main (int argc, char **argv)
 			}
 			else if (r[ij] == 0.0) continue;
 
-			if (Ctrl->C.active) GMT_get_rgb24 (value, Ctrl->G.fill.rgb);
+			if (Ctrl->C.active) GMT_get_rgb_from_z (value, Ctrl->G.fill.rgb);
 
 			x = GMT_i_to_x (i, h[0].x_min, h[0].x_max, h[0].x_inc, h[0].xy_off, h[0].nx);
 			GMT_geo_to_xy (x, y, &plot_x, &plot_y);
@@ -487,17 +483,17 @@ int main (int argc, char **argv)
 	GMT_free ((void *)r);
 	GMT_free ((void *)theta);
 
-	Free_Grdvector_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdvector_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Grdvector_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdvector_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDVECTOR_CTRL *C;
 	
-	C = (struct GRDVECTOR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVECTOR_CTRL), "New_Grdvector_Ctrl");
+	C = (struct GRDVECTOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDVECTOR_CTRL), "New_grdvector_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	GMT_init_fill (&C->G.fill, -1, -1, -1);
@@ -509,7 +505,7 @@ void *New_Grdvector_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void Free_Grdvector_Ctrl (struct GRDVECTOR_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
+void Free_grdvector_Ctrl (struct GRDVECTOR_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
