@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: minmax.c,v 1.37 2006/10/29 01:38:25 pwessel Exp $
+ *    $Id: minmax.c,v 1.71 2010/06/25 20:33:49 guru Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -31,47 +31,52 @@
 struct MINMAX_CTRL {	/* All control options for this program (except common args) */
 	/* active is TRUE if the option has been activated */
 	struct C {	/* -C */
-		BOOLEAN active;
+		GMT_LONG active;
 	} C;
 	struct E {	/* -E<L|l|H|h><col> */
-		BOOLEAN active;
-		BOOLEAN abs;
-		int mode;
-		int col;
+		GMT_LONG active;
+		GMT_LONG abs;
+		GMT_LONG mode;
+		GMT_LONG col;
 	} E;
 	struct I {	/* -Idx[/dy[/<dz>..] */
-		BOOLEAN active;
-		int ncol;
-		double inc[BUFSIZ];
+		GMT_LONG active;
+		GMT_LONG ncol;
+		double inc[GMT_MAX_COLUMNS];
 	} I;
-	struct T {	/* -T<dz> */
-		BOOLEAN active;
+	struct S {	/* -S[x|y] */
+		GMT_LONG active;
+		GMT_LONG xbar, ybar;
+	} S;
+	struct T {	/* -T<dz>[/<col>] */
+		GMT_LONG active;
 		double inc;
+		GMT_LONG col;
 	} T;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN  error = FALSE, nofile = TRUE, done = FALSE, got_stuff = FALSE, first, give_r_string = FALSE;
-	BOOLEAN  dateline = FALSE, brackets = FALSE, work_on_abs_value, special = FALSE;
+	GMT_LONG  error = FALSE, nofile = TRUE, done = FALSE, got_stuff = FALSE, first, give_r_string = FALSE;
+	GMT_LONG  dateline = FALSE, brackets = FALSE, work_on_abs_value, special = FALSE, quad[4] = {FALSE, FALSE, FALSE, FALSE};
 
-	char line[BUFSIZ], file[BUFSIZ], chosen[BUFSIZ];
-
-	int n, i, j, ncol, n_files = 0, fno, n_args, n_fields, n_expected_fields;
+	char line[BUFSIZ], file[BUFSIZ], chosen[BUFSIZ], buffer[BUFSIZ], *not_used = NULL;
 
+	GMT_LONG i, j, ncol, n_files = 0, fno, n_args, n_read, n_fields, n_expected_fields, quad_no, n;
+	
 	double *xyzmin, *xyzmax, west, east, south, north, low, high, value, e_min = DBL_MAX, e_max = -DBL_MAX, *in;
-	double xmin1 = 360.0, xmin2 = 360.0, xmax1 = -360.0, xmax2 = -360.0, lon_w;
+	double xmin1 = 360.0, xmin2 = 360.0, xmax1 = -360.0, xmax2 = -360.0;
 
 	FILE *fp = NULL;
 
 	struct MINMAX_CTRL *Ctrl;
 
-	int strip_blanks_and_output (double x, int col);
-	void *New_Minmax_Ctrl (), Free_Minmax_Ctrl (struct MINMAX_CTRL *C);
+	GMT_LONG strip_blanks_and_output (double x, GMT_LONG col);
+	void *New_minmax_Ctrl (), Free_minmax_Ctrl (struct MINMAX_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct MINMAX_CTRL *) New_Minmax_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
+	Ctrl = (struct MINMAX_CTRL *) New_minmax_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
 	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -79,9 +84,11 @@ int main (int argc, char **argv)
 				/* Common parameters */
                       
 				case 'H':
+				case 'M':
 				case ':':
   				case 'b':
   				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
@@ -104,7 +111,6 @@ int main (int argc, char **argv)
 							break;
 						case 'H':
 							Ctrl->E.abs = TRUE;
-							break;
 						case 'h':
 							Ctrl->E.mode = +1;
 							break;
@@ -115,14 +121,11 @@ int main (int argc, char **argv)
 					}
 					Ctrl->E.col = atoi (&argv[i][3]);
                                         break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'I':
 					Ctrl->I.active = TRUE;
 					if (argv[i][2] == 'p') special = TRUE;
 					j = (special) ? 3 : 2;
-					Ctrl->I.ncol = GMT_getincn (&argv[i][j], Ctrl->I.inc, BUFSIZ);
+					Ctrl->I.ncol = GMT_getincn (&argv[i][j], Ctrl->I.inc, GMT_MAX_COLUMNS);
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
 					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -131,7 +134,18 @@ int main (int argc, char **argv)
 					break;
 				case 'T':
 					Ctrl->T.active = TRUE;
-					Ctrl->T.inc = atof (&argv[i][2]);
+					j = sscanf (&argv[i][2], "%lf/%" GMT_LL "d", &Ctrl->T.inc, &Ctrl->T.col);
+					if (j == 1) Ctrl->T.col = 0;
+					break;
+				case 'S':
+					Ctrl->S.active = TRUE;
+					j = 2;
+					while (argv[i][j]) {
+						if (argv[i][j] == 'x') Ctrl->S.xbar = TRUE;
+						if (argv[i][j] == 'y') Ctrl->S.ybar = TRUE;
+						j++;
+					}
+					if (j == 2) Ctrl->S.xbar = Ctrl->S.ybar = TRUE;
 					break;
 				default:
 					error = TRUE;
@@ -145,8 +159,8 @@ int main (int argc, char **argv)
 
 	if (error || GMT_give_synopsis_and_exit) {	/* Because it's ok to give no arguments */
 		fprintf (stderr, "minmax %s - Find extreme values in ASCII tables\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: minmax [files] [-C] [-E<L|l|H|h><col>] [%s] [-I[p]<dx>[/<dy>[/<dz>..]] [%s]\n", GMT_H_OPT, GMT_M_OPT);
-		fprintf (stderr, "\t[-T<dz>] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "usage: minmax [files] [-C] [-E<L|l|H|h><col>] [%s] [-I[p]<dx>[/<dy>[/<dz>..]]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-S[x][y]] [-T<dz>[/<col>]] [%s] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_m_OPT);
              
               	if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
  
@@ -160,17 +174,23 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   given determines how many columns are rounded off to the nearest multiple.\n");
 		fprintf (stderr, "\t   If only one increment is given we also use it for the second column (for backwards compatibility).\n");
 		fprintf (stderr, "\t   To override this behaviour, use -Ip<dx>.\n");
-		GMT_explain_option ('M');
+		fprintf (stderr, "\t-S adds extra space for error bars. Useful together with -I.\n");
+		fprintf (stderr, "\t   -Sx leaves space for horizontal error bar using value in third (2) column.\n");
+		fprintf (stderr, "\t   -Sy leaves space for vertical error bar using value in third (2) column.\n");
+		fprintf (stderr, "\t   -S or -Sxy leaves space for both error bars using values in third&fourth (2&3) columns.\n");
 		fprintf (stderr, "\t-T returns textstring -Tzmin/zmax/dz to nearest multiple of the given dz\n");
-		fprintf (stderr, "\t   Calculations are based on the first column only\n");
+		fprintf (stderr, "\t   Calculations are based on the first (0) column only.  Append /<col> to use another column\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
+	
+	GMT_check_lattice (&Ctrl->I.inc[0], &Ctrl->I.inc[1], NULL, &Ctrl->I.active);
 	if (Ctrl->I.active && !special && Ctrl->I.ncol == 1) {		/* Special case of dy = dx if not given */
 		Ctrl->I.inc[1] = Ctrl->I.inc[0];
 		Ctrl->I.ncol = 2;
@@ -214,11 +234,9 @@ int main (int argc, char **argv)
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (n_files > 0)
@@ -241,17 +259,17 @@ int main (int argc, char **argv)
 				strcpy (gmtdefs.output_degree_format, "-D");
 			else
 				strcpy (gmtdefs.output_degree_format, "+D");
-			GMT_geo_C_format (gmtdefs.output_degree_format, &GMT_io.geo);
+			GMT_err_fail (GMT_geo_C_format (gmtdefs.output_degree_format, &GMT_io.geo), "");
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: OUTPUT_DEGREE_FORMAT reset from D to %s to ensure east > west\n", GMT_program, gmtdefs.output_degree_format);
 		}
 		else if (!strcmp (gmtdefs.output_degree_format, "ddd:mm:ss")) {
 			strcpy (gmtdefs.output_degree_format, "ddd:mm:ssF");
-			GMT_geo_C_format (gmtdefs.output_degree_format, &GMT_io.geo);
+			GMT_err_fail (GMT_geo_C_format (gmtdefs.output_degree_format, &GMT_io.geo), "");
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: OUTPUT_DEGREE_FORMAT reset from ddd:mm:ss to %s to ensure east > west\n", GMT_program, gmtdefs.output_degree_format);
 		}
 	}
 
-	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : GMT_MAX_COLUMNS;
 
 	for (fno = 1; !done && fno < n_args; fno++) {     /* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -274,33 +292,45 @@ int main (int argc, char **argv)
 			}
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
-		n = ncol = 0;
-		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
+		n = ncol = n_read = 0;
+		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : GMT_MAX_COLUMNS;
 		first = TRUE;
 
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
 
 		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+			n_read++;
+
 			while ((GMT_io.status & GMT_IO_SEGMENT_HEADER) && !(GMT_io.status & GMT_IO_EOF)) {
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
 			}
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
-				exit (EXIT_FAILURE);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				continue;
 			}
 
 			if (first) {	/* First time, allocate # of columns */
 
 				ncol = n_expected_fields;
-				if (Ctrl->E.active && Ctrl->E.col >= ncol) {
-	  				fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Chosen column exceeds column range\n", GMT_program);
+				if (Ctrl->S.active && 2 + Ctrl->S.xbar + Ctrl->S.ybar > ncol) {
+					fprintf (stderr, "%s: Not enough columns to support the -S option\n", GMT_program);
 					exit (EXIT_FAILURE);
 				}
+				if (Ctrl->E.active && Ctrl->E.col >= ncol) {
+	  				fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Chosen column exceeds column range (0-%ld)\n", GMT_program, ncol-1);
+					exit (EXIT_FAILURE);
+				}
+				if (Ctrl->T.active && Ctrl->T.col >= ncol) {
+					fprintf (stderr, "%s: GMT SYNTAX ERROR -T option: Chosen column exceeds column range (0-%ld)\n", GMT_program, ncol-1);
+					exit (EXIT_FAILURE);
+				}
+				if (Ctrl->T.active) ncol = Ctrl->T.col+1;
+				if (give_r_string) ncol = 2;
 
 				/* Now we know # of columns, so allocate memory */
 
@@ -319,7 +349,7 @@ int main (int argc, char **argv)
 
 			/* Decode all fields and update minmax arrays */
 
-			while (dateline && in[0] > 180.0) in[0] -= 360.0;
+			while (dateline && in[GMT_X] > 180.0) in[GMT_X] -= 360.0;
 
 			if (Ctrl->E.active && !GMT_is_dnan (in[Ctrl->E.col])) {
 				value = (work_on_abs_value) ? fabs (in[Ctrl->E.col]) : in[Ctrl->E.col];
@@ -336,16 +366,22 @@ int main (int argc, char **argv)
 				for (i = 0; i < ncol; i++) {
 					if (GMT_is_dnan (in[i])) continue;
 					if (i == 0 && GMT_io.in_col_type[i] == GMT_IS_LON) {
-						lon_w = in[i];
-						if (in[i] < 0.0) in[i] += 360.0;	/* Start off with everything in 0-360 range */
-						if (in[i] < 180.0) {
-							xmin1 = MIN (in[i], xmin1);
-							xmax1 = MAX (in[i], xmax1);
-						}
-						else {
-							xmin2 = MIN (lon_w, xmin2);
-							xmax2 = MAX (lon_w, xmax2);
-						}
+						while (in[i] < 0.0) in[i] += 360.0;	/* Start off with everything in 0-360 range */
+						xmin1 = MIN (in[i], xmin1);
+						xmax1 = MAX (in[i], xmax1);
+						quad_no = (int)floor (in[i]/90.0);	/* Yields quadrants 0-3 */
+						if (quad_no == 4) quad_no = 0;		/* When in[i] == 360.0 */
+						quad[quad_no] = TRUE;
+						while (in[i] > 180.0) in[i] -= 360.0;	/* Switch to -180+/180 range */
+						xmin2 = MIN (in[i], xmin2);
+						xmax2 = MAX (in[i], xmax2);
+					}
+					else if ((i == 0 && Ctrl->S.xbar) || (i == 1 && Ctrl->S.ybar)) {
+						/* Add/subtract value from error bar column */
+						j = (i == 1 && Ctrl->S.xbar) ? 3 : 2;
+						value = fabs(in[j]);
+						if (in[i] - value < xyzmin[i]) xyzmin[i] = in[i] - value;
+						if (in[i] + value > xyzmax[i]) xyzmax[i] = in[i] + value;
 					}
 					else {
 						if (in[i] < xyzmin[i]) xyzmin[i] = in[i];
@@ -363,34 +399,49 @@ int main (int argc, char **argv)
 		if (!got_stuff) got_stuff = (n > 0);	/* We were able to open and read at least 1 record from a file */
 
 		if (GMT_io.in_col_type[0] == GMT_IS_LON) {	/* Now determine longitude range */
-			if (xmin1 == 360.0) {	/* Only negative longitudes found */
+			int n_quad;
+			n_quad = (int)(quad[0] + quad[1] + quad[2] + quad[3]);	/* How many quadrants had data */
+			if (quad[0] && quad[3]) {	/* Longitudes on either side of Greenwich only, must use -180/+180 notation */
 				xyzmin[0] = xmin2;
 				xyzmax[0] = xmax2;
+				GMT_io.geo.range = 2;	/* Override this setting explicitly */
 			}
-			else if (xmin2 == 360.0) {	/* Only positive longitudes found */
+			else if (quad[1] && quad[2]) {	/* Longitudes on either side of the date line, must user 0/360 notation */
 				xyzmin[0] = xmin1;
 				xyzmax[0] = xmax1;
+				GMT_io.geo.range = 0;	/* Override this setting explicitly */
 			}
-			else if ((xmin1 - xmax2) < 90.0) {	/* Crossed Greenwich */
-				xyzmin[0] = xmin2;
-				xyzmax[0] = xmax1;
+			else if (n_quad == 2 && ((quad[0] && quad[2]) || (quad[1] && quad[3]))) {	/* Funny quadrant gap, pick shortest longitude extent */
+				if ((xmax1 - xmin1) < (xmax2 - xmin2)) {	/* 0/360 more compact */
+					xyzmin[0] = xmin1;
+					xyzmax[0] = xmax1;
+					GMT_io.geo.range = 0;	/* Override this setting explicitly */
+				}
+				else {						/* -180/+180 more compact */
+					xyzmin[0] = xmin2;
+					xyzmax[0] = xmax2;
+					GMT_io.geo.range = 2;	/* Override this setting explicitly */
+				}
 			}
-			else {					/* Crossed Dateline */
+			else {						/* Either will do, use default settings */
 				xyzmin[0] = xmin1;
-				xyzmax[0] = xmax2;
+				xyzmax[0] = xmax1;
 			}
 			if (xyzmin[0] > xyzmax[0]) xyzmin[0] -= 360.0;
 			if (xyzmin[0] < 0.0 && xyzmax[0] < 0.0) xyzmin[0] += 360.0, xyzmax[0] += 360.0;
 		}
 		if (give_r_string && got_stuff) {
-			west  = MIN (west, xyzmin[0]);		east  = MAX (east, xyzmax[0]);
-			south = MIN (south, xyzmin[1]);		north = MAX (north, xyzmax[1]);
+			west  = MIN (west, xyzmin[0]);
+			east  = MAX (east, xyzmax[0]);
+			south = MIN (south, xyzmin[1]);
+			north = MAX (north, xyzmax[1]);
 		}
 		else if (Ctrl->T.active && got_stuff) {
-			west  = MIN (west, xyzmin[0]);		east  = MAX (east, xyzmax[0]);
+			west  = MIN (west, xyzmin[Ctrl->T.col]);
+			east  = MAX (east, xyzmax[Ctrl->T.col]);
 		}
 		else if (!Ctrl->E.active && n > 0) {
-			if (!Ctrl->C.active) fprintf (GMT_stdout, "%s: N = %d\t", file, n);
+			if (!Ctrl->C.active) {sprintf (buffer, "%s: N = %ld\t", file, n);	GMT_fputs (buffer, GMT_stdout);}
 			for (i = 0; i < ncol; i++) {
 				if (xyzmin[i] == DBL_MAX) {
 					low = high = GMT_d_NaN;
@@ -403,36 +454,39 @@ int main (int argc, char **argv)
 					low = xyzmin[i];
 					high = xyzmax[i];
 				}
-				if (brackets) fputc ('<', GMT_stdout);
+				if (brackets) GMT_fputs ("<", GMT_stdout);
 				GMT_ascii_output_one (GMT_stdout, low, i);
-				(Ctrl->C.active) ? fputc ('\t', GMT_stdout) : fputc ('/', GMT_stdout);
+				(Ctrl->C.active) ? GMT_fputs ("\t", GMT_stdout) : GMT_fputs ("/", GMT_stdout);
 				GMT_ascii_output_one (GMT_stdout, high, i);
-				if (brackets) fputc ('>', GMT_stdout);
-				if (i < (ncol - 1)) fputc ('\t', GMT_stdout);
+				if (brackets) GMT_fputs (">", GMT_stdout);
+				if (i < (ncol - 1)) GMT_fputs ("\t", GMT_stdout);
 			}
-			fprintf (GMT_stdout, "\n");
+			GMT_fputs ("\n", GMT_stdout);
 		}
 	}
 	if (got_stuff) {
 		if (give_r_string) {
-			west  = floor (west / Ctrl->I.inc[0]) * Ctrl->I.inc[0];	east  = ceil (east / Ctrl->I.inc[0]) * Ctrl->I.inc[0];
-			south = floor (south / Ctrl->I.inc[1]) * Ctrl->I.inc[1];	north = ceil (north / Ctrl->I.inc[1]) * Ctrl->I.inc[1];
+			west  = floor (west / Ctrl->I.inc[0]) * Ctrl->I.inc[0];
+			east  = ceil (east / Ctrl->I.inc[0]) * Ctrl->I.inc[0];
+			south = floor (south / Ctrl->I.inc[1]) * Ctrl->I.inc[1];
+			north = ceil (north / Ctrl->I.inc[1]) * Ctrl->I.inc[1];
 			if (east < west) east += 360.0;
-			fprintf (GMT_stdout, "-R");
-			strip_blanks_and_output (west, 0);	fputc ('/', GMT_stdout);
-			strip_blanks_and_output (east, 0);	fputc ('/', GMT_stdout);
-			strip_blanks_and_output (south, 1);	fputc ('/', GMT_stdout);
-			strip_blanks_and_output (north, 1);	fputc ('\n', GMT_stdout);
+			GMT_fputs ("-R", GMT_stdout);
+			strip_blanks_and_output (west, 0);	GMT_fputs ("/", GMT_stdout);
+			strip_blanks_and_output (east, 0);	GMT_fputs ("/", GMT_stdout);
+			strip_blanks_and_output (south, 1);	GMT_fputs ("/", GMT_stdout);
+			strip_blanks_and_output (north, 1);	GMT_fputs ("\n", GMT_stdout);
 		}
 		else if (Ctrl->T.active) {	/* -T option */
-			west  = floor (west / Ctrl->T.inc) * Ctrl->T.inc;	east  = ceil (east / Ctrl->T.inc) * Ctrl->T.inc;
-			fprintf (GMT_stdout, "-T");
-			strip_blanks_and_output (west, 0);	fputc ('/', GMT_stdout);
-			strip_blanks_and_output (east, 0);	fputc ('/', GMT_stdout);
-			strip_blanks_and_output (Ctrl->T.inc, 0);	fputc ('\n', GMT_stdout);
+			west  = floor (west / Ctrl->T.inc) * Ctrl->T.inc;
+			east  = ceil (east / Ctrl->T.inc) * Ctrl->T.inc;
+			GMT_fputs ("-T", GMT_stdout);
+			strip_blanks_and_output (west, 0);	GMT_fputs ("/", GMT_stdout);
+			strip_blanks_and_output (east, 0);	GMT_fputs ("/", GMT_stdout);
+			strip_blanks_and_output (Ctrl->T.inc, 0);	GMT_fputs ("\n", GMT_stdout);
 		}
 		else if (Ctrl->E.active)
-			fprintf (GMT_stdout, "%s", chosen);
+			GMT_fputs (chosen, GMT_stdout);
 	}
 	else if (!got_stuff)
 		fprintf (stderr, "%s: No input data found!\n", GMT_program);
@@ -440,29 +494,29 @@ int main (int argc, char **argv)
 	GMT_free ((void *)xyzmin);
 	GMT_free ((void *)xyzmax);
 
-	Free_Minmax_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_minmax_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-int strip_blanks_and_output (double x, int col)
+GMT_LONG strip_blanks_and_output (double x, GMT_LONG col)
 {
 	/* Alternative to GMT_ascii_output_one that strips off leading blanks first */
 
-	int k;
+	GMT_LONG k;
 	char text[GMT_TEXT_LEN];
 
 	GMT_ascii_format_one (text, x, GMT_io.out_col_type[col]);
 	for (k = 0; text[k] && text[k] == ' '; k++);
-	return (fprintf (GMT_stdout, "%s", &text[k]));
+	return (GMT_fputs (&text[k], GMT_stdout));
 }
 
-void *New_Minmax_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_minmax_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct MINMAX_CTRL *C;
 	
-	C = (struct MINMAX_CTRL *) GMT_memory (VNULL, 1, sizeof (struct MINMAX_CTRL), "New_Minmax_Ctrl");
+	C = (struct MINMAX_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct MINMAX_CTRL), "New_minmax_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -471,6 +525,6 @@ void *New_Minmax_Ctrl () {	/* Allocate and initialize a new control structure */
 	return ((void *)C);
 }
 
-void Free_Minmax_Ctrl (struct MINMAX_CTRL *C) {	/* Deallocate control structure */
+void Free_minmax_Ctrl (struct MINMAX_CTRL *C) {	/* Deallocate control structure */
 	GMT_free ((void *)C);	
 }
