@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: nearneighbor.c,v 1.57 2009/01/09 04:02:34 guru Exp $
+ *	$Id: nearneighbor.c,v 1.71 2010/06/24 19:12:07 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -36,35 +36,35 @@
 struct NEARNEIGHBOR_CTRL {	/* All control options for this program (except common args) */
 	/* active is TRUE if the option has been activated */
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct E {	/* -E<empty> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<grdfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct L {	/* -L<flag> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char mode[4];
 	} L;
 	struct N {	/* -N<sectors> */
-		BOOLEAN active;
-		int sectors, min_sectors;
+		GMT_LONG active;
+		GMT_LONG sectors, min_sectors;
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
 	struct W {	/* -W */
-		BOOLEAN active;
+		GMT_LONG active;
 	} W;
 };
 
@@ -79,14 +79,13 @@ struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
 
 int main (int argc, char **argv)
 {
-	GMT_LONG k, ij, n, nm, n_alloc = GMT_CHUNK, n_set, n_filled, n_almost, n_none;
-	
-	int i, j, i0, j0, *di, dj, sector, n_fields;
-	int n_files = 0, n_args, fno, n_expected_fields, distance_flag = 0;
-	int max_di, actual_max_di, ii, jj, n_req, x_wrap, y_wrap;
+	GMT_LONG i, j, k, i0, j0, ij, n, nm, n_alloc = GMT_CHUNK, n_set, n_filled, n_almost, n_none;
+	GMT_LONG *di, dj, sector, n_read, n_fields, ii, jj;
+	GMT_LONG n_files = 0, n_args, fno, n_expected_fields, distance_flag = 0;
+	GMT_LONG max_di, actual_max_di, n_req, x_wrap, y_wrap;
 
-	BOOLEAN error = FALSE, done = FALSE, nofile = TRUE;
-	BOOLEAN wrap_180, replicate_x, replicate_y;
+	GMT_LONG error = FALSE, done = FALSE, nofile = TRUE;
+	GMT_LONG wrap_180, replicate_x, replicate_y;
 
 	double weight, weight_sum, grd_sum, *x0, *y0, dx, dy, delta, distance = 0.0, factor;
 	double *in, *shrink, x_left, x_right, y_top, y_bottom, xinc2, yinc2, idx, idy;
@@ -94,7 +93,7 @@ int main (int argc, char **argv)
 
 	float *grd;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], *not_used = NULL;
 
 	FILE *fp = NULL;
 
@@ -104,13 +103,13 @@ int main (int argc, char **argv)
 	struct NEARNEIGHBOR_POINT  *point;
 	struct NEARNEIGHBOR_CTRL *Ctrl;
 
-	void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, GMT_LONG id);
+	void assign_node (struct NEARNEIGHBOR_NODE **node, GMT_LONG n_sector, GMT_LONG sector, double distance, GMT_LONG id);
 	void free_node (struct NEARNEIGHBOR_NODE *node);
-	void *New_Nearneighbor_Ctrl (), Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C);
+	void *New_nearneighbor_Ctrl (), Free_nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct NEARNEIGHBOR_CTRL *) New_Nearneighbor_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct NEARNEIGHBOR_CTRL *) New_nearneighbor_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	GMT_boundcond_init (&edgeinfo);
 
@@ -169,7 +168,7 @@ int main (int argc, char **argv)
 					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
-					n = sscanf (&argv[i][2], "%d/%d", &Ctrl->N.sectors, &Ctrl->N.min_sectors);
+					n = sscanf (&argv[i][2], "%" GMT_LL "d/%" GMT_LL "d", &Ctrl->N.sectors, &Ctrl->N.min_sectors);
 					if (n < 1) Ctrl->N.sectors = 4;
 					if (n < 2) Ctrl->N.min_sectors = Ctrl->N.sectors;
 					break;
@@ -204,7 +203,7 @@ int main (int argc, char **argv)
 		fprintf(stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
 		fprintf(stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf(stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf(stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf(stderr, "\t   If the current ELLIPSOID is sperical then great circle distances are used.\n");
 		fprintf(stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr, "\t-E value to use for empty nodes [Default is NaN].\n");
 		fprintf(stderr, "\t-F Force pixel registration [Default is gridline registration].\n");
@@ -254,7 +253,7 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.active) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 	if (Ctrl->L.active && GMT_boundcond_parse (&edgeinfo, Ctrl->L.mode)) error++;
@@ -263,14 +262,14 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
 
 	header.x_inc = Ctrl->I.xinc;
 	header.y_inc = Ctrl->I.yinc;
-	header.node_offset = Ctrl->F.active;
+	header.node_offset = (int)Ctrl->F.active;
 	GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
 	GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 
@@ -296,11 +295,11 @@ int main (int argc, char **argv)
 	grid_node = (struct NEARNEIGHBOR_NODE **) GMT_memory (VNULL, (size_t)nm, sizeof (struct NEARNEIGHBOR_NODE *), GMT_program);
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 
-	di = (int *) GMT_memory (VNULL, (size_t)header.ny, sizeof (int), GMT_program);
-	shrink = (double *) GMT_memory (VNULL, (size_t)header.ny, sizeof (double), GMT_program);
+	di = (GMT_LONG *) GMT_memory (VNULL, header.ny, sizeof (GMT_LONG), GMT_program);
+	shrink = (double *) GMT_memory (VNULL, header.ny, sizeof (double), GMT_program);
 
-	x0 = (double *) GMT_memory (VNULL, (size_t)header.nx, sizeof (double), GMT_program);
-	y0 = (double *) GMT_memory (VNULL, (size_t)header.ny, sizeof (double), GMT_program);
+	x0 = (double *) GMT_memory (VNULL, header.nx, sizeof (double), GMT_program);
+	y0 = (double *) GMT_memory (VNULL, header.ny, sizeof (double), GMT_program);
 	for (i = 0; i < header.nx; i++) x0[i] = header.x_min + i * header.x_inc + xinc2;
 	for (j = 0; j < header.ny; j++) y0[j] = header.y_max - j * header.y_inc - yinc2;
 	if (distance_flag) {	/* Input data is geographical */
@@ -343,7 +342,7 @@ int main (int argc, char **argv)
 	y_top = header.y_max + dj * header.y_inc;	y_bottom = header.y_min - dj * header.y_inc;
 	x_width = header.x_max - header.x_min;		y_width = header.y_max - header.y_min;
 	half_x_width = 0.5 * x_width;			half_y_width = 0.5 * y_width;
-	n = 0;
+	n = n_read = 0;
 	replicate_x = (edgeinfo.nxp && !header.node_offset);	/* Gridline registration has duplicate column */
 	replicate_y = (edgeinfo.nyp && !header.node_offset);	/* Gridline registration has duplicate row */
 	x_wrap = header.nx - 1;			/* Add to node index to go to right column */
@@ -366,48 +365,51 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+			n_read++;
+
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
-				exit (EXIT_FAILURE);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				continue;
 			}
 
-			if (in[0] < x_left || in[0] > x_right) continue;
-			if (in[1] < y_bottom || in[1] > y_top) continue;
+			if (GMT_is_dnan (in[GMT_Z])) continue;					/* Skip if z = NaN */
+			if (GMT_y_is_outside (in[GMT_Y], y_bottom, y_top)) continue;		/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], x_left, x_right)) continue;	/* Outside x-range (or longitude) */
 
-			point[n].x = (float)in[0];
-			point[n].y = (float)in[1];
-			point[n].z = (float)in[2];
+			point[n].x = (float)in[GMT_X];
+			point[n].y = (float)in[GMT_Y];
+			point[n].z = (float)in[GMT_Z];
 			if (Ctrl->W.active) point[n].w = (float)in[3];
 
 			/* Find indices of the node closest to this data point */
 
-			i0 = GMT_x_to_i(in[0], header.x_min, header.x_inc, header.xy_off, header.nx);
-			j0 = GMT_y_to_j(in[1], header.y_min, header.y_inc, header.xy_off, header.ny);
+			i0 = GMT_x_to_i(in[GMT_X], header.x_min, header.x_inc, header.xy_off, header.nx);
+			j0 = GMT_y_to_j(in[GMT_Y], header.y_min, header.y_inc, header.xy_off, header.ny);
 
 			/* Loop over all nodes within radius of this node */
 
 			for (j = j0 - dj; j <= (j0 + dj); j++) {
 
-				jj = j;
+				jj = (int)j;
 				if (GMT_y_out_of_bounds (&jj, &header, &edgeinfo, &wrap_180)) continue;	/* Outside y-range */
 
 				for (i = i0 - di[jj]; i <= (i0 + di[jj]); i++) {
 
-					ii = i;
+					ii = (int)i;
 					if (GMT_x_out_of_bounds (&ii, &header, &edgeinfo, wrap_180)) continue;	/* Outside x-range */ 
 
 					/* Here, (ii,jj) is index of a node (k) inside the grid */
 
-					distance = (GMT_distance_func) (x0[ii], y0[jj], in[0], in[1]);
+					distance = (GMT_distance_func) (x0[ii], y0[jj], in[GMT_X], in[GMT_Y]);
 
 					if (distance > Ctrl->S.radius) continue;	/* Data constraint is too far from this node */
 
 					k = GMT_IJ (jj, ii, header.nx);
-					dx = in[0] - x0[ii];	dy = in[1] - y0[jj];
+					dx = in[GMT_X] - x0[ii];	dy = in[GMT_Y] - y0[jj];
 
 					/* Check for wrap-around in x or y.  This should only occur if the
 					   search radius is larger than 1/2 the grid width/height so that
@@ -496,9 +498,9 @@ int main (int argc, char **argv)
 			grd[ij] = (float)(grd_sum / weight_sum);
 			free_node (grid_node[ij]);
 		}
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10d\r", GMT_program, j);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10ld\r", GMT_program, j);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10d\n", GMT_program, j);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10ld\n", GMT_program, j);
 
 	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
@@ -517,14 +519,14 @@ int main (int argc, char **argv)
 	GMT_free ((void *)x0);
 	GMT_free ((void *)y0);
 
-	Free_Nearneighbor_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_nearneighbor_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-struct NEARNEIGHBOR_NODE *add_new_node(int n)
+struct NEARNEIGHBOR_NODE *add_new_node (GMT_LONG n)
 {
 	struct NEARNEIGHBOR_NODE *new;
 	
@@ -536,9 +538,9 @@ struct NEARNEIGHBOR_NODE *add_new_node(int n)
 	return (new);
 }
 
-void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, GMT_LONG id)
+void assign_node (struct NEARNEIGHBOR_NODE **node, GMT_LONG n_sector, GMT_LONG sector, double distance, GMT_LONG id)
 {
-	struct NEARNEIGHBOR_NODE *add_new_node(int n);
+	struct NEARNEIGHBOR_NODE *add_new_node(GMT_LONG n);
 
 	/* Allocates node space if not already used and updates the value if closer to node */
 
@@ -559,10 +561,10 @@ void free_node (struct NEARNEIGHBOR_NODE *node)
 	GMT_free ((void *)node);
 }
 
-void *New_Nearneighbor_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_nearneighbor_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct NEARNEIGHBOR_CTRL *C;
 
-	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_CTRL), "New_Nearneighbor_Ctrl");
+	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_CTRL), "New_nearneighbor_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.sectors = 4;
@@ -570,7 +572,7 @@ void *New_Nearneighbor_Ctrl () {	/* Allocate and initialize a new control struct
 	return ((void *)C);
 }
 
-void Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C) {	/* Deallocate control structure */
+void Free_nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C) {	/* Deallocate control structure */
 	if (C->G.file) free ((void *)C->G.file);
 	GMT_free ((void *)C);
 }
