@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psmask.c,v 1.71 2009/01/12 15:51:58 guru Exp $
+ *	$Id: psmask.c,v 1.90 2010/07/12 21:31:14 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -42,58 +42,55 @@
 
 struct PSMASK_CTRL {
 	struct C {	/* -C */
-		BOOLEAN active;
+		GMT_LONG active;
 	} C;
 	struct D {	/* -D<dumpfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
-		int min_pts;
+		GMT_LONG min_pts;
 	} D;
 	struct E {	/* -Eazim/elev */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth, elevation;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 };
 
 struct PSMASK_INFO {
-	BOOLEAN first_dump;
-	int p[5], i_off[5], j_off[5], k_off[5], offset;
+	GMT_LONG first_dump;
+	GMT_LONG p[5], i_off[5], j_off[5], k_off[5], offset;
 	unsigned int bit[32];
 };
 
 int main (int argc, char **argv)
 {
-	GMT_LONG ij, n, nm;
-	
-	int i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
-	int section, n_fields, distance_flag = 0, n_plus = 0, k;
+	GMT_LONG ij, n, nm, n_read, n_alloc;
+	GMT_LONG i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
+	GMT_LONG section, n_fields, distance_flag = 0, n_plus = 0, k;
+	GMT_LONG error = FALSE, first = TRUE, node_only;
 
-	GMT_LONG n_read, n_alloc;
-	BOOLEAN error = FALSE, first = TRUE, node_only;
-
-	char line[BUFSIZ], *grd;
+	char line[BUFSIZ], *grd, *not_used = NULL;
 
 	double *in, distance, x0, y0, x1, y1, shrink = 1.0;
 	double xinc2, yinc2, *x, *y;
@@ -105,17 +102,17 @@ int main (int argc, char **argv)
 	struct PSMASK_CTRL *Ctrl;
 
 #ifdef DEBUG
-	BOOLEAN debug = FALSE;
+	GMT_LONG debug = FALSE;
 #endif
-	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag);
-	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file);
+	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], GMT_LONG id, GMT_LONG flag);
+	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, GMT_LONG id, char *file);
 	void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e);
-	int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max);
-	void *New_Psmask_Ctrl (), Free_Psmask_Ctrl (struct PSMASK_CTRL *C);
+	GMT_LONG clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, GMT_LONG *edge, GMT_LONG first, double **x, double **y, GMT_LONG *max);
+	void *New_psmask_Ctrl (), Free_psmask_Ctrl (struct PSMASK_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PSMASK_CTRL *)New_Psmask_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct PSMASK_CTRL *)New_psmask_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	memset ((void *)&info, 0, sizeof (struct PSMASK_INFO));
 	info.first_dump = TRUE;
@@ -227,9 +224,9 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"psmask %s - Masking or clipping of 2-D data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psmask <xyz-file> %s %s %s\n", GMT_I_OPT, GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[%s] [-C] -D[-]<file>[+n<n_pts>] [%s] [-F] [-G<fill>] [%s] [-K] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_H_OPT, GMT_Mo_OPT);
+		fprintf (stderr, "\t[%s] [-C] -D[-]<file>[+n<n_pts>] [%s] [-F] [-G<fill>] [%s] [-K]\n", GMT_B_OPT, GMT_E_OPT, GMT_H_OPT);
 		fprintf (stderr, "\t[-N] [-O] [-P] [-S<radius>[k]] [-T] [%s] [-V] [%s]\n", GMT_U_OPT, GMT_X_OPT);
-		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [-n]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [%s] [-n]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_b_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -249,7 +246,6 @@ int main (int argc, char **argv)
 		GMT_fill_syntax ('G', "Select fill color/pattern [Default is no fill].");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N will invert the sense of the clipping [or tiling]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -259,7 +255,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Default is -S0, i.e., only the nearest node is considered reliable\n");
 		fprintf (stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf (stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is spherical then great circle distances are used.\n");
 		fprintf (stderr, "\t-T will paint tiles.  [Default will trace data outline]\n");
 		fprintf (stderr, "\t   If set you must also specify a color/fill with -G\n");
 		GMT_explain_option ('U');
@@ -272,6 +268,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -312,7 +309,7 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 			error++;
 		}
-		h.node_offset = Ctrl->F.active;
+		h.node_offset = (int)Ctrl->F.active;
 		h.x_inc = Ctrl->I.xinc;
 		h.y_inc = Ctrl->I.yinc;
 		h.xy_off = 0.5 * h.node_offset;
@@ -325,15 +322,24 @@ int main (int argc, char **argv)
 
 	/* Now test also if we must provide a default -J */
 	if (Ctrl->D.active && project_info.projection < 0) {		/* Is this the right way of testing it? */
-		GMT_parse_J_option ("x1");
+		GMT_parse_J_option ("x1d");
 	}
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
+	if (distance_flag) {
+		shrink = cosd (0.5 * (h.y_min + h.y_max));
+		di = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink) + GMT_CONV_LIMIT);
+		dj = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc) + GMT_CONV_LIMIT);
+	}
+	else {
+		di = irint (0.5 * Ctrl->S.radius / h.x_inc + GMT_CONV_LIMIT);
+		dj = irint (0.5 * Ctrl->S.radius / h.y_inc + GMT_CONV_LIMIT);
+	}
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	if (!Ctrl->C.active) GMT_err_fail (GMT_map_setup (h.x_min, h.x_max, h.y_min, h.y_max), "");
@@ -361,15 +367,6 @@ int main (int argc, char **argv)
 
 		/* Add GMT_CONV_LIMIT to ensure that special case radius = inc --> irint(0.5) actually rounds to 1 */
 		
-		if (distance_flag) {
-			shrink = cos (0.5 * (h.y_min + h.y_max) * D2R);
-			di = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink) + GMT_CONV_LIMIT);
-			dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc) + GMT_CONV_LIMIT);
-		}
-		else {
-			di = irint (0.5 * Ctrl->S.radius / h.x_inc + GMT_CONV_LIMIT);
-			dj = irint (0.5 * Ctrl->S.radius / h.y_inc + GMT_CONV_LIMIT);
-		}
 		if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
 		switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
 
@@ -397,7 +394,7 @@ int main (int argc, char **argv)
 #endif
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 		n_read = 0;
@@ -408,17 +405,21 @@ int main (int argc, char **argv)
 			}
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
-				exit (EXIT_FAILURE);
-			}
 			n_read++;
 
+			if (GMT_io.status & GMT_IO_MISMATCH) {
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				continue;
+			}
+
+			if (GMT_y_is_outside (in[GMT_Y], h.y_min, h.y_max)) continue;		/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], h.x_min, h.x_max)) continue;	/* Outside x-range (or longitude) */
+
 			/* Determine the node closest to the data point */
 
-			i = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			i = GMT_x_to_i (in[GMT_X], h.x_min, h.x_inc, h.xy_off, h.nx);
 			if (i < 0 || i >= h.nx) continue;
-			j = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			j = GMT_y_to_j (in[GMT_Y], h.y_min, h.y_inc, h.xy_off, h.ny);
 			if (j < 0 || j >= h.ny) continue;
 
 			if (node_only) {
@@ -472,8 +473,8 @@ int main (int argc, char **argv)
 			y = (double *) GMT_memory (VNULL, (size_t)GMT_CHUNK, sizeof (double), GMT_program);
 			n_alloc = GMT_CHUNK;
 
-			n_edges = h.ny * (int )ceil (h.nx / 16.0);
-			edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
+			n_edges = h.ny * (GMT_LONG )ceil (h.nx / 16.0);
+			edge = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_edges, sizeof (GMT_LONG), GMT_program);
 
 			GMT_map_basemap ();
 
@@ -496,8 +497,7 @@ int main (int argc, char **argv)
 			GMT_free ((void *)y);
 		}
 		else {	/* Just paint tiles */
-			int start;
-			GMT_LONG n_use, np, plot_n;
+			GMT_LONG start, n_use, np, plot_n;
 			double *grd_x, grd_y, y_bot, y_top, *xx, *yy, *xp, *yp;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling...\n", GMT_program);
 			grd_x = (double *) GMT_memory (VNULL, (size_t)h.nx, sizeof (double), GMT_program);
@@ -509,7 +509,7 @@ int main (int argc, char **argv)
 				y_top = grd_y + yinc2;
 				ij = GMT_IJ (j, 1, h.nx);
 				for (i = 0; i < h.nx; i++, ij++) {
-					if (((int)grd[ij]) == 0) continue;
+					if (((GMT_LONG)grd[ij]) == 0) continue;
 
 					np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - xinc2, grd_x[i] + xinc2, y_bot, y_top);
 					plot_n = GMT_clip_to_map (xx, yy, np, &xp, &yp);
@@ -519,21 +519,20 @@ int main (int argc, char **argv)
 					
 					if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
 
-					/* First truncate agains left border */
+						/* First truncate against left border */
 
-						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
+						GMT_n_plot = GMT_truncate (xp, yp, plot_n, start, -1);
 						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &Ctrl->G.fill, FALSE);
 
-						/* Then truncate agains right border */
+						/* Then truncate against right border */
 
-						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
+						GMT_n_plot = GMT_truncate (xp, yp, plot_n, start, +1);
 						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &Ctrl->G.fill, FALSE);
 					}
-					else {
+					else
 						GMT_fill (xp, yp, plot_n, &Ctrl->G.fill, FALSE);
-					}
 					GMT_free ((void *)xp);
 					GMT_free ((void *)yp);
 				}
@@ -559,14 +558,14 @@ int main (int argc, char **argv)
 
 	GMT_plotend ();
 
-	Free_Psmask_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_psmask_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag)
+void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], GMT_LONG id, GMT_LONG flag)
 {
 	GMT_LONG i;
 	double x, y;
@@ -583,17 +582,17 @@ void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id,
 	if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, nn);
 
 	if (nn > 0) {
-		sprintf (comment, "Start of clip path sub-segment %d", id);
+		sprintf (comment, "Start of clip path sub-segment %ld", id);
 		ps_comment (comment);
 	}
 	ps_clipon (xx, yy, nn, rgb, flag);
 	if (nn > 0) {
-		sprintf (comment, "End of clip path sub-segment %d", id);
+		sprintf (comment, "End of clip path sub-segment %ld", id);
 		ps_comment (comment);
 	}
 }
 
-void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file)
+void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, GMT_LONG id, char *file)
 {
 	GMT_LONG i;
 	double out[2];
@@ -618,36 +617,36 @@ void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_L
 	}
 	else {
 		if (GMT_io.binary[GMT_OUT])
-			sprintf (fname, "%s_%d.b", file, id);
+			sprintf (fname, "%s_%ld.b", file, id);
 		else
-			sprintf (fname, "%s_%d.xy", file, id);
+			sprintf (fname, "%s_%ld.xy", file, id);
 		if ((fp = GMT_fopen (fname, GMT_io.w_mode)) == NULL) {
 			fprintf (stderr, "%s: Unable to create file %s - exiting\n", GMT_program, fname);
 			exit (EXIT_FAILURE);
 		}
 	}
 	for (i = 0; i < nn; i++) {
-		out[0] = xx[i];	out[1] = yy[i];
+		out[GMT_X] = xx[i];	out[GMT_Y] = yy[i];
 		GMT_output (fp, 2, out);
 	}
 	GMT_fclose (fp);
 }
 
-int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max)
+GMT_LONG clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, GMT_LONG *edge, GMT_LONG first, double **x, double **y, GMT_LONG *max)
 {
 	/* The routine finds the zero-contour in the grd dataset.  it assumes that
 	 * no node has a value exactly == 0.0.  If more than max points are found
 	 * trace_clip_contours will try to allocate more memory in blocks of GMT_CHUNK points
 	 */
 	 
-	static int i0, j0, side;
+	static GMT_LONG i0, j0, side;
 	GMT_LONG ij;
-	int i, j, n = 0, n_edges, edge_word, edge_bit;
-	BOOLEAN go_on = TRUE;
-	int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max);
+	GMT_LONG i, j, n = 0, n_edges, edge_word, edge_bit;
+	GMT_LONG go_on = TRUE;
+	GMT_LONG trace_clip_contours (struct PSMASK_INFO *info, char *grd, GMT_LONG *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, GMT_LONG i, GMT_LONG j, GMT_LONG kk, GMT_LONG *max);
 	 
 	 
-	n_edges = h->ny * (int) ceil (h->nx / 16.0);
+	n_edges = h->ny * (GMT_LONG) ceil (h->nx / 16.0);
 	 
 	 /* Reset edge-flags to zero, if necessary */
 	 if (first) {
@@ -668,8 +667,8 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 		for (j = j0; go_on && j < h->ny; j++) {
 			ij = GMT_IJ (j, i0, h->nx);
 			for (i = i0; go_on && i < h->nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
-				edge_word = ij / 32 + info->offset;
-				edge_bit = ij % 32;
+				edge_word = (GMT_LONG)(ij / 32 + info->offset);
+				edge_bit = (GMT_LONG)(ij % 32);
 				if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij-h->nx]) == 1)) { /* Start tracing contour */
 					*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 					*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
@@ -692,8 +691,8 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 		for (j = j0; go_on && j < h->ny; j++) {
 			ij = GMT_IJ (j, i0, h->nx);
 			for (i = i0; go_on && i < h->nx-1; i++, ij++) {
-				edge_word = ij / 32 + info->offset;
-				edge_bit = ij % 32;
+				edge_word = (GMT_LONG)(ij / 32 + info->offset);
+				edge_bit = (GMT_LONG)(ij % 32);
 				if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij+1]) == 1)) { /* Start tracing contour */
 					*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 					*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
@@ -711,11 +710,11 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 	return (n);
 }
 
-int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max)
+GMT_LONG trace_clip_contours (struct PSMASK_INFO *info, char *grd, GMT_LONG *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, GMT_LONG i, GMT_LONG j, GMT_LONG kk, GMT_LONG *max)
 {
-	int n = 1, k, k0, n_cuts, kk_opposite, first_k, more;
-	int edge_word, edge_bit, m;
-	GMT_LONG ij, ij0;
+	GMT_LONG n = 1, k, k0, n_cuts, kk_opposite, first_k, more;
+	GMT_LONG edge_word, edge_bit;
+	GMT_LONG ij, ij0, m;
 	double xk[4], yk[4], x0, y0;
 
 	m = *max - 2;
@@ -737,8 +736,8 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 			/* Skip edge already has been used */
 
 			ij0 = GMT_IJ (j + info->j_off[k], i + info->i_off[k], h->nx);
-			edge_word = ij0 / 32 + info->k_off[k] * info->offset;
-			edge_bit = ij0 % 32;
+			edge_word = (GMT_LONG)(ij0 / 32 + info->k_off[k] * info->offset);
+			edge_bit = (GMT_LONG)(ij0 % 32);
 			if (edge[edge_word] & info->bit[edge_bit]) continue;
 
 			/* Skip if no zero-crossing on this edge */
@@ -820,8 +819,8 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 		}
 		if (more) {	/* Mark this edge as used */
 			ij0 = GMT_IJ (j + info->j_off[kk], i + info->i_off[kk], h->nx);
-			edge_word = ij0 / 32 + info->k_off[kk] * info->offset;
-			edge_bit = ij0 % 32;
+			edge_word = (GMT_LONG)(ij0 / 32 + info->k_off[kk] * info->offset);
+			edge_bit = (GMT_LONG)(ij0 % 32);
 			edge[edge_word] |= info->bit[edge_bit];
 		}
 
@@ -852,10 +851,10 @@ void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e)
 	}
 }
 
-void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSMASK_CTRL *C;
 	
-	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSMASK_CTRL), "New_Psmask_Ctrl");
+	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSMASK_CTRL), "New_psmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -865,7 +864,7 @@ void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	return ((void *)C);
 }
 
-void Free_Psmask_Ctrl (struct PSMASK_CTRL *C) {	/* Deallocate control structure */
+void Free_psmask_Ctrl (struct PSMASK_CTRL *C) {	/* Deallocate control structure */
 	if (C->D.file) free ((void *)C->D.file);	
 	GMT_free ((void *)C);	
 }
