@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: surface.c,v 1.59 2009/01/09 20:50:17 guru Exp $
+ *	$Id: surface.c,v 1.73 2010/06/24 19:12:07 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -88,45 +88,45 @@
 
 struct SURFACE_CTRL {
 	struct A {	/* -A<aspect_ratio> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} A;
 	struct C {	/* -C<converge_limit> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} C;
 	struct G {	/* -G<file> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct L {	/* -Ll|u<limit> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *low, *high;
 		double min, max;
-		int lmode, hmode;
+		GMT_LONG lmode, hmode;
 	} L;
 	struct N {	/* -N<max_iterations> */
-		BOOLEAN active;
-		int value;
+		GMT_LONG active;
+		GMT_LONG value;
 	} N;
 	struct Q {	/* -Q */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Q;
 	struct S {	/* -S<radius>[m|c] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
 	struct T {	/* -T<tension>[i][b] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double b_tension, i_tension;
 	} T;
 	struct Z {	/* -Z<over_relaxation_parameter> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} Z;
 };
@@ -156,29 +156,29 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 					 * I means just interpolate from larger grid */
 	char format[BUFSIZ];
 	char *low_file, *high_file;	/* Pointers to grids with low and high limits, if selected */
-	int grid, old_grid;		/* Node spacings  */
-	int n_fact;			/* Number of factors in common (ny-1, nx-1) */
-	int factors[32];		/* Array of common factors */
-	int long_verbose;
-	int set_low;			/* 0 unconstrained,1 = by min data value, 2 = by user value */
-	int set_high;			/* 0 unconstrained,1 = by max data value, 2 = by user value */
-	size_t n_alloc;
-	GMT_LONG npoints;			/* Number of data points */
+	GMT_LONG grid, old_grid;	/* Node spacings  */
+	GMT_LONG n_fact;		/* Number of factors in common (ny-1, nx-1) */
+	GMT_LONG factors[32];		/* Array of common factors */
+	GMT_LONG long_verbose;
+	GMT_LONG set_low;		/* 0 unconstrained,1 = by min data value, 2 = by user value */
+	GMT_LONG set_high;		/* 0 unconstrained,1 = by max data value, 2 = by user value */
+	GMT_LONG n_alloc;
+	GMT_LONG npoints;		/* Number of data points */
 	GMT_LONG ij_sw_corner, ij_se_corner,ij_nw_corner, ij_ne_corner;
-	GMT_LONG n_empty;			/* No of unconstrained nodes at initialization  */
-	GMT_LONG nx;				/* Number of nodes in x-dir. */
-	GMT_LONG ny;				/* Number of nodes in y-dir. (Final grid) */
+	GMT_LONG n_empty;		/* No of unconstrained nodes at initialization  */
+	GMT_LONG nx;			/* Number of nodes in x-dir. */
+	GMT_LONG ny;			/* Number of nodes in y-dir. (Final grid) */
 	GMT_LONG nxny;			/* Total number of grid nodes without boundaries  */
 	GMT_LONG mx;
 	GMT_LONG my;
 	GMT_LONG mxmy;			/* Total number of grid nodes with boundaries  */
-	GMT_LONG block_nx;			/* Number of nodes in x-dir for a given grid factor */
-	GMT_LONG block_ny;			/* Number of nodes in y-dir for a given grid factor */
-	GMT_LONG max_iterations;		/* Max iter per call to iterate */
+	GMT_LONG block_nx;		/* Number of nodes in x-dir for a given grid factor */
+	GMT_LONG block_ny;		/* Number of nodes in y-dir for a given grid factor */
+	GMT_LONG max_iterations;	/* Max iter per call to iterate */
 	GMT_LONG total_iterations;
 	GMT_LONG grid_east;
-	GMT_LONG offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
-	BOOLEAN constrained;		/* TRUE if set_low or set_high is TRUE */
+	GMT_LONG offset[25][12];	/* Indices of 12 nearby points in 25 cases of edge conditions  */
+	GMT_LONG constrained;		/* TRUE if set_low or set_high is TRUE */
 	float *lower, *upper;		/* arrays for minmax values, if set */
 	float *u;			/* Pointer to grid array */
 	double low_limit, high_limit;	/* Constrains on range of solution */
@@ -207,34 +207,34 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 };
 
 struct SURFACE_SUGGESTION {	/* Used to find top ten list of faster grid dimensions  */
-	int nx;
-	int ny;
+	GMT_LONG nx;
+	GMT_LONG ny;
 	double factor;	/* Speed up by a factor of factor  */
 };
 
 int compare_points (const void *point_1v, const void *point_2v);
 GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b);	/* Finds the greatest common divisor  */
-int get_prime_factors (GMT_LONG n, int *f);
-int iterate (struct SURFACE_INFO *C, int mode);
-void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform);
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f);
+GMT_LONG iterate (struct SURFACE_INFO *C, GMT_LONG mode);
+void load_constraints (struct SURFACE_INFO *C, GMT_LONG transform);
 
 int main (int argc, char **argv)
 {
 	char	modifier;
 
-	int	i, error = FALSE, n_files = 0, key;
+	GMT_LONG	i, error = FALSE, n_files = 0, key;
 
 	FILE	*fp_in = NULL;	/* File pointer  */
 	
 	struct SURFACE_INFO C;
 	struct SURFACE_CTRL *Ctrl;
 	
-	void suggest_sizes_for_surface (int factors[], int nx, int ny);
+	void suggest_sizes_for_surface (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny);
 	void set_grid_parameters (struct SURFACE_INFO *C);
 	void read_data (FILE *fp, struct SURFACE_INFO *C);
 	void throw_away_unusables (struct SURFACE_INFO *C);
 	void remove_planar_trend (struct SURFACE_INFO *C);
-	int rescale_z_values (struct SURFACE_INFO *C);
+	GMT_LONG rescale_z_values (struct SURFACE_INFO *C);
 	void smart_divide (struct SURFACE_INFO *C);
 	void set_offset (struct SURFACE_INFO *C);
 	void set_index (struct SURFACE_INFO *C);
@@ -246,11 +246,11 @@ int main (int argc, char **argv)
 	void replace_planar_trend (struct SURFACE_INFO *C);
 	void write_output (struct SURFACE_INFO *C, char *grdfile);
 	void load_parameters (struct SURFACE_INFO *C, struct SURFACE_CTRL *Ctrl);
-	void *New_Surface_Ctrl (), Free_Surface_Ctrl (struct SURFACE_CTRL *C);
+	void *New_surface_Ctrl (), Free_surface_Ctrl (struct SURFACE_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
 
-	Ctrl = (struct SURFACE_CTRL *)New_Surface_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct SURFACE_CTRL *)New_surface_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	memset ((void *)&C, 0, sizeof (struct SURFACE_INFO));
 	memset ((void *)&GMT_Surface_Global, 0, sizeof (struct SURFACE_GLOBAL));
@@ -486,7 +486,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	load_parameters (&C, Ctrl);	/* Pass parameters from parsing control to surface INFO structure */
@@ -514,7 +514,7 @@ int main (int argc, char **argv)
 	GMT_Surface_Global.y_min = C.h.y_min;
 
 	/* New stuff here for v4.3:  Check out the grid dimensions:  */
-	C.grid = gcd_euclid (C.nx-1, C.ny-1);
+	C.grid = (int)gcd_euclid (C.nx-1, C.ny-1);
 
 	if (gmtdefs.verbose || Ctrl->Q.active) {
 		sprintf (C.format, "%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
@@ -548,9 +548,7 @@ int main (int argc, char **argv)
 		replace_planar_trend (&C);
 		write_output (&C, Ctrl->G.file);
 		GMT_free ((void *) C.u);
-		if (C.set_low) GMT_free ((void *)C.lower);
-		if (C.set_high) GMT_free ((void *)C.upper);
-		Free_Surface_Ctrl (Ctrl);	/* Deallocate control structure */
+		Free_surface_Ctrl (Ctrl);	/* Deallocate control structure */
 
 		GMT_end (argc, argv);
 
@@ -561,7 +559,7 @@ int main (int argc, char **argv)
 
 	/* Set up factors and reset grid to first value  */
 
-	C.grid = gcd_euclid (C.nx-1, C.ny-1);
+	C.grid = (int)gcd_euclid (C.nx-1, C.ny-1);
 	C.n_fact = get_prime_factors ((GMT_LONG)C.grid, C.factors);
 	set_grid_parameters (&C);
 	while (C.block_nx < 4 || C.block_ny < 4) {
@@ -614,7 +612,7 @@ int main (int argc, char **argv)
 
 	GMT_free ((void *) C.u);
 
-	Free_Surface_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_surface_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
@@ -667,7 +665,7 @@ void set_coefficients (struct SURFACE_INFO *C)
 void set_offset (struct SURFACE_INFO *C)
 {
 	GMT_LONG	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
-	int	i, j, kase;
+	GMT_LONG	i, j, kase;
 
 	add_w[0] = -C->my; add_w[1] = add_w[2] = add_w[3] = add_w[4] = -C->grid_east;
 	add_w2[0] = -2 * C->my;  add_w2[1] = -C->my - C->grid_east;  add_w2[2] = add_w2[3] = add_w2[4] = -2 * C->grid_east;
@@ -1043,41 +1041,48 @@ void new_initialize_grid (struct SURFACE_INFO *C)
 void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 {
 	GMT_LONG	i, j, k, kmax = 0, kmin = 0;
-	int n_fields, n_expected_fields;
-	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX;
-	char	buffer[BUFSIZ];
+	GMT_LONG n_read, n_fields, n_expected_fields;
+	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX, wesn_lim[4];
+	char	buffer[BUFSIZ], *not_used = NULL;
 
 	C->data = (struct SURFACE_DATA *) GMT_memory (VNULL, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
 
 	/* Read in xyz data and computes index no and store it in a structure */
 
-	k = 0;
+	n_read = k = 0;
 	C->z_mean = 0;
+	/* Initially allow points to be within 1 grid spacing of the grid */
+	wesn_lim[0] = C->h.x_min - C->grid_xinc;	wesn_lim[1] = C->h.x_max + C->grid_xinc;
+	wesn_lim[2] = C->h.y_min - C->grid_yinc;	wesn_lim[3] = C->h.y_max + C->grid_yinc;
+	
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
-	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
+	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp_in);
 
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+		n_read++;
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, k);
-			exit (EXIT_FAILURE);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+			continue;
 		}
 
-		if (GMT_is_dnan (in[2])) continue;
+		if (GMT_is_dnan (in[GMT_Z])) continue;
+		if (GMT_y_is_outside (in[GMT_Y], wesn_lim[2], wesn_lim[3])) continue;	/* Outside y-range */
+		if (GMT_x_is_outside (&in[GMT_X], wesn_lim[0], wesn_lim[1])) continue;	/* Outside x-range (or longitude) */
 
-		i = (GMT_LONG)floor(((in[0]-C->h.x_min)*C->r_grid_xinc) + 0.5);
+		i = (GMT_LONG)floor(((in[GMT_X]-C->h.x_min)*C->r_grid_xinc) + 0.5);
 		if (i < 0 || i >= C->block_nx) continue;
-		j = (GMT_LONG)floor(((in[1]-C->h.y_min)*C->r_grid_yinc) + 0.5);
+		j = (GMT_LONG)floor(((in[GMT_Y]-C->h.y_min)*C->r_grid_yinc) + 0.5);
 		if (j < 0 || j >= C->block_ny) continue;
 
 		C->data[k].index = i * C->block_ny + j;
-		C->data[k].x = (float)in[0];
-		C->data[k].y = (float)in[1];
-		C->data[k].z = (float)in[2];
-		if (zmin > in[2]) zmin = in[2], kmin = k;
-		if (zmax < in[2]) zmax = in[2], kmax = k;
+		C->data[k].x = (float)in[GMT_X];
+		C->data[k].y = (float)in[GMT_Y];
+		C->data[k].z = (float)in[GMT_Z];
+		if (zmin > in[GMT_Z]) zmin = in[GMT_Z], kmin = k;
+		if (zmax < in[GMT_Z]) zmax = in[GMT_Z], kmax = k;
 		k++;
-		C->z_mean += in[2];
+		C->z_mean += in[GMT_Z];
 		if (k == (GMT_LONG)C->n_alloc) {
 			C->n_alloc <<= 1;
 			C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
@@ -1145,7 +1150,7 @@ void write_output (struct SURFACE_INFO *C, char *grdfile)
 	if (C->set_high) GMT_free ((void *)C->upper);
 }
 
-int iterate (struct SURFACE_INFO *C, int mode)
+GMT_LONG iterate (struct SURFACE_INFO *C, GMT_LONG mode)
 {
 
 	GMT_LONG	i, j, k, ij, kase, briggs_index, ij_v2;
@@ -1386,7 +1391,7 @@ int iterate (struct SURFACE_INFO *C, int mode)
 	if (gmtdefs.verbose && !C->long_verbose) fprintf (stderr, C->format,
 		C->grid, C->mode_type[mode], iteration_count, max_change, current_limit, C->total_iterations);
 
-	return (iteration_count);
+	return ((int)iteration_count);
 }
 
 void check_errors (struct SURFACE_INFO *C) {
@@ -1631,18 +1636,19 @@ void throw_away_unusables (struct SURFACE_INFO *C)
 			last_index = C->data[k].index;
 		}
 	}
-	/* Sort again; this time the SURFACE_OUTSIDE points will be thrown away  */
-
-	qsort ((void *)C->data, (size_t)C->npoints, sizeof (struct SURFACE_DATA), compare_points);
-	C->npoints -= n_outside;
-	C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->npoints, sizeof(struct SURFACE_DATA), GMT_program);
-	if (gmtdefs.verbose && (n_outside)) {
-		fprintf (stderr, "%s: %ld unusable points were supplied; these will be ignored.\n", GMT_program, n_outside);
-		fprintf (stderr, "%s: You should have pre-processed the data with block-mean, -median, or -mode.\n", GMT_program);
+	
+	if (n_outside) {	/* Sort again; this time the SURFACE_OUTSIDE points will be thrown away  */
+		qsort ((void *)C->data, (size_t)C->npoints, sizeof (struct SURFACE_DATA), compare_points);
+		C->npoints -= n_outside;
+		C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->npoints, sizeof(struct SURFACE_DATA), GMT_program);
+		if (gmtdefs.verbose) {
+			fprintf (stderr, "%s: %ld unusable points were supplied; these will be ignored.\n", GMT_program, n_outside);
+			fprintf (stderr, "%s: You should have pre-processed the data with block-mean, -median, or -mode.\n", GMT_program);
+		}
 	}
 }
 
-int rescale_z_values (struct SURFACE_INFO *C)
+GMT_LONG rescale_z_values (struct SURFACE_INFO *C)
 {
 	GMT_LONG	i;
 	double	ssz = 0.0;
@@ -1667,7 +1673,7 @@ int rescale_z_values (struct SURFACE_INFO *C)
 	return (0);
 }
 
-void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
+void load_constraints (struct SURFACE_INFO *C, GMT_LONG transform)
 {
 	GMT_LONG i, j, ij;
 	double yy;
@@ -1689,7 +1695,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		}
 		if (transform) {
 			for (j = ij = 0; j < C->ny; j++) {
-				yy = C->ny - j - 1;
+				yy = (double)(C->ny - j - 1);
 				for (i = 0; i < C->nx; i++, ij++) {
 					if (GMT_is_fnan (C->lower[ij])) continue;
 					C->lower[ij] -= (float)(C->plane_c0 + C->plane_c1 * i + C->plane_c2 * yy);
@@ -1713,7 +1719,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		}
 		if (transform) {
 			for (j = ij = 0; j < C->ny; j++) {
-				yy = C->ny - j - 1;
+				yy = (double)(C->ny - j - 1);
 				for (i = 0; i < C->nx; i++, ij++) {
 					if (GMT_is_fnan (C->upper[ij])) continue;
 					C->upper[ij] -= (float)(C->plane_c0 + C->plane_c1 * i + C->plane_c2 * yy);
@@ -1725,7 +1731,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 	}
 }
 
-double guess_surface_time (int factors[], int nx, int ny)
+double guess_surface_time (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny)
 {
 	/* Routine to guess a number proportional to the operations
 	 * required by surface working on a user-desired grid of
@@ -1753,9 +1759,9 @@ double guess_surface_time (int factors[], int nx, int ny)
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
 	GMT_LONG	gcd;		/* Current value of the gcd  */
-	int	nxg, nyg;	/* Current value of the grid dimensions  */
-	int	nfactors = 0;	/* Number of prime factors of current gcd  */
-	int	factor;		/* Currently used factor  */
+	GMT_LONG	nxg, nyg;	/* Current value of the grid dimensions  */
+	GMT_LONG	nfactors = 0;	/* Number of prime factors of current gcd  */
+	GMT_LONG	factor;		/* Currently used factor  */
 	/* Doubles are used below, even though the values will be integers,
 		because the multiplications might reach sizes of O(n**3)  */
 	double	t_sum;		/* Sum of values of T at each grid cycle  */
@@ -1765,8 +1771,8 @@ double guess_surface_time (int factors[], int nx, int ny)
 	gcd = gcd_euclid((GMT_LONG)nx, (GMT_LONG)ny);
 	if (gcd > 1) {
 		nfactors = get_prime_factors(gcd, factors);
-		nxg = nx/gcd;
-		nyg = ny/gcd;
+		nxg = (GMT_LONG)(nx/gcd);
+		nyg = (GMT_LONG)(ny/gcd);
 		if (nxg < 3 || nyg < 3) {
 			factor = factors[nfactors - 1];
 			nfactors--;
@@ -1795,7 +1801,7 @@ double guess_surface_time (int factors[], int nx, int ny)
 	return(t_sum);
 }
 
-void suggest_sizes_for_surface (int factors[], int nx, int ny)
+void suggest_sizes_for_surface (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny)
 {
 	/* Calls guess_surface_time for a variety of trial grid
 	 * sizes, where the trials are highly composite numbers
@@ -1809,14 +1815,14 @@ void suggest_sizes_for_surface (int factors[], int nx, int ny)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
-	double	guess_surface_time(int factors[], int nx, int ny);
+	double	guess_surface_time(GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny);
 	double	users_time;	/* Time for user's nx, ny  */
 	double	current_time;	/* Time for current nxg, nyg  */
-	int	i;
-	int	nxg, nyg;	/* Guessed by this routine  */
-	int	nx2, ny2, nx3, ny3, nx5, ny5;	/* For powers  */
-	int	xstop, ystop;	/* Set to 2*nx, 2*ny  */
-	int	n_sug = 0;	/* N of suggestions found  */
+	GMT_LONG	i;
+	GMT_LONG	nxg, nyg;	/* Guessed by this routine  */
+	GMT_LONG	nx2, ny2, nx3, ny3, nx5, ny5;	/* For powers  */
+	GMT_LONG	xstop, ystop;	/* Set to 2*nx, 2*ny  */
+	GMT_LONG	n_sug = 0;	/* N of suggestions found  */
 	int	compare_sugs(const void *point_1, const void *point_2);	/* Sort suggestions decreasing  */
 	struct SURFACE_SUGGESTION *sug = NULL;
 
@@ -1855,7 +1861,7 @@ void suggest_sizes_for_surface (int factors[], int nx, int ny)
 	if (n_sug) {
 		qsort((void *)sug, (size_t)n_sug, sizeof(struct SURFACE_SUGGESTION), compare_sugs);
 		for (i = 0; i < n_sug && i < 10; i++) {
-			fprintf (stderr, "%s:  HINT:  Choosing nx = %d, ny = %d might cut run time by a factor of %.8g\n",
+			fprintf (stderr, "%s:  HINT:  Choosing nx = %ld, ny = %ld might cut run time by a factor of %.8g\n",
 				GMT_program, sug[i].nx, sug[i].ny, sug[i].factor);
 		}
 		GMT_free ((void *)sug);
@@ -1877,7 +1883,7 @@ int compare_sugs (const void *point_1, const void *point_2)
 		return(0);
 }
 
-int get_prime_factors (GMT_LONG n, int *f)
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1906,7 +1912,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	GMT_LONG	current_factor;	/* The factor currently being tried  */
 	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
-	int	n_factors = 0;	/* Returned; one if n is prime  */
+	GMT_LONG	n_factors = 0;	/* Returned; one if n is prime  */
 	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
 	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
 	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
@@ -1914,19 +1920,15 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 
 	/* Initialize m and max_factor  */
-#ifdef __LP64__
-	m = labs(n);
-#else
-	m = abs(n);
-#endif
+	m = GMT_abs(n);
 	if (m < 2) return(0);
-	max_factor = (int)floor(sqrt((double)m));
+	max_factor = (GMT_LONG)floor(sqrt((double)m));
 
 	/* First find the 2s  */
 	current_factor = 2;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1935,7 +1937,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 	current_factor = 3;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1944,7 +1946,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 	current_factor = 5;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1983,7 +1985,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 		while(!(m%current_factor)) {
 			m /= current_factor;
-			f[n_factors] = current_factor;
+			f[n_factors] = (GMT_LONG)current_factor;
 			n_factors++;
 		}
 	}
@@ -1992,7 +1994,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	if (m > 1) {
 		/* m is an additional prime factor of n  */
-		f[n_factors] = m;
+		f[n_factors] = (GMT_LONG)m;
 		n_factors++;
 	}
 	return (n_factors);
@@ -2012,7 +2014,7 @@ GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b)
 	 *
 	 * Walter H. F. Smith, 25 Feb 1992, after D. E. Knuth, vol. II  */
 
-	int	u,v,r;
+	GMT_LONG u,v,r;
 
 	u = MAX(IABS(a), IABS(b));
 	v = MIN(IABS(a), IABS(b));
@@ -2049,10 +2051,10 @@ void load_parameters (struct SURFACE_INFO *C, struct SURFACE_CTRL *Ctrl)
 	C->converge_limit = Ctrl->C.value;
 }
 
-void *New_Surface_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_surface_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SURFACE_CTRL *C;
 	
-	C = (struct SURFACE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SURFACE_CTRL), "New_Surface_Ctrl");
+	C = (struct SURFACE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SURFACE_CTRL), "New_surface_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.value = 250;
@@ -2062,7 +2064,7 @@ void *New_Surface_Ctrl () {	/* Allocate and initialize a new control structure *
 	return ((void *)C);
 }
 
-void Free_Surface_Ctrl (struct SURFACE_CTRL *C) {	/* Deallocate control structure */
+void Free_surface_Ctrl (struct SURFACE_CTRL *C) {	/* Deallocate control structure */
 	if (C->G.file) free ((void *)C->G.file);	
 	if (C->L.low) free ((void *)C->L.low);	
 	if (C->L.high) free ((void *)C->L.high);	
