@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: trend2d.c,v 1.27 2006/04/13 08:34:41 pwessel Exp $
+ *	$Id: trend2d.c,v 1.50 2010/05/10 23:12:32 remko Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -71,24 +71,24 @@
 
 struct TREND2D_CTRL {
 	struct C {	/* -C<condition_#> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} C;
 	struct F {	/* -F<xymrw> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char col[TREND2D_N_OUTPUT_CHOICES];	/* Character codes for desired output in the right order */
 	} F;
 	struct I {	/* -I[<confidence>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} I;
 	struct N {	/* -N<n_model>[r] */
-		BOOLEAN active;
-		BOOLEAN robust;
-		int value;
+		GMT_LONG active;
+		GMT_LONG robust;
+		GMT_LONG value;
 	} N;
 	struct W {	/* -W */
-		BOOLEAN active;
+		GMT_LONG active;
 	} W;
 };
 
@@ -103,41 +103,40 @@ struct	TREND2D_DATA {
 
 int main(int argc, char **argv)
 {
-	int	i, j, k, n_data, n_outputs, n_model, n_model_max, np, significant, rank, n_req;
+	GMT_LONG	i, j, k, n_outputs, n_model, significant, rank, n_req, np;
+	GMT_LONG	n_data;
 
-	BOOLEAN	error = FALSE, weighted_output = FALSE;
+	GMT_LONG	error = FALSE, weighted_output = FALSE;
 
 	double	*gtg, *v, *gtd, *lambda, *workb, *workz, *c_model, *o_model, *w_model, *work;	/* Arrays  */
 	double	xmin, xmax, ymin, ymax, c_chisq, o_chisq, w_chisq, scale = 1.0, prob;
-	double	get_chisq(struct TREND2D_DATA *data, int n_data, int n_model);
+	double	get_chisq(struct TREND2D_DATA *data, GMT_LONG n_data, GMT_LONG n_model);
 
-	char	output_choice[TREND2D_N_OUTPUT_CHOICES], format[BUFSIZ];
+	char format[BUFSIZ];
 
 	FILE	*fp = NULL;
 
 	struct	TREND2D_DATA *data;
 	struct TREND2D_CTRL *Ctrl;
 
-	void read_data(struct TREND2D_DATA **data, int *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp);
-	void write_output(struct TREND2D_DATA *data, int n_data, char *output_choice, int n_outputs);
-	void transform_x(struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax);
-	void untransform_x(struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax);
-	void recompute_weights(struct TREND2D_DATA *data, int n_data, double *work, double *scale);
-	void allocate_array_space(int np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
+	void read_data(struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, GMT_LONG weighted_input, double **work, FILE *fp);
+	void write_output(struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, GMT_LONG n_outputs);
+	void transform_x(struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax);
+	void untransform_x(struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax);
+	void recompute_weights(struct TREND2D_DATA *data, GMT_LONG n_data, double *work, double *scale);
+	void allocate_array_space(GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
 	void free_the_memory(double *gtg, double *v, double *gtd, double *lambda, double *workb, double *workz, double *c_model, double *o_model, double *w_model, struct TREND2D_DATA *data, double *work);
-	void calc_m_and_r(struct TREND2D_DATA *data, int n_data, double *model, int n_model, double *grow);
-	void move_model_a_to_b(double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b);
-	void load_gtg_and_gtd(struct TREND2D_DATA *data, int n_data, double *gtg, double *gtd, double *grow, int n_model, int mp);
-	void solve_system(double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir);
-	void *New_Trend2d_Ctrl (), Free_Trend2d_Ctrl (struct TREND2D_CTRL *C);
+	void calc_m_and_r(struct TREND2D_DATA *data, GMT_LONG n_data, double *model, GMT_LONG n_model, double *grow);
+	void move_model_a_to_b(double *model_a, double *model_b, GMT_LONG n_model, double *chisq_a, double *chisq_b);
+	void load_gtg_and_gtd(struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, GMT_LONG n_model, GMT_LONG mp);
+	void solve_system(double *gtg, double *gtd, double *model, GMT_LONG n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, GMT_LONG *ir);
+	void *New_trend2d_Ctrl (), Free_trend2d_Ctrl (struct TREND2D_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct TREND2D_CTRL *)New_Trend2d_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct TREND2D_CTRL *)New_trend2d_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	n_outputs = 0;
-	n_model_max = 0;
-	for (i = 0; i < TREND2D_N_OUTPUT_CHOICES; i++) output_choice[i] = 0;
 	sprintf (format, "%s\t", gmtdefs.d_format);
 
 	for (i = 1; i < argc; i++) {
@@ -151,7 +150,7 @@ int main(int argc, char **argv)
 				case 'b':
 				case 'f':
 				case '\0':
-					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
+					error += (GMT_LONG)GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
 
 				/* Supplemental parameters */
@@ -243,14 +242,6 @@ int main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option.  Give 0 < confidence level < 1.0\n", GMT_program);
 		error++;
 	}
-	if (n_outputs == 0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Must specify at least one output columns \n", GMT_program);
-		error++;
-	}
-	if (n_outputs > TREND2D_N_OUTPUT_CHOICES) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Too many output columns specified (%d)\n", GMT_program, n_outputs);
-		error++;
-	}
 	if (Ctrl->N.value <= 0 || Ctrl->N.value > 10) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option.  Must request 1-10 parameters\n", GMT_program);
 		error++;
@@ -262,12 +253,12 @@ int main(int argc, char **argv)
 	n_req = (Ctrl->W.active) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_req) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 
 	for (k = n_outputs = 0; k < TREND2D_N_OUTPUT_CHOICES && Ctrl->F.col[k]; k++) {
-		if (!strchr ("xymrw", Ctrl->F.col[k])) {
+		if (!strchr ("xyzmrw", Ctrl->F.col[k])) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Unrecognized output choice %c\n", GMT_program, Ctrl->F.col[k]);
 			error++;
 		}
@@ -277,16 +268,18 @@ int main(int argc, char **argv)
 		n_outputs++;
 	}
         if (n_outputs == 0) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Must specify at least one output columns \n", GMT_program);
+                fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Must specify at least one output column\n", GMT_program);
                 error++;
         }
+	if (n_outputs > TREND2D_N_OUTPUT_CHOICES) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Too many output columns specified (%ld)\n", GMT_program, n_outputs);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -294,7 +287,7 @@ int main(int argc, char **argv)
 #endif
 	project_info.w = 0;	project_info.e = 360.0;	/* For -L not to cause trouble in GMT_input */
 	np = Ctrl->N.value;	/* Row dimension for matrices gtg and v  */
-	allocate_array_space(np, &gtg, &v, &gtd, &lambda, &workb, &workz, &c_model, &o_model, &w_model);
+	allocate_array_space((GMT_LONG)np, &gtg, &v, &gtd, &lambda, &workb, &workz, &c_model, &o_model, &w_model);
 
 	read_data(&data, &n_data, &xmin, &xmax, &ymin, &ymax, Ctrl->W.active, &work, fp);
 
@@ -313,11 +306,11 @@ int main(int argc, char **argv)
 	transform_x(data, n_data, xmin, xmax, ymin, ymax);	/* Set domain to [-1, 1] or [-pi, pi]  */
 
 	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s:  Read %d data with X values from %.8g to %.8g\n", GMT_program, n_data, xmin, xmax);
+		fprintf(stderr,"%s:  Read %ld data with X values from %.8g to %.8g\n", GMT_program, n_data, xmin, xmax);
 		fprintf(stderr,"N_model\tRank\tChi_Squared\tSignificance\n");
 	}
 
-	sprintf (format, "%%d\t%%d\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
+	sprintf (format, "%%ld\t%%ld\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
 
 	if (Ctrl->I.active) {
 		n_model = 1;
@@ -429,22 +422,23 @@ int main(int argc, char **argv)
 
 	untransform_x(data, n_data, xmin, xmax, ymin, ymax);
 
-	write_output(data, n_data, output_choice, n_outputs);
+	write_output(data, n_data, Ctrl->F.col, n_outputs);
 
 	free_the_memory(gtg, v, gtd, lambda, workb, workz, c_model, o_model, w_model, data, work);
 
-	Free_Trend2d_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_trend2d_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp)
+void read_data (struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, GMT_LONG weighted_input, double **work, FILE *fp)
 {
 
-	int	i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
-	char	buffer[BUFSIZ];
+	GMT_LONG	n_read, i, n_alloc = GMT_CHUNK;
+	GMT_LONG n_expected_fields, n_fields;
+	char	buffer[BUFSIZ], *not_used = NULL;
 	double	*in;
 
 	if (fp == NULL) {
@@ -455,20 +449,21 @@ void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *x
 	}
 	(*data) = (struct TREND2D_DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct TREND2D_DATA), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
-	i = 0;
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp);
+	i = n_read = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted_input;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
+		n_read++;
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, i);
-			exit (EXIT_FAILURE);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+			continue;
 		}
 
-		(*data)[i].x = in[0];
-		(*data)[i].y = in[1];
-		(*data)[i].z = in[2];
+		(*data)[i].x = in[GMT_X];
+		(*data)[i].y = in[GMT_Y];
+		(*data)[i].z = in[GMT_Z];
 		(*data)[i].w = (weighted_input) ? in[3] : 1.0;
 
 		if (i) {
@@ -486,9 +481,14 @@ void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *x
 		i++;
 
 		if (i == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			*data = (struct TREND2D_DATA *) GMT_memory ((void *)*data, (size_t)n_alloc, sizeof(struct TREND2D_DATA), GMT_program);
 		}
+		if (i == INT_MAX) {
+			fprintf (stderr, "%s: ERROR: Cannot process more than %d data points\n", GMT_program, INT_MAX);
+			GMT_free ((void *)data);
+			exit (EXIT_FAILURE);
+		}
 	}
 	if (fp != GMT_stdin) GMT_fclose(fp);
 	*data = (struct TREND2D_DATA *) GMT_memory ((void *)*data, (size_t)i, sizeof(struct TREND2D_DATA), GMT_program);
@@ -496,7 +496,7 @@ void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *x
 	*n_data = i;
 }
 
-void allocate_array_space (int np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model)
+void allocate_array_space (GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model)
 {
 	*gtg = (double *) GMT_memory (VNULL, (size_t)(np*np), sizeof(double), GMT_program);
 	*v = (double *) GMT_memory (VNULL, (size_t)(np*np), sizeof(double), GMT_program);
@@ -509,9 +509,10 @@ void allocate_array_space (int np, double **gtg, double **v, double **gtd, doubl
 	*w_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 }
 
-void write_output (struct TREND2D_DATA *data, int n_data, char *output_choice, int n_outputs)
+void write_output (struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, GMT_LONG n_outputs)
 {
-	int	i, j;
+	GMT_LONG	i;
+	GMT_LONG j;
 	double out[6];
 
 	for (i = 0; i < n_data; i++) {
@@ -556,9 +557,9 @@ void free_the_memory (double *gtg, double *v, double *gtd, double *lambda, doubl
 	GMT_free ((void *)gtg);
 }
 
-void transform_x (struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax)
+void transform_x (struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax)
 {
-	int	i;
+	GMT_LONG	i;
 	double	offsetx, scalex;
 	double	offsety, scaley;
 
@@ -573,9 +574,9 @@ void transform_x (struct TREND2D_DATA *data, int n_data, double xmin, double xma
 	}
 }
 
-void untransform_x (struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax)
+void untransform_x (struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax)
 {
-	int	i;
+	GMT_LONG	i;
 	double	offsetx, scalex;
 	double	offsety, scaley;
 
@@ -590,9 +591,9 @@ void untransform_x (struct TREND2D_DATA *data, int n_data, double xmin, double x
 	}
 }
 
-double get_chisq (struct TREND2D_DATA *data, int n_data, int n_model)
+double get_chisq (struct TREND2D_DATA *data, GMT_LONG n_data, GMT_LONG n_model)
 {
-	int	i, nu;
+	GMT_LONG	i, nu;
 	double	chi = 0.0;
 
 
@@ -609,9 +610,9 @@ double get_chisq (struct TREND2D_DATA *data, int n_data, int n_model)
 	return(chi);
 }
 
-void recompute_weights (struct TREND2D_DATA *data, int n_data, double *work, double *scale)
+void recompute_weights (struct TREND2D_DATA *data, GMT_LONG n_data, double *work, double *scale)
 {
-	int	i;
+	GMT_LONG	i;
 	double	k, ksq, rr;
 
 	/* First find median { fabs(data[].r) },
@@ -644,7 +645,7 @@ void recompute_weights (struct TREND2D_DATA *data, int n_data, double *work, dou
 	}
 }
 
-void load_g_row (double x, double y, int n, double *gr)
+void load_g_row (double x, double y, GMT_LONG n, double *gr)
       	     	/* Current data position, appropriately normalized.  */
    	  	/* Number of model parameters, and elements of gr[]  */
       	     	/* Elements of row of G matrix.  */
@@ -653,7 +654,7 @@ void load_g_row (double x, double y, int n, double *gr)
 		G matrix (Menke notation), where x,y is the ith datum's
 		location.  */
 
-	int	j;
+	GMT_LONG	j;
 
 	j = 0;
 	while (j < n) {
@@ -693,12 +694,12 @@ void load_g_row (double x, double y, int n, double *gr)
 	}
 }
 
-void calc_m_and_r (struct TREND2D_DATA *data, int n_data, double *model, int n_model, double *grow)
+void calc_m_and_r (struct TREND2D_DATA *data, GMT_LONG n_data, double *model, GMT_LONG n_model, double *grow)
 {
 	/*	model[n_model] holds solved coefficients of m_type model.
 		grow[n_model] is a vector for a row of G matrix.  */
 
-	int	i, j;
+	GMT_LONG	i, j;
 	for (i = 0; i < n_data; i++) {
 		load_g_row(data[i].x, data[i].y, n_model, grow);
 		data[i].m = 0.0;
@@ -709,22 +710,23 @@ void calc_m_and_r (struct TREND2D_DATA *data, int n_data, double *model, int n_m
 	}
 }
 
-void move_model_a_to_b (double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b)
+void move_model_a_to_b (double *model_a, double *model_b, GMT_LONG n_model, double *chisq_a, double *chisq_b)
 {
-	int	i;
+	GMT_LONG	i;
 	for(i = 0; i<  n_model; i++) {
 		model_b[i] = model_a[i];
 	}
 	*chisq_b = *chisq_a;
 }
 
-void load_gtg_and_gtd (struct TREND2D_DATA *data, int n_data, double *gtg, double *gtd, double *grow, int n_model, int mp)
+void load_gtg_and_gtd (struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, GMT_LONG n_model, GMT_LONG mp)
       	    	      
       			  
    			    	/* mp is row dimension of gtg  */
 {
 
-	int	i, j, k;
+	GMT_LONG	i;
+	GMT_LONG j, k;
 	double	wz;
 
 	/* First zero the contents for summing:  */
@@ -759,10 +761,11 @@ void load_gtg_and_gtd (struct TREND2D_DATA *data, int n_data, double *gtg, doubl
 	}
 }
 
-void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir)
+void solve_system (double *gtg, double *gtd, double *model, GMT_LONG n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, GMT_LONG *ir)
 {
 
-	int	i, j, k, rank = 0, n, m, nrots;
+	GMT_LONG	i, j, k, rank = 0, nrots;
+	GMT_LONG n, m;
 	double	c_test, temp_inverse_ij;
 
 	if (n_model == 1) {
@@ -791,10 +794,10 @@ void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp,
 	}
 }
 
-void *New_Trend2d_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_trend2d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct TREND2D_CTRL *C;
 	
-	C = (struct TREND2D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct TREND2D_CTRL), "New_Trend2d_Ctrl");
+	C = (struct TREND2D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct TREND2D_CTRL), "New_trend2d_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->C.value = 1.0e06;		/* Condition number for matrix solution  */	
@@ -802,6 +805,6 @@ void *New_Trend2d_Ctrl () {	/* Allocate and initialize a new control structure *
 	return ((void *)C);
 }
 
-void Free_Trend2d_Ctrl (struct TREND2D_CTRL *C) {	/* Deallocate control structure */
+void Free_trend2d_Ctrl (struct TREND2D_CTRL *C) {	/* Deallocate control structure */
 	GMT_free ((void *)C);	
 }
